/*
 * -- cl_info.cpp -- 
 * 
 * Zeigt die GNU-Lizenz und die Hilfe zu makergaulxx an
 * 
 * File:   cl_info.cpp
 * Copyright 2013 Christoph Bultmann
 * 
 * This file is part of makergaulxx.
 *
 * makergaulxx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * makergaulxx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with makergaulxx.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "cl_info.h"
#include <gmp.h>
#include <mpfr.h>
//Konstruktor
cl_info::cl_info()
    {
    }


//Anzeige der GPL
void cl_info::show_license()
    {
        cout << "\n";
        cout << "*---------------------- makergaulxx "<<MAKERGAULXX_VERSIONSTRING<<" license -----------------------*\n";
        cout << "|                                                                        |\n";
        cout << "|  Copyright 2013 Christoph Bultmann                                     |\n";
        cout << "|                                                                        |\n";
        cout << "|  makergaulxx is free software: you can redistribute it and/or modify   |\n";
        cout << "|  it under the terms of the GNU General Public License as published by  |\n";
        cout << "|  the Free Software Foundation, either version 3 of the License, or     |\n";
        cout << "|  (at your option) any later version.                                   |\n";
        cout << "|                                                                        |\n";
        cout << "|  makergaulxx is distributed in the hope that it will be useful,        |\n";
        cout << "|  but WITHOUT ANY WARRANTY; without even the implied warranty of        |\n";
        cout << "|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |\n";
        cout << "|  GNU General Public License for more details.                          |\n";
        cout << "|                                                                        |\n";
        cout << "|  You should have received a copy of the GNU General Public License     |\n";
        cout << "|  along with makergaulxx.  If not, see <http://www.gnu.org/licenses/>.  |\n";
        cout << "*------------------------------------------------------------------------*\n";
    }

//Zeigt die Versionen der Programteile und die wichtigsten Einstellungen, mit denen diese kompiliert wurden
void cl_info::show_version()                                                    
    {
        cout << "\n";
        cout << "*--- makergaulxx "<<MAKERGAULXX_VERSIONSTRING<<" info ---*\n\n";
        cout << "makergaulxx:\n";
        cout << "   version "<<MAKERGAULXX_VERSIONSTRING<<"."<<"\n";
        cout << "   built for architecture: 32 Bit\n";
        cout << "   built for cpu-family: pentiumpro and higher\n\n";
        cout << "GNU Multi Precision Library (GMP):\n";
        cout << "   version "<<__GNU_MP_VERSION<<"."<<__GNU_MP_VERSION_MINOR<<"."<<__GNU_MP_VERSION_PATCHLEVEL<<"."<<"\n";
        cout << "   built for architecture: 32 Bit\n";
        cout << "   built for cpu-family: pentiumpro and higher\n\n";
        cout << "GNU Multiple Precision Floating-Point Reliable Library (MPFR):\n";
        cout << "   version "<<MPFR_VERSION_STRING<<"."<<"\n";
        cout << "   built for architecture: 32 Bit\n";
        cout << "   built for cpu-family: pentiumpro and higher\n\n";
    }

//Anzeige der Hilfe
int cl_info::show_help()
    {
        char run = 'q';
        cout << "\n";
        cout << "*--- makergaulxx "<<MAKERGAULXX_VERSIONSTRING<<  " help ---*\n\n";
        cout << "Starting parameters:\n\n";
        cout << "   -h                          Help. Here you are.\n";
        cout << "   -l                          Show program license (GPL 3).\n";
        cout << "   -run $par1 $par2 ...        Run analysis with needed parameters.\n";
        cout << "   -v                          Version and build information.\n\n";
        cout << "For getting more information about \"-run\" enter \"r\", for leaving\n";
        cout << "press any other key. \n";
        
        cin >> run ;                                                                                                    //Eingabebuchstabe speichern
        
        if(cin.fail()) {return(0);}                                                                                     //Irgendwas längeres eingegeben? Abbrechen!
        if(run != 'r') {return(0);}                                                                                     //Irgendwas anderes eingegeben? Abbrechen!
        
        cout << "\n";
        cout << "*--- makergaulxx "<<MAKERGAULXX_VERSIONSTRING<<  " \"-run\" parameters ---*\n\n";
        cout << "Note: $par1 to $parX are placeholders you have to substitute by\n";
        cout << "the described values of your program / measurement data.\n\n\n";
        cout << "$par1                  Number of paramerters the selcted modeling\n";
        cout << "                       algorithm will return includung the fitness \n";
        cout << "                       value. Must be > 2 because of\n";
        cout << "                       returning-par[0] = fitness-value and \n";
        cout << "                       returning-par[1] = DNA are required in every case.\n";
        cout << "$par2                  Number of included fluorescence values.\n";
        cout << "$par3                  Number of additional fix parameters. -1 if not used.\n";
        cout << "$par4                  Downhill-Simplex tryouts (Loops).\n";
        cout << "$par5                  Reflexion factor.\n";
        cout << "$par6                  Expansion factor.\n";
        cout << "$par7                  Contraction factor.\n";
        cout << "$par8                  Test value for fitness at control cycle a.\n";
        cout << "$par9                  Test value for fitness at control cycle b.\n";
        cout << "$par10                 Test value for fitness at control cycle c.\n";
        cout << "$par11                 Target value for fitness at the last cycle.\n";
        cout << "$par12                 (Not used, but required. Takes any Integer.)\n\n";
        cout << "$par13                 Chosen modeling algorythm.\n";
        cout << "                       1 = MAKERGAUL-Analysis ($par1 must be 5)\n";
        cout << "                       2 = MAK2-Analysis ($par1 must be 4)\n\n\n";
        cout << "                       3 = MAKERGAUL-Analysis ($par1 must be 4)\n\n\n";
        cout << "$par14                     Limits for the returned results. Beginning\n";
        cout << "   to                      with $par13, the uper and the lower limit\n";
        cout << "$par[14 + 2 * ($par1-1)]   for every returning-par are set alternating,\n";
        cout << "                           excluding returning-par[0] = fitness-value.\n\n\n";
        cout << "$par[14 + 2 * ($par1-1)]   \n";
        cout << "   to                              Measured values \n";
        cout << "$par[(14 + 2 * ($par1-1)) + $par2]\n\n";
        cout << "$par[(14 + 2 * ($par1-1)) + $par2]\n";
        cout << "   to                              Additional fix parameters, if $par3 >= 1.\n";
        cout << "$par[last]\n\n\n";
        cout << "At the end, the sum of all $par must be ($par1 - 1) * 2 + $par2 + 14 + $par3(if given) \n";
        cout << "or the program will fail! Enter any key to quit.\n";
        cin >> run ;
        return(0);
    }
//Destructor
cl_info::~cl_info()
    {
    }

