/* 
 * -- cl_points.cpp --
 * 
 * Stellt die Methoden eines Punktobjekts für die Downhill-Simplex-Analyse bereit
 * 
 * File:   cl_points.cpp
 * Copyright 2013 Christoph Bultmann
 * 
 * This file is part of makergaulxx.
 *
 * makergaulxx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * makergaulxx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with makergaulxx.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "cl_points.h"

void (cl_points::*ptr_fitness) (vector< mpf_class >&);                                                                  //FUNKTIONSZEIGER Speichert die Adresse der ausgewählten Fitness-Funktion. Lässt sich nicht ohne Probleme in Header definieren!
int (cl_points::*ptr_seed_p_range) (int);                                                                               //FUNKTIONSZEIGER Speichert die Adresse der ausgewählten Seed-Funktion. Lässt sich nicht ohne Probleme in Header definieren!
void (cl_points::*ptr_seed_wrong) (int);                                                                                //FUNKTIONSZEIGER Speichert die Adresse der ausgewählten Seed-Funktion. Lässt sich nicht ohne Probleme in Header definieren!
int (cl_points::*ptr_set_fix_value) (int,string);                                                                       //FUNKTIONSZEIGER Speichert die Adresse der ausgewählten Set-Fix-Funktion. Lässt sich nicht ohne Probleme in Header definieren!
//Leerer Konstruktor möglich (Vorsicht!)
cl_points::cl_points()
    {
    
    }
//Konstruktor der Punktewolke
cl_points::cl_points(int par_pts,int par_res ,int par_m_values, int par_alg) 
    {
        init(par_pts, par_res, par_m_values, par_alg);
    }

//Die eigentliche Initialisierungsfunktion
void cl_points::init(int par_pts,int par_res ,int par_m_values, int par_alg)
    {
        time_t unixtime = time(0);                                                                                      //UNIX-time für Startwert der Zufallszahlenfunktion benutzen
        srand(unixtime * unixtime);                                                                                     //Startwert der rand()-Funktion setzen
        
        ptr_seed_p_range = &cl_points::seed_p_range_default;                                                            //Per Default: Die Funktion zum korrigiern falscher Werte innerhalb eines Loops 
        ptr_seed_wrong = &cl_points::seed_wrong_weak;                                                                   //Per Default: Die Funktion zum korrigiern falscher Werte innerhalb eines Cycles benutzt weiche Grenzsetzung
        ptr_set_fix_value = &cl_points::set_fix_none;                                                                   //Per Default: Keine Fixwerte akzeptieren
        
        if(par_alg == 1)                                                                                                //MAKERGAUL-Algorythmus gewählt? Die entsprechenden Variablen initialisieren!
            {
                dna = 0; 
                e_usable = 0; 
                m_value_mw = 0; 
                ssre_step = 0; 
                ssre = 0;
            }
        if(par_alg == 2)                                                                                                //MAK2-Algorythmus gewählt? Die entsprechenden Variablen initialisieren!
            {
                ptr_seed_wrong = &cl_points::seed_wrong_hard;                                                           //MAK2 benötigt harte Grenzsetzung
                sse_step = 0; 
                sse = 0;
                mpf_init(mpf_temp);
                mpfr_init(mpfr_dna);                                                                                            
                mpfr_init(mpfr_const); 
                mpfr_init(mpfr_basis);  
                mpfr_init(mpfr_logres);                                                                                 //Zwischenspeicher für die Logarithmusberechnung (Grund für die Verwendung der MPFR-Bibliothek!)
                mpfr_init(mpfr_addtemp); 
            }
          if(par_alg == 3)                                                                                              //MAKERGAUL-C-Algorythmus gewählt? Die entsprechenden Variablen initialisieren!
            {
                ptr_set_fix_value = &cl_points::set_fix_makergaul_c;                                                    //Als Fixwert den Factor_inhibition = Inhibitionskonstante = f_inhib akzeptieren
                dna = 0; 
                e_usable = 0; 
                m_value_mw = 0; 
                ssre_step = 0; 
                ssre = 0;
                f_inhib = 0;
            }
        
        set_number_of(par_pts, par_res, par_m_values); 
    }

//Anzahl der Punkte und der Parameter festlegen
void cl_points::set_number_of(int par_pts = 0,int par_res = 0,int par_m_values=0) 
    {
        
        a_loop = 0;
        l_imp_loop = 0;
        p_range.resize(par_pts);                                                                                        //Rangpunkte Ebene 1: Anzahl Rangpunkte
        h_range.resize(par_pts);                                                                                        //Hilfsarray Sortierung Ebene 1: Anzahl Rangpunkte
        p_result.resize(par_res);                                                                                       //Ergebnispunkt: Anzahl Ergebniss-Parameter
        p_center.resize(par_res);                                                                                       //Zentrumspunkt: Anzahl Ergebniss-Parameter
        p_reflexion.resize(par_res);                                                                                    //Reflexionspunkt: Anzahl Ergebniss-Parameter
        p_expansion.resize(par_res);                                                                                    //Expansionspunkt: Anzahl Ergebniss-Parameter
        p_contraction.resize(par_res);                                                                                  //Kontraktionspunkt: Anzahl Ergebniss-Parameter
        p_borders.resize(par_res);                                                                                      //Grenzwerte Ebene 1: Anzahl Ergebniss-Parameter 
        f_itv_borders.resize(par_res);                                                                                  //Intervall der Grenzwerte: Anzahl Ergebniss-Parameter 
        i_itv_borders.resize(par_res);                                                                                  //Integer-Intervall der Grenzwerte: Anzahl Ergebniss-Parameter 
        m_values.resize(par_m_values);                                                                                  //Messwerte: Anzahl der Messwerte
        c_values.resize(par_m_values);                                                                                  //Berechnete Modellwerte: Anzahl der Messwerte
        //Erste Initialisierung mit Default-Wert(1)
        for(int i = 0; i < par_res; i++) 
                   {   
                       p_result.at(i) = 10000;
                       p_center.at(i) = 1;                                              
                       p_reflexion.at(i) = 1;
                       p_expansion.at(i) = 1;
                       p_contraction.at(i) = 1; 
                       p_borders.at(i).resize(2);                                                                       //Grenzwerte Ebene 2: Oberer und unterer Grenzwert 
                       p_borders.at(i).at(0) = -1;
                       p_borders.at(i).at(1) = 1;
                       f_itv_borders.at(i) = 2;                                                                                 
                       i_itv_borders.at(i) = 1;
                   }
        for(int i = 0; i < par_pts;i++) 
            {
               p_range.at(i).resize(par_res);                                                                           //Rangpunkte Ebene 2: Anzahl Ergebniss-Parameter
               h_range.at(i).resize(par_res);                                                                           //Hilfsarray Ebene 2: Anzahl Ergebniss-Parameter
               //Alle Ergebnissparameter mit Default-Wert(1) initialisieren                                   
               for(int a = 0; a< par_res;a++) 
                   {
                       p_range.at(i).at(a) = 1;  
                       h_range.at(i).at(a) = 1;                                                                         //Hilfsarrays auch initialisieren
                   }
            }
        //Erste Initialisierung der Messwerte und berechneten Werte mit Default-Wert(1)
        for(int i = 0; i < par_m_values; i++) 
                   {   
                        m_values.at(i) = 0;  
                        c_values.at(i) = 0;
                   }
    }

//Festlegen der Fitnessfunktion und abgleich, ob genügend Ergebnissparameter vorgegeben werden    
bool cl_points::set_analysis(int par_alg, int par_res)
    {
        if(par_alg == 1 && par_res == 5)
            {   
                ptr_fitness = &cl_points::makergaul;                                                                    //...Speicheradresse der MAKERGAUL-Funktion in Funktionszeiger schreiben
                return(true);
            }
        if(par_alg == 2 && par_res == 4)                                                                            
            {   
                ptr_fitness = &cl_points::mak2;                                                                         //...Speicheradresse der MAK2-Funktion in Funktionszeiger schreiben
                return(true);
            }
        if(par_alg == 3 && par_res == 4)
            {   
                ptr_fitness = &cl_points::makergaul_c;                                                                  //...Speicheradresse der MAKERGAUL_C-Funktion in Funktionszeiger schreiben
                return(true);
            }
        return(false);
    }

//Messwerte in Array schreiben(Eingabetyp mpf_class)
int cl_points::set_m_value(int par_num = 0, mpf_class par_value = 0) 
    {
        //mp_exp_t ex;
        try{cl_points::m_values.at(par_num) = par_value;}                                                               //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#m_value(" <<par_num <<") dosn't exist!;";
                return(1);
            }
        //cout << "MEASURED VALUE"<< par_num << "#" << m_values.at(par_num).get_str(ex) << "E" << ex << ";"; 
        return(0);
    }
//Messwerte in Array schreiben(Eingabetyp String)
int cl_points::set_m_value(int par_num = 0, string par_value = "0") 
    {
        mpf_class m(0);                                                
        m = par_value;                                                                                                  //Typumwandlung durch Klasse MPF
        //mp_exp_t ex;
        try{cl_points::m_values.at(par_num) = m;}                                                                       //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#m_value(" <<par_num <<") dosn't exist!;";
                return(1);
            }
        //cout << "MEASURED VALUE"<< par_num << "#" << m_values.at(par_num).get_str(ex) << "E" << ex << ";"; 
        return(0);
    }

//Berechnete Modellwerte in Array schreiben(Eingabetyp mpf_class)
int cl_points::set_c_value(int par_num = 0, mpf_class par_value = 0) 
    {
        try{cl_points::c_values.at(par_num) = par_value;}                                                               //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#c_value(" <<par_num <<") dosn't exist!;";
                return(1);
            }
        return(0);
    }
//Berechnete Modellwerte in Array schreiben(Eingabetyp String)
int cl_points::set_c_value(int par_num = 0, string par_value = "0") 
    {
        mpf_class m(0);                                                
        m = par_value;                                                                                                  //Typumwandlung durch Klasse MPF
        try{cl_points::c_values.at(par_num) = m;}                                                                       //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#c_value(" <<par_num <<") dosn't exist!;";
                return(1);
            }
        return(0);
    }

//Berechnete Fixwerte in Array schreiben (Hüllfunktion für Funktionszeiger, Eingabetyp String)
int cl_points::set_fix_value(int par_num = 0, string par_value = "0") 
    {
        (this->*ptr_set_fix_value)(par_num,par_value);
        return(0);
    }

//Berechnete Fixwerte in Array schreiben (Nullfunktion = keine Fixwerte akzeptiert, Eingabetyp String)
int cl_points::set_fix_none(int par_num = 0, string par_value = "0") 
    {
        return(0);
    }

//Berechnete Fixwerte in Array schreiben (Nur ersten Fixwert akzeptiren als Faktor_inhibition, Eingabetyp String)
int cl_points::set_fix_makergaul_c(int par_num = 0, string par_value = "0") 
    {
        if(par_num == 0) {f_inhib = par_value;}
        return(0);
    }

//Grenzwert für einen Ergebnissparameter setzen (Eingabetyp mpf_class)
int cl_points::set_border(int par_res = 0, mpf_class par_border_down = 0, mpf_class par_border_up = 0) 
    {
        mpz_class h_down(0);
        try{cl_points::p_borders.at(par_res).at(0) = par_border_down;}                                                  //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#p_borders(" <<par_res << "," <<0 <<") dosn't exist!;";
                return(1);
            }
        try{cl_points::p_borders.at(par_res).at(1) = par_border_up;}                                                    //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#p_borders(" <<par_res << "," <<1 <<") dosn't exist!;";
                return(1);
            }
        f_itv_borders.at(par_res) = p_borders.at(par_res).at(1)-p_borders.at(par_res).at(0);                            //Schritt 1: Intervall zwischen oberer und unterer Grenze bestimmen
        h_down = floor(p_borders.at(par_res).at(1) / p_borders.at(par_res).at(0));                                      //Schritt 2: Variable, die das Intervall als Integer ausdrückt (gerundet)
        while(h_down > 9)                                                                                               //Bis h_down nicht mehr durch zehn teilbar ist...
            {               
                    h_down = h_down/10;                                                                                 //...durch zehn teilen...                                                                           
                    i_itv_borders.at(par_res) = i_itv_borders.at(par_res)*10;                                           //...und i_itv_borders um eine Dezimalstelle erweitern       
            }
        if(i_itv_borders.at(par_res)<1000){i_itv_borders.at(par_res) = 1000;}
        return(0);
    }
//Grenzwert für einen Ergebnissparameter setzen (Eingabetyp string)
int cl_points::set_border(int par_res = 0, string par_border_down = 0, string par_border_up = 0) 
    {
        mpf_class down(1), up(1);                                                                                
        mpz_class h_down(0); 
        down = par_border_down;                                                                                         //String-Umwandlung durch mpf-Klasse durchführen lassen
        up = par_border_up;
        try{cl_points::p_borders.at(par_res).at(0) = down;}                                                             //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#p_borders(" <<par_res << "," <<0 <<") dosn't exist!;";
                return(1);
            }
        try{cl_points::p_borders.at(par_res).at(1) = up;}                                                               //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#p_borders(" <<par_res << "," <<1 <<") dosn't exist!;";
                return(1);
            }
        f_itv_borders.at(par_res) = p_borders.at(par_res).at(1)-p_borders.at(par_res).at(0);                            //Schritt 1: Intervall zwischen oberer und unterer Grenze bestimmen
        h_down = floor(p_borders.at(par_res).at(1) / p_borders.at(par_res).at(0));                                      //Schritt 2: Variable, die das Intervall als Integer ausdrückt (gerundet)
        while(h_down > 9)                                                                                               //Bis h_down nicht mehr durch zehn teilbar ist...
            {               
                    h_down = h_down/10;                                                                                 //...durch zehn teilen...                                                                           
                    i_itv_borders.at(par_res) = i_itv_borders.at(par_res)*10;                                           //...und i_itv_borders um eine Dezimalstelle erweitern       
            }
        if(i_itv_borders.at(par_res)<1000){i_itv_borders.at(par_res) = 1000;}
        return(0);
    }

//Ergebniss-Werte in Punktarray schreiben(Eingabetyp mpf_class)
int cl_points::set_point_value(int par_pts = 0,int par_res = 0, mpf_class par_value = 0) 
    {
        try{cl_points::p_range.at(par_pts).at(par_res) = par_value;}                                                    //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#p_range(" <<par_pts << "," <<par_res <<") dosn't exist!;";
                return(1);
            }
        return(0);
    }
//Ergebniss-Werte in Punktarray schreiben(Eingabetyp string)
int cl_points::set_point_value(int par_pts = 0,int par_res = 0, string par_value = "0") 
    {
        mpf_class m(0);                                                
        m = par_value;                                                                                                  //Typumwandlung durch Klasse MPF
        try{cl_points::p_range.at(par_pts).at(par_res) = m;}                                                            //Zugriff prüfen
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
            {
                cout << "ERR#p_range(" <<par_pts << "," <<par_res <<") dosn't exist!;";
                return(1);
            }
        return(0);
    }

//Die Rangfolge der Punkte feststellen (Standart: sortieren nach dem Ergebnissparameter [x][0], meist SSE )
int cl_points::sort_points() 
    {   
        vector <mpf_class> h_sorting(p_range.size());                                                                   //Hilfsvektor zum Sortieren der Rangpunkte
        for(unsigned int i = 0; i < p_range.size(); i++)                                                                //Ausgelesener Unterarray[i][par_res-1] wird zu Hilfsarray[i]
            {
                try{h_sorting.at(i) = p_range.at(i).at(0);}                                                             //Sortiert wird immer nach Parameter 0 (SSE oder ähnliches)
                catch(out_of_range& fehler)                                                                             //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen
                    {
                        cout << "ERR#p_range(" <<i+1 << "," <<0 <<") dosn't exist!;";
                        return(1);
                    }
                for(unsigned int a = 0; a <p_range.at(i).size(); a++)
                    {
                        h_range.at(i).at(a) = p_range.at(i).at(a);                                                      //Rangpunkt-Array in Zwischen-Array kopieren
                    }
            }
        sort(h_sorting.begin(),h_sorting.end());                                                                        //Den Hilfsarray mit den ausgewählten Ergebnissparametern aus p_range[0][par_res-1] bis p_range[x][par_res-1] sortieren
        for(unsigned int i = 0; i < h_sorting.size(); i++)
            {
                for(unsigned int a = 0; a < h_range.size(); a++)
                    {
                        if(h_sorting.at(i) == h_range.at(a).at(0))                                                      //Wenn Parameter der Sortierliste h_sorting(i) mit dem aus h_range(a)(0) übereinstimmt,...
                            {
                                for(unsigned int x = 0; x < h_range.at(a).size(); x++)
                                    {
                                        p_range.at(i).at(x) = h_range.at(a).at(x);                                      //...wird p_range(i)(x) mit h_range(a)(x) überschrieben
                                    }
                            }                                                        
                    }
            }
        return(0);
    }
void cl_points::build_center() 
    {   
        //mp_exp_t u;
        for(unsigned int i = 1; i < p_center.size(); i++)                                                               //erst mit erstem Ergebnissparameter beginnen. Parameter 0 bleibt hier leer.(Paralellität mit anderen Punkten soll erhalten bleiben)
            { 
                mpf_class sum;                                                                                          //Zentrum besteht aus Mittelwert aller Ergebnissparameter
                sum = 0;                                                                                                //Summe für jeden Ergebnissparameter(i) wieder auf Null setzen)
                
                for(unsigned int a = 0; a < (p_range.size()-1); a++)                                                      //Alle Rangpunkte außer dem Letzten (=schlechtesten) miteinbeziehen
                    {
                        sum = sum+p_range.at(a).at(i);                                                                  //Summe aller Ergebnissparameter(i) bilden
                    }
                p_center.at(i) = sum/(p_range.size()-1);                                                                //Zentrumspunkt(i) = Mittelwert aller Ergebnisswerte(i) außer dem des schlechtesten Punktes
                //cout<<"NUM CENTER PAR "<<i<<"#"<<p_center.at(i).get_str(u)<<"E"<<u<<";";
            }                                                                   
    }


//Reflexionspunkt bestimmen
void cl_points::build_reflexion(mpf_class par_factor)                                                                   //par_factor = Reflexionsfaktor, der vom Benutzer eingestellt werden kann
    {   
        //mp_exp_t u;
        for(unsigned int i = 1; i < p_reflexion.size(); i++)                                                            //erst mit erstem Ergebnissparameter beginnen. Parameter 0 ist für Fitness-Wert reserviert und wird anders berechnet
            {
                p_reflexion.at(i) = p_center.at(i) + par_factor * (p_center.at(i) - p_range.at(p_range.size()-1).at(i));//Reflexionspunkt wird aus Zentrum + (Zentrum - dem schlechtesten Punkt) * Reflexionsfaktor gebildet
                //cout<<"NUM REFLEXION PAR "<<i<<"#"<<p_reflexion.at(i).get_str(u)<<"E"<<u<<";";   
            }
        (this->*ptr_fitness)(p_reflexion);                                                                              //p_reflection[0]: Fitness-Wert (SSRE)
        //cout<<"NUM REFLEXION PAR "<<0<<"#"<<p_reflexion.at(0).get_str(u)<<"E"<<u<<";"; 
                                                                              
    }
//Expansionspunkt bestimmen
void cl_points::build_expansion(mpf_class par_factor)                                                                   //par_factor = Expansionssfaktor, der vom Benutzer eingestellt werden kann
    {   
        //mp_exp_t u;
        for(unsigned int i = 1; i < p_expansion.size(); i++)                                                            //erst mit erstem Ergebnissparameter beginnen. Parameter 0 ist für Fitness-Wert reserviert und wird anders berechnet
            {
                p_expansion.at(i) = p_center.at(i) + par_factor * (p_center.at(i) - p_range.at(p_range.size()-1).at(i));//Expansionssfaktor wird aus Zentrum + (Zentrum - dem schlechtesten Punkt) * Reflexionsfaktor gebildet
                //cout<<"NUM EXPANSION PAR "<<i<<"#"<<p_expansion.at(i).get_str(u)<<"E"<<u<<";";           
            }  
        (this->*ptr_fitness)(p_expansion);                                                                              //p_expansion[0]: Fitness-Wert (SSRE)
        //cout<<"NUM EXPANSION PAR "<<0<<"#"<<p_expansion.at(0).get_str(u)<<"E"<<u<<";";    
    }
//Kontraktionspunkt bestimmen
void cl_points::build_contraction(mpf_class par_factor, bool par_neg)                                                   //par_factor = Kontraktionsfaktor, der vom Benutzer eingestellt werden kann, par_neg = Kontraktionsmodus(gespiegelt oder ungespiegelt))
    {   
        //mp_exp_t u;
        if(par_neg == false)                                                                                            //positives Vorzeichen des Kontraktionsfaktors?
            {
                for(unsigned int i = 1; i < p_contraction.size(); i++)                                                  //erst mit erstem Ergebnissparameter beginnen. Parameter 0 ist für Fitness-Wert reserviert und wird anders berechnet
                    {
                        //cout<<"NUM WORST_VALUE "<<i<<"#"<<p_range.at(p_range.size()-1).at(i).get_str(u)<<"E"<<u<<";"; 
                        //cout<<"NUM FACTOR #"<<par_factor.get_str(u)<<"E"<<u<<";";  
                        //cout<<"NUM CONTRACTION CENTER "<<i<<"#"<<p_center.at(i).get_str(u)<<"E"<<u<<";";  
                        p_contraction.at(i) = p_center.at(i) + par_factor * (p_range.at(p_range.size()-1).at(i) - p_center.at(i));//Kontraktionspunkt wird aus Zentrum + (schlechtester Punkt - Zentrum) * Reflexionsfaktor gebildet
                        //cout<<"NUM CONTRACTION PAR "<<i<<"#"<<p_contraction.at(i).get_str(u)<<"E"<<u<<";";           
                    } 
            }
        else                                                                                                            //negatives Vorzeichen des Kontraktionsfaktors?
            {

                for(unsigned int i = 1; i < p_contraction.size(); i++)                                                  //erst mit erstem Ergebnissparameter beginnen. Parameter 0 ist für Fitness-Wert reserviert und wird anders berechnet
                    {
                        //cout<<"NUM WORST_VALUE "<<i<<"#"<<p_range.at(p_range.size()-1).at(i).get_str(u)<<"E"<<u<<";";  
                        //cout<<"NUM FACTOR #"<<par_factor.get_str(u)<<"E"<<u<<";";  
                        //cout<<"NUM CONTRACTION CENTER "<<i<<"#"<<p_center.at(i).get_str(u)<<"E"<<u<<";";  
                        p_contraction.at(i) = p_center.at(i) - par_factor * (p_range.at(p_range.size()-1).at(i) - p_center.at(i));  //Kontraktionspunkt wird aus Zentrum - (schlechtester Punkt - Zentrum) * Reflexionsfaktor gebildet
                        //cout<<"NUM CONTRACTION 2 PAR "<<i<<"#"<<p_contraction.at(i).get_str(u)<<"E"<<u<<";";          
                    } 
            }
        (this->*ptr_fitness)(p_contraction);                                                                            //p_contraction[0]: Fitness-Wert (SSRE)
        //cout<<"NUM CONTRACTION PAR "<<0<<"#"<<p_contraction.at(0).get_str(u)<<"E"<<u<<";"; 
    }

//Generalkontraktion (ACHTUNG! KEINE GRENZWERT-ÜBERPRÜFUNG!)
void cl_points::general_contraction()                                                  
    {   
        //mp_exp_t u;
        for(unsigned int i = 1; i < p_range.size(); i++)                                                                //Für alle Rangunkte außer dem Besten...
            {
                for(unsigned int a = 1; a < p_range.at(i).size(); a++)                                                  //...für jeden Parameter außer den Fitnesswert...
                    {
                        p_range.at(i).at(a) = (p_range.at(i).at(a)+p_range.at(0).at(a))/2;                              //...den Wert des besten Rankpunktes addieren und das Ergebnis durch 2 dividieren
                        //cout<<"NUM GENERAL CONTRACTION PAR "<<a<<"#"<<p_range.at(i).at(a).get_str(u)<<"E"<<u<<";";          
                    } 
                (this->*ptr_fitness)(p_range.at(i));
                //cout<<"NUM GENERAL CONTRACTION PAR 0#"<<p_range.at(i).at(0).get_str(u)<<"E"<<u<<";";
            } 
    }

//Gültigkeit eines Rangpunktes bestimmen
bool cl_points::check_p_range(int par_pts)                                                                              //par_pts = Nummer des zu untersuchenden Rangpunktes
    {   
        return(colliding_borders(p_range.at(par_pts)));                                                                 //Überprüft die Grenzwerte der Parameter, korrigiert ggf. Abweichungen und berechnet dann neue Fitness-Werte.
    }
//Gültigkeit des Reflexionspunktes bestimmen
bool cl_points::check_reflexion()                                                                               
    {   
        return(colliding_borders(p_reflexion));                                                                         //Überprüft die Grenzwerte der Parameter, korrigiert ggf. Abweichungen und berechnet dann neue Fitness-Werte.
    }
//Gültigkeit des Expansionspunktes bestimmen
bool cl_points::check_expansion()                                                                               
    {   
        return(colliding_borders(p_expansion));                                                                         //Überprüft die Grenzwerte der Parameter, korrigiert ggf. Abweichungen und berechnet dann neue Fitness-Werte.
    }
//Gültigkeit des Kontraktionspunktes bestimmen
bool cl_points::check_contraction()                                                                               
    {   
        return(colliding_borders(p_contraction));                                                                       //Überprüft die Grenzwerte der Parameter, korrigiert ggf. Abweichungen und berechnet dann neue Fitness-Werte.
    }

//Bestimmen, ob angegebener Punkt besser oder schlechter als aktueller bester Rangpunkt ist
bool cl_points::compare_break(mpf_class par_brk)                                                                        //par_brk = Vergleichswert (Präzision)
    {   
        //mp_exp_t ex;
        if(p_range.at(0).at(0) > par_brk) {return(true);}
        return(false);                                                                                                  //p_reflection[0]: Fitness-Wert (SSRE)
    }

//Schreibt einen Rangpunkt in den Ergebnissarray
void cl_points::p_range_to_result(int par_pts)                                                                          //par_pts = Nummer des Rangpunktes            
    {   
        if(p_result.at(0) > p_range.at(par_pts).at(0))                                                                  //Ergebniss nur ersetzen, wenn der neue Rangpunkt besser ist
            {                                                                                                           
                for(unsigned int i = 0; i < p_range.at(par_pts).size(); i++)
                    {
                        p_result.at(i) = p_range.at(par_pts).at(i);
                    }  
            }
        else {l_imp_loop++;}                                                                                            //Neuen Wert für last_improvement_loop eintragen (Wichtig für die Weite des Zufallsintervalls zum setzen der Startwerte eines Punktes am Beginn eines neuen Loops)
    }

//Speichern, welcher Loop gerade läuft
void cl_points::loop(int par_loop)                                                                                
    {   
        a_loop = par_loop;                                                                        
    }

//Speichern, welcher Cycle gerade läuft
void cl_points::cycle(int par_cycle)                                                                          
    {   
        a_cycle = par_cycle;
    }



//Ergebnissparameter eines Rangpunkt setzen( Hüllfunktion, leitet die Anfrage über eien Funktionszeiger weiter)
int cl_points::seed_p_range(int par_pts)                                                                                //par_pts = Nummer des Rangpunktes
    {   
        (this->*ptr_seed_p_range)(par_pts);
        return(0);
    }

//Ergebnissparameter eines Rangpunkt zufällig, innerhalb der Grenzen der Ergebnissparameter setzen
int cl_points::seed_p_range_default(int par_pts)                                                                        //par_pts = Nummer des Rangpunktes
    {   
        mp_exp_t ex;
        
        for(unsigned int i = 1; i < p_borders.size(); i++)                                                              //Für alle Ergebnissparameter des Rangpunktes durchführen (Außer [0] = SSRE!)
            {
                if(rand()%3 > 0)                                                                                        //Zufällig für eine von zwei Optionen entscheiden (Links oder rechts vom bisherigen Ergebniss neuen Rangpunkt setzen)
                    {
                        mpf_betterval = p_result.at(i) + p_result.at(i)/(1+rand()%(100));                               //Parameter aus Intervall positiv = Ergebnissparameter + Ergebnissparameter/ (1+Zufallszahl % last_improved_loop)  ==> Zufallsbereich zieht sich mit jeder erfolgreichen Parameterübernahme weiter zu
                    }
                else{mpf_betterval = p_result.at(i) - p_result.at(i)/(1+rand()%(100));}                                 //Parameter aus Intervall negativ = Ergebnissparameter + Ergebnissparameter/ (1+Zufallszahl % last_improved_loop)  ==> Zufallsbereich zieht sich mit jeder erfolgreichen Parameterübernahme weiter zu

                p_range.at(par_pts).at(i) = mpf_betterval;                                                              //Rangpunkt-Parameter wird durch neuen Parameter ersetzt

                cout << "NUM SEED POINT "<< par_pts << " PAR " << i << "#" << p_range.at(par_pts).at(i).get_str(ex) << "E" << ex << ";";
            }
        (this->*ptr_fitness)(p_range.at(par_pts));                                                                      //Schritt 4: für [0] die SSRE bestimmen]
        return(0);
    }

//Ergebnissparameter eines Rangpunkt zufällig, innerhalb der Grenzen der Ergebnissparameter setzen (Hüllfunktion, leitet die Anfrage über eien Funktionszeiger weiter)
void cl_points::seed_wrong(int par_pts)                                                                                 //par_pts = Nummer des Rangpunktes
    {   
        (this->*ptr_seed_wrong)(par_pts);
    }

//Ergebnissparameter eines Rangpunkt zufällig, innerhalb der Grenzen der Ergebnissparameter setzen
void cl_points::seed_wrong_hard(int par_pts)                                                                            //par_pts = Nummer des Rangpunktes
    {   
         //mp_exp_t ex;
    
         for(unsigned int i = 1; i < p_range.at(par_pts).size(); i++)                                                   //Für alle Ergebnissparameter des Rangpunktes durchführen (Außer [0] = SSRE!)
            {
                if(p_range.at(par_pts).at(i) < p_borders.at(i).at(0) || p_range.at(par_pts).at(i) > p_borders.at(i).at(1))//Für jeden Parameter, der außerhalb der Grenzen liegt, neuen Wert bestimmen (Achtung: Nur ein Zufallsfaktor für alle veränderten Parameter!)
                    {
                        p_range.at(par_pts).at(i) = p_borders.at(i).at(0) + f_itv_borders.at(i)/(1+rand()%i_itv_borders.at(i));   //Unterer Grenzwert + Intervall(Float) /(1; damit nie durch 0 geteilt wird! + Parameter-Zufallszahl%Intervall(Integer))
                        //cout << "NUM SEED New "<< par_pts << " PAR " << i << "#" << p_range.at(par_pts).at(i).get_str(ex) << "E" << ex << ";";
                    }
            }
        (this->*ptr_fitness)(p_range.at(par_pts)); 
    }

//Ergebnissparameter eines Rangpunkt zufällig, innerhalb der Grenzen der Ergebnissparameter setzen
void cl_points::seed_wrong_weak(int par_pts)                                                                            //par_pts = Nummer des Rangpunktes
    {   
        //mp_exp_t ex;
        
        for(unsigned int i = 1; i < p_range.at(par_pts).size(); i++)                                                    //Für alle Ergebnissparameter des Rangpunktes durchführen (Außer [0] = SSRE!)
            {
                if(p_range.at(par_pts).at(i) < p_borders.at(i).at(0) || p_range.at(par_pts).at(i) > p_borders.at(i).at(1))//Für jeden Parameter, der außerhalb der Grenzen liegt, neuen Wert bestimmen (Achtung: Nur ein Zufallsfaktor für alle veränderten Parameter!)
                    {
                        if(rand()%3 > 0)
                            {
                                mpf_betterval = p_range.at(0).at(i) + p_range.at(0).at(i)/(1+rand()%100);               //Parameter aus Intervall positiv = Bester Rangpunkt + Bester Rangpunkt/ (1+Zufallszahl % 100)  ==> Zufallsbereich um besten Rankpunkt: Grenzverletzung unwahrscheinlicher, aber immer noch möglich
                            }
                        else{mpf_betterval = p_range.at(0).at(i) - p_range.at(0).at(i)/(1+rand()%100);}                 //Parameter aus Intervall positiv = Bester Rangpunkt + Bester Rangpunkt/ (1+Zufallszahl % 100)  ==> Zufallsbereich um besten Rankpunkt: Grenzverletzung unwahrscheinlicher, aber immer noch möglich
                        
                        p_range.at(par_pts).at(i) = mpf_betterval;                                                      //Schritt 2: Unterer Grenzwert + Intervall /(1+ Parameter-Zufallszahl; damit nie durch 0 geteilt wird!)
                        //cout << "NUM SEED New "<< par_pts << " PAR " << i << "#" << p_range.at(par_pts).at(i).get_str(ex) << "E" << ex << ";";
                    }
            }
        (this->*ptr_fitness)(p_range.at(par_pts));                                                                       //Schritt 3: für [0] die SSRE bestimmen]
    }

//Vergleicht zwei Rangpunkte miteinander
int cl_points::compare_pts(int par_pts_1, int par_pts_2)                                                                //par_pts_1 = Nummer des 1.Rangpunktes; par_pts_2 = Nummer des 2.Rangpunktes;
    {                                                                                                     
        if(p_range.at(par_pts_1).at(0) > p_range.at(par_pts_2).at(0)) {return(1);}
        else if(p_range.at(par_pts_1).at(0) < p_range.at(par_pts_2).at(0)) {return(-1);}
        return(0);
    }
//Vergleicht den angegebenen Rangpunkte mit dem schlechtesten Rangpunkt
int cl_points::compare_last(int par_pts)                                                                                //par_pts = Nummer des 1.Rangpunktes; 
    {                                                                                                     
        return(compare_all(par_pts,p_range.at(p_range.size()-1)));
    }
//Vergleicht den angegebenen Rangpunkte mit dem schlechtesten Rangpunkt
int cl_points::compare_ref(int par_pts)                                                                                 //par_pts = Nummer des 1.Rangpunktes; 
    {                                                                                                     
        return(compare_all(par_pts,p_reflexion));
    }
//Vergleicht den angegebenen Rangpunkte mit dem schlechtesten Rangpunkt
int cl_points::compare_ex(int par_pts)                                                                                  //par_pts = Nummer des 1.Rangpunktes; 
    {                                                                                                     
        return(compare_all(par_pts,p_expansion));
    }
//Vergleicht den angegebenen Rangpunkte mit dem schlechtesten Rangpunkt
int cl_points::compare_con(int par_pts)                                                                                 //par_pts = Nummer des 1.Rangpunktes; 
    {                                                                                                     
        return(compare_all(par_pts,p_contraction));
    }
//Vergleicht einen Rangpunkt mit einem anderen Punkt (gesteuert über Chars)
int cl_points::compare_result(int par_pts_1)                                                                            //par_pts_1 = Nummer des 1.Rangpunktes; l_r_e_c: "l" = letzter Rangpunkt; "r" = Reflexionspunkt; "e" =Expansionspunkt; "c" = Kontraktionspunkt;
    {   
        return(compare_all(par_pts_1,p_result));                                                                        //'l': par_vec_pt = letzter Rangpunkt
    }

//Ersetzt den angegebenen Rangpunkte durch den zweiten angegebenen Rangpunkt
void cl_points::replace_p_range(int par_pts_1,int par_pts_2)                                                            //par_pts_1 = Nummer des 1.Rangpunktes, par_pts_2 = Nummer des 2.Rangpunktes; 
    {                                                                                                     
        replace_all(par_pts_1,p_range.at(par_pts_2));
        //cout << "REPLACE POINT NUM "<< par_pts_1 << " with POINT "<< par_pts_2 <<"#geändert;";
    }
//Ersetzt den angegebenen Rangpunkte durch den schlechtesten Rangpunkt
void cl_points::replace_last(int par_pts)                                                                               //par_pts = Nummer des 1.Rangpunktes; 
    {                                                                                                     
        replace_all(par_pts,p_range.at(p_range.size()-1));
        //cout << "REPLACE LAST with POINT NUM "<< par_pts << "#geändert;";
    }
//Ersetzt den angegebenen Rangpunkte durch den Reflexionspunkt
void cl_points::replace_with_ref(int par_pts)                                                                           //par_pts = Nummer des 1.Rangpunktes; 
    {                                                                                                     
        replace_all(par_pts,p_reflexion);
        //cout << "REPLACE POINT NUM"<< par_pts << " with CON#geändert;";
    }
//Ersetzt den angegebenen Rangpunkte durch den Expansionspunkt
void cl_points::replace_with_ex(int par_pts)                                                                            //par_pts = Nummer des 1.Rangpunktes; 
    {                                                                                                     
        replace_all(par_pts,p_expansion);
        //cout << "REPLACE POINT NUM"<< par_pts << " with CON#geändert;";
    }
//Ersetzt den angegebenen Rangpunkte durch den Kontraktionspunkt
void cl_points::replace_with_con(int par_pts)                                                                           //par_pts = Nummer des 1.Rangpunktes; 
    {                                                                                                     
        replace_all(par_pts,p_contraction);
        //cout << "REPLACE POINT NUM"<< par_pts << " with CON#geändert;";
    }


//Gibt den gewählten Rangpunkt als Ausgabestream aus
void cl_points::out_p_range(int par_pts)                                                                                //par_pts = Nummer des Rangpunktes
    {   
        mp_exp_t ex;                                                                                                    //Variable zum Speichern des Exponenten, der bei der String-Auflösung eines mpf_class-Objekts entsteht
        //cout << "NUM RANGE" << par_pts << " PAR"<< 0 << "#" << p_range.at(par_pts).at(0).get_str(ex) << "E" << ex << ";";
      
        for(unsigned int i = 0; i < p_range.at(par_pts).size(); i++)                                                    //Für alle Ergebnissparameter des Rangpunktes durchführen 
            {
                cout << "NUM RANGE" << par_pts << " PAR"<< i << "#" << p_range.at(par_pts).at(i).get_str(ex) << "E" << ex << ";"; 
            }
    
    }

//Gibt den gewählten Rangpunkt und die berechneten Modellwerte als Ausgabestream aus
void cl_points::out_p_result()                                                                                          //par_pts = Nummer des Rangpunktes
    {   
        //mp_exp_t ex;                                                                                                  //Variable zum Speichern des Exponenten, der bei der String-Auflösung eines mpf_class-Objekts entsteht
        int n = 20;                                                                                                     //Anzahl der Nachkommastellen 
        cout << "BEGIN_RESULTS#Beginn Ergebniss;";                                                                      //Beginn des Ergebnissblocks
        for(unsigned int i = 0; i < p_result.size(); i++)                                                               //Für alle Ergebnissparameter des Rangpunktes durchführen 
            {
                cout << "NUM RESULT_PAR "<< i << "#";
                gmp_printf ("%.*Ff;", n, p_result.at(i).__get_mp());
            }
        (this->*ptr_fitness)(p_result);                                                                                 //Sicherstellen, dass c_value auch die richtigen Modellwerte enthält
        for(unsigned int i = 0; i < c_values.size(); i++)                                                               //Für alle Ergebnissparameter des Ergebnisspunktes durchführen 
            {
                cout << "NUM COMPUTED_PAR " << i <<"#";
                gmp_printf ("%.*Ff;", n, c_values.at(i).__get_mp());
            }
        cout << "END_RESULTS#Beginn Ergebniss;";                                                                        //Ende des Ergebnissblocks
    }

//Gibt die Werte passend für das HTML-Log-Div aus
void cl_points::out_stat()                                                                                
    {   
        //mp_exp_t ex;                                                                                                  //Variable zum Speichern des Exponenten, der bei der String-Auflösung eines mpf_class-Objekts entsteht
        int n = 10;                                                                                                     //Anzahl der Nachkommastellen
                                        
        cout << "NUM STAT_RANGE PAR 0#";
        gmp_printf ("%.*Ff;", n, p_range.at(0).at(0).__get_mp());                                                       //Fitness-Wert des aktuell besten Rangpunktes angeben
        cout << "NUM STAT_RESULT PAR 0#";
        gmp_printf ("%.*Ff;", n, p_result.at(0).__get_mp());                                                            //Fitness-Wert des aktuellen Ergebnissparameters ausgeben  
        
gmp_printf ("NUM STAT_RESULT PAR 0%.*Ff with %d digits\n", n, p_result.at(0).__get_mp(), n);
    }

//---------------Private functions

//Gültigkeit eines Punktes anhand der festgelegten Grenzwerte bestimmen
bool cl_points::colliding_borders(vector< mpf_class >& par_vec_pt)                                                      //par_vect_pt = Referenz auf den zu untersuchenden Punktvektor; par_changed = Initialisierungswert für Fitness-Überprüfung, true = Fitness wird in jedem Fall überprüft
    {   
                                                                                                                        //Überprüfung, ob sich etwas geändert hat
        //mp_exp_t ex; 
        for(unsigned int i = 1; i < par_vec_pt.size(); i++)
            {
                if(par_vec_pt.at(i) < p_borders.at(i).at(0))                                                            //Unterschreiten des unteren Grenzwertes für den Parameter
                    {
                        //cout << "GRENZE UNTEN"<< i << "#" << p_borders.at(i).at(0).get_str(ex) << "E" << ex << ";"; 
                        return(true);                                                                                   //Verletzung zurückgeben
                    }
                else if(par_vec_pt.at(i) > p_borders.at(i).at(1))                                                       //Überschreiten des oberen Grenzwertes für den Parameter
                    {
                        //cout << "GRENZE OBEN"<< i << "#" << p_borders.at(i).at(1).get_str(ex) << "E" << ex << ";"; 
                        return(true);                                                                                   //Verletzung zurückgeben
                    } 
            }                                                                                                           
        return(false);
    }

void cl_points::correct_borders(vector< mpf_class >& par_vec_pt)                                                        //par_vect_pt = Referenz auf den zu untersuchenden Punktvektor; par_changed = Initialisierungswert für Fitness-Überprüfung, true = Fitness wird in jedem Fall überprüft
    {   
                                                                                                                        //Überprüfung, ob sich etwas geändert hat
        //mp_exp_t ex; 
        for(unsigned int i = 1; i < par_vec_pt.size(); i++)
            {
                if(par_vec_pt.at(i) < p_borders.at(i).at(0))                                                            //Unterschreiten des unteren Grenzwertes für den Parameter
                    {
                        //cout << "NUM WERT PAR "<< i << "#" << par_vec_pt.at(i).get_str(ex) << "E" << ex << ";"; 
                        par_vec_pt.at(i) = p_borders.at(i).at(0);                                                       //Wert auf unteren Grenzwert setzen
                        
                        //cout << "GRENZE UNTEN"<< i << "#" << p_borders.at(i).at(0).get_str(ex) << "E" << ex << ";"; 
                    }
                else if(par_vec_pt.at(i) > p_borders.at(i).at(1))                                                       //Überschreiten des oberen Grenzwertes für den Parameter
                    {
                        //cout << "NUM WERT PAR "<< i << "#" << par_vec_pt.at(i).get_str(ex) << "E" << ex << ";"; 
                        par_vec_pt.at(i) = p_borders.at(i).at(1);                                                       //Wert auf unteren Grenzwert setzen
                        
                        //cout << "GRENZE OBEN"<< i << "#" << p_borders.at(i).at(1).get_str(ex) << "E" << ex << ";"; 
                    } 
            }
        (this->*ptr_fitness)(par_vec_pt);                                                                               //par_vec_pt[0]: Fitness-Wert (SSRE) (kann p_range[x][0] oder p_reflexion[0] oder p_expansion[0] oder p_contraction[0] sein)                                                                                                 
    }

//berechnet die Fitness eines Punktes anhand der vorliegenden Funktionen
void cl_points::makergaul(vector< mpf_class >& par_vec_pt)                                                              //par_vec_p: Punkt, dessen Fitness geprüft werden soll                      
    {   
        //Fitness nach dem MAKERGAUL-Modell
        //Wertung der Ergebnissparameter(aus par_vec_pt entnommen): 
        //[0] = SSRE, hier wird das Ergebniss der Funktion gespeichert
        //[1] = DNA
        //[2] = max. Enzymmenge
        //[3] = Inhibitionskonstante
        //[4] = Basisfluoreszens
        //[5-x] = nicht verarbeitete Parameter, werden nicht berücksichtigt!
        //mp_exp_t u;
        
        try{c_values.at(0) = par_vec_pt.at(1) + par_vec_pt.at(4);}                                                      //Der erste berechnete Modellwert entspricht der Start DNA[0] + der Start_Basisfluoreszens[4]
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen (auf die Gültigkeit der Felder 2 und 3 wird nicht geprüft!)
            {
                cout << "ERR#par_vec_pt(1 or 4) dosn't exist!;";
                exit(1);
            }                   
        if(par_vec_pt.at(3) < p_borders.at(2).at(0))                                                                    //Die MAKERGAUL-Funktion bildet bei Unterschreiten des Grenzwerts der Inhibitionskonstante ein Nebenminimum, auf das sehr oft iteriert wird...
            {
                par_vec_pt.at(0)=10000;                                                                                 //...daher wird eine solche Unterschreitung mit einer Fitnesstrafe belegt!
                return;
            } 
        ssre_step = 0;
        ssre = 0;
        m_value_mw = m_values.at(0);                                                                                    //Ersten Messwert = Startsumme der Messwerte
        dna = par_vec_pt.at(1);                                                                                         //Erster Wert für dna_last entspricht der Start-DNA[0]
        e_usable = par_vec_pt.at(2);                                                                                    //vor dem ersten Zyklus entspricht die nutzbare Enzymmenge der maximalen Enzymmenge
        //cout << "NUM DNA 0#"<< dna.get_str(u) <<"E" << u << ";";  
        //cout << "NUM C_VALUES 0#"<< c_values.at(0).get_str(u) <<"E" << u << ";";  
        //cout << "NUM E_USABLE 0#"<< e_usable.get_str(u) <<"E" << u << ";";  
        //cout << "NUM INHIB#"<< par_vec_pt.at(3).get_str(u) <<"E" << u << ";"; 
        //cout << "NUM BASIS#"<< par_vec_pt.at(4).get_str(u) <<"E" << u << ";"; 
        //cout << "NUM C_VALUES MW 0#"<< c_value_mw.get_str(u) <<"E" << u << ";"; 
        for(unsigned int i = 1; i < m_values.size(); i++)                                                               //Alle folgenden Werte c_values[1-x] werden durch die MAKERGAUL-Formel berechnet
            {
                 e_usable = e_usable/(1 + par_vec_pt.at(3)*dna);                                                        //1.Schritt: verfügbare Polymerase: poly_usable_now = poly_usable_last / (1 + inhibition_k * dna_last)
                 dna = dna + dna * e_usable/(e_usable+dna);                                                             //2.Schritt: aktuelle DNA bestimmen: dna_now =dna_last + dna_last * e_usable_now/(e_usable_now + dna_last)
                 c_values.at(i) = dna + par_vec_pt.at(4);                                                               //3.Schritt: Modellwert bestimmen: Basisfluoreszens[4] + dna_now
                 m_value_mw = m_value_mw + m_values.at(i);                                                              //4.Schritt: Messwert zur Summe der Messwerte hinzuaddieren
        //cout << "NUM DNA "<<i<<"#"<< dna.get_str(u) <<"E" << u << ";";  
        //cout << "NUM C_VALUES "<<i<<"#"<< c_values.at(0).get_str(u) <<"E" << u << ";";  
        //cout << "NUM E_USABLE "<<i<<"#"<< e_usable.get_str(u) <<"E" << u << ";";  
        //cout << "NUM C_VALUES MW"<<i<<"#"<< c_value_mw.get_str(u) <<"E" << u << ";"; 
            }
        
        m_value_mw = m_value_mw/m_values.size();                                                                        //5.Schritt: Aus Summe der Messwerte wird Mittelwert der Messwerte
        //cout << "NUM M_VALUES MW END#"<< m_value_mw.get_str(u) <<"E" << u << ";"; 
        for(unsigned int i = 0; i < c_values.size(); i++)                                                               //Sum of Relative Squared Errors (SSRE) als Fitnesswert berechnen
            {
                ssre_step = (m_values.at(i)-c_values.at(i))/(m_values.at(i)-m_value_mw);                                //1.Schritt: (Messwert[i] - Modellwert[i]) / (Messwert(i) - Mittelwert_Messwerte)
                ssre_step = ssre_step * ssre_step;                                                                      //2.Schritt: Quadrieren (leider keinen Quadrierungsoperator für die Klasse gefunden)
                ssre = ssre + ssre_step;                                                                                //3.Schritt: Zur SSRE addieren
        //cout << "NUM SSRE STEP "<<i<<"#"<< ssre_step.get_str(u) <<"E" << u << ";";  
        //cout << "NUM C_VALUE #"<< c_values.at(i).get_str(u) <<"E" << u << ";";  
        //cout << "NUM M_VALUE "<<i<<"#"<< m_values.at(i).get_str(u) <<"E" << u << ";";  
        //cout << "NUM SSRE"<<i<<"#"<< ssre.get_str(u) <<"E" << u << ";"; 
            }
        par_vec_pt.at(0) = ssre;                                                                                        //SSRE in par_vec_pt.at(0) abspeichern (erster Wert jedes Punktes immer Fitness-Wert!)
        //cout << "NUM SSRE END#"<< ssre.get_str(u) <<"E" << u << ";";
    }

//berechnet die Fitness eines Punktes anhand der vorliegenden Funktionen
void cl_points::makergaul_c(vector< mpf_class >& par_vec_pt)                                                              //par_vec_p: Punkt, dessen Fitness geprüft werden soll                      
    {   
        //Fitness nach dem MAKERGAUL-C-Modell
        //Wertung der Ergebnissparameter(aus par_vec_pt entnommen): 
        //[0] = SSRE, hier wird das Ergebniss der Funktion gespeichert
        //[1] = DNA
        //[2] = max. Enzymmenge
        //[3] = Basisfluoreszens
        //[4-x] = nicht verarbeitete Parameter, werden nicht berücksichtigt!
        //mp_exp_t u;
        //In dieser Abwandlung von MAKERGAUL wird die Inhibitionskonstante auf einen konstanten, vorher ermittelten Wert gesetzt (f_inhib).
        try{c_values.at(0) = par_vec_pt.at(1) + par_vec_pt.at(3);}                                                      //Der erste berechnete Modellwert entspricht der Start DNA[0] + der Start_Basisfluoreszens[4]
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen (auf die Gültigkeit der Felder 2 und 3 wird nicht geprüft!)
            {
                cout << "ERR#par_vec_pt(1 or 3) dosn't exist!;";
                exit(1);
            }                   
        ssre_step = 0;
        ssre = 0;
        m_value_mw = m_values.at(0);                                                                                    //Ersten Messwert = Startsumme der Messwerte
        dna = par_vec_pt.at(1);                                                                                         //Erster Wert für dna_last entspricht der Start-DNA[0]
        e_usable = par_vec_pt.at(2);                                                                                    //vor dem ersten Zyklus entspricht die nutzbare Enzymmenge der maximalen Enzymmenge
        //cout << "NUM DNA 0#"<< dna.get_str(u) <<"E" << u << ";";  
        //cout << "NUM C_VALUES 0#"<< c_values.at(0).get_str(u) <<"E" << u << ";";  
        //cout << "NUM E_USABLE 0#"<< e_usable.get_str(u) <<"E" << u << ";";  
        //cout << "NUM INHIB#"<< par_vec_pt.at(3).get_str(u) <<"E" << u << ";"; 
        //cout << "NUM BASIS#"<< par_vec_pt.at(4).get_str(u) <<"E" << u << ";"; 
        //cout << "NUM C_VALUES MW 0#"<< c_value_mw.get_str(u) <<"E" << u << ";"; 
        for(unsigned int i = 1; i < m_values.size(); i++)                                                               //Alle folgenden Werte c_values[1-x] werden durch die MAKERGAUL-Formel berechnet
            {
                 e_usable = e_usable/(1 + f_inhib * dna);                                                               //1.Schritt: verfügbare Polymerase: poly_usable_now = poly_usable_last / (1 + f_inhib(experimentell ermittelt) * dna_last)
                 dna = dna + dna * e_usable/(e_usable+dna);                                                             //2.Schritt: aktuelle DNA bestimmen: dna_now =dna_last + dna_last * e_usable_now/(e_usable_now + dna_last)
                 c_values.at(i) = dna + par_vec_pt.at(3);                                                               //3.Schritt: Modellwert bestimmen: Basisfluoreszens[4] + dna_now
                 m_value_mw = m_value_mw + m_values.at(i);                                                              //4.Schritt: Messwert zur Summe der Messwerte hinzuaddieren
        //cout << "NUM DNA "<<i<<"#"<< dna.get_str(u) <<"E" << u << ";";  
        //cout << "NUM C_VALUES "<<i<<"#"<< c_values.at(0).get_str(u) <<"E" << u << ";";  
        //cout << "NUM E_USABLE "<<i<<"#"<< e_usable.get_str(u) <<"E" << u << ";";  
        //cout << "NUM C_VALUES MW"<<i<<"#"<< c_value_mw.get_str(u) <<"E" << u << ";"; 
            }
        
        m_value_mw = m_value_mw/m_values.size();                                                                        //5.Schritt: Aus Summe der Messwerte wird Mittelwert der Messwerte
        //cout << "NUM M_VALUES MW END#"<< m_value_mw.get_str(u) <<"E" << u << ";"; 
        for(unsigned int i = 0; i < c_values.size(); i++)                                                               //Sum of Relative Squared Errors (SSRE) als Fitnesswert berechnen
            {
                ssre_step = (m_values.at(i)-c_values.at(i))/(m_values.at(i)-m_value_mw);                                //1.Schritt: (Messwert[i] - Modellwert[i]) / (Messwert(i) - Mittelwert_Messwerte)
                ssre_step = ssre_step * ssre_step;                                                                      //2.Schritt: Quadrieren (leider keinen Quadrierungsoperator für die Klasse gefunden)
                ssre = ssre + ssre_step;                                                                                //3.Schritt: Zur SSRE addieren
        //cout << "NUM SSRE STEP "<<i<<"#"<< ssre_step.get_str(u) <<"E" << u << ";";  
        //cout << "NUM C_VALUE #"<< c_values.at(i).get_str(u) <<"E" << u << ";";  
        //cout << "NUM M_VALUE "<<i<<"#"<< m_values.at(i).get_str(u) <<"E" << u << ";";  
        //cout << "NUM SSRE"<<i<<"#"<< ssre.get_str(u) <<"E" << u << ";"; 
            }
        par_vec_pt.at(0) = ssre;                                                                                        //SSRE in par_vec_pt.at(0) abspeichern (erster Wert jedes Punktes immer Fitness-Wert!)
        //cout << "NUM SSRE END#"<< ssre.get_str(u) <<"E" << u << ";";
    }

//berechnet die Fitness eines Punktes anhand der vorliegenden Funktionen
void cl_points::mak2(vector< mpf_class >& par_vec_pt)                                                                   //par_vec_p: Punkt, dessen Fitness geprüft werden soll                      
    {   
        //Fitness nach dem MAK2-Modell
        //Wertung der Ergebnissparameter(aus par_vec_pt entnommen): 
        //[0] = SSE, hier wird das Ergebniss der Funktion gespeichert
        //[1] = DNA
        //[2] = Reaktionskonstante
        //[3] = Basisfluoreszens
        //[4-x] = nicht verarbeitete Parameter, werden nicht berücksichtigt!
    
        //mp_exp_t u;
        sse = 0;                                                                                                        //Alle wichtigen Zwischenvariablen von Werten befreien
        sse_step = 0;
        mpfr_set_si(mpfr_logres,0,MPFR_RNDN);
        mpf_set_ui(mpf_temp,0);
                                                                                                                        //Der Datentyp mpf_t liegt mp-float-Objekten sowohl in MPIR als auch in MPFR zugrunde. Daher wird der Ergebnissparameter als mpf_t aus dem mpf_class-Objekt extrahiert und in den MPFR-Typen mpfr_t konvertiert.
        mpfr_set_f(mpfr_dna,par_vec_pt.at(1).get_mpf_t(),MPFR_RNDN);                                                    //par_vec_pt.at(1) = DNA(0)                           
        mpfr_set_f(mpfr_const,par_vec_pt.at(2).get_mpf_t(),MPFR_RNDN);                                                  //par_vec_pt.at(2) = Reaktionskonstante
        mpfr_set_f(mpfr_basis,par_vec_pt.at(3).get_mpf_t(),MPFR_RNDN);                                                  //par_vec_pt.at(3) = Basisfluoreszens
        
        
        try{c_values.at(0) = par_vec_pt.at(1) + par_vec_pt.at(3);}                                                      //Der Erste berechnete Modellwert entspricht der Start DNA[0] + der Start_Basisfluoreszens[3]
        catch(out_of_range& fehler)                                                                                     //Bei Versuch, in ungültiges Feld zu schreiben, mit Fehler abbrechen (auf die Gültigkeit des Feldes 2 wird nicht geprüft!)
            {
                cout << "ERR#par_vec_pt(1 or 3) dosn't exist!;";
                exit(1);
            }                   
  
        for(unsigned int i = 0; i < m_values.size(); i++)                                                               //Alle folgenden Werte c_values[1-x] werden durch die MAK2-Formel berechnet
            {
                 mpfr_div(mpfr_logres,mpfr_dna,mpfr_const,MPFR_RNDN);
                 mpfr_add_ui(mpfr_logres,mpfr_logres,1,MPFR_RNDN);
                 mpfr_log(mpfr_logres,mpfr_logres,MPFR_RNDN);                                                           //1.Schritt: Logarithmus innerhalb der MAK2-Funktion berechen: logresult = ln(1+(dna_last/const))
                 mpfr_mul(mpfr_addtemp,mpfr_const,mpfr_logres,MPFR_RNDN);                                                             
                 mpfr_add(mpfr_dna,mpfr_dna,mpfr_addtemp,MPFR_RNDN);                                                    //2.Schritt: aktuelle DNA bestimmen: dna_now = dna_last + const * logresult
                 
                 mpfr_get_f(mpf_temp,mpfr_dna,MPFR_RNDN);                                                               //3.Schritt: DNA von mpfr_t wieder in mpf_t zurückkonvertieren
                                                                                                                        
                 mpf_class obj_temp(mpf_temp);                                                                          //4.Schritt: Zwischen-mpf-class-Objekt initialisieren (Der einzige Weg, einem mpf_class-Objekt einen mpf_t-Wert zuzuordnen)
                 c_values.at(i) = obj_temp + par_vec_pt.at(3);                                                          //5.Schritt: Modellwert bestimmen: dna_now + Basisfluoreszens[3]
            }
                 
        //cout << "NUM M_VALUES MW END#"<< m_value_mw.get_str(u) <<"E" << u << ";"; 
        for(unsigned int i = 0; i < c_values.size(); i++)                                                               //Sum of Squared Errors (SSE) als Fitnesswert berechnen
            {
                sse_step = m_values.at(i)-c_values.at(i);                                                               //1.Schritt: Messwert[i] - Modellwert[i]
                sse_step = sse_step * sse_step;                                                                         //2.Schritt: Quadrieren
                sse = sse + sse_step;                                                                                   //3.Schritt: Zur SSE addieren
        //cout << "NUM SSRE STEP "<<i<<"#"<< sse_step.get_str(u) <<"E" << u << ";";  
        //cout << "NUM C_VALUE #"<< c_values.at(i).get_str(u) <<"E" << u << ";";  
        //cout << "NUM M_VALUE "<<i<<"#"<< m_values.at(i).get_str(u) <<"E" << u << ";";  
        //cout << "NUM SSRE"<<i<<"#"<< sse.get_str(u) <<"E" << u << ";"; 
            }
        par_vec_pt.at(0) = sse;                                                                                         //SSE in par_vec_pt.at(0) abspeichern (erster Wert jedes Punktes immer Fitness-Wert!)
        //cout << "NUM SSE END#"<< sse.get_str(u) <<"E" << u << ";";

    }


//Vergleicht einen Rangpunkt mit einem beliebigen anderen Punkt
int cl_points::compare_all(int par_pts, vector< mpf_class > &par_vec_pt)                                                //par_pts_1 = Nummer des Rangpunktes; par_vec_pt = Referenz anuf den anderen Punkt;
    {                                                                                                     
        if(p_range.at(par_pts).at(0) > par_vec_pt.at(0)) {return(1);}
        else if(p_range.at(par_pts).at(0) < par_vec_pt.at(0)) {return(-1);}
        return(0);
    }

//Ersetzt einen Rangpunkt durch einen beliebigen anderen Punkt
void cl_points::replace_all(int par_pts, vector< mpf_class > &par_vec_pt)                                               //par_pts_1 = Nummer des Rangpunktes; par_vec_pt = Referenz anuf den anderen Punkt;
    {                                                                                                     
        for(unsigned int i = 0; i < par_vec_pt.size(); i++)
            {
               p_range.at(par_pts).at(i) = par_vec_pt.at(i);
               //cout << "REPLACE NUM PAR"<< i << "#geändert;";
            }
    }
//Destruktor der Punktewolke
cl_points::~cl_points() 
    {
  
    }

