/*
 * -- cl_points.h -- 
 * 
 * Stellt die Eigenschaften eines Punktobjekts  für die Downhill-Simplex-Analyse bereit
 * 
 * File:   cl_points.h
 * Copyright 2013 Christoph Bultmann
 * 
 * This file is part of makergaulxx.
 *
 * makergaulxx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * makergaulxx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with makergaulxx.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CL_POINTS_H
#define	CL_POINTS_H

#include <iostream>
#include <vector>
//GMP GNU Multi Precision-Bibliothek (Version: 5.1.2)(c und c++-Header -> benötigt für MPFR)
#include <gmp.h>
#include <gmpxx.h>
//MPFR Multi-Precision-Float-Bibliothek (Version: 3.1.2, Zusätzlich verwendet um MP-Ln-Funktion in MAK2 benutzen zu können)
#include <mpfr.h>
//Zeit(wird benötigt, um Seed für rand() zu gewinnen, da sonst immer wieder die selben "Zufallszahlen" erscheinen)
#include <time.h>

using namespace std;


class cl_points {                              
    
public:
    cl_points();                                                                //Leerer Konstruktor zum Deklaration. Initialisieren nicht vergessen!
    cl_points(int par_pts, int par_res, int par_m_values, int par_alg);         //Konstruktor. benötigt Anzahl der Simplexpunkt, Anzahl der Ergebnissparameter, Anzahl der Messwerte
    void init(int par_pts,int par_res ,int par_m_values, int par_alg);          //Initialisierungsfunktion
    
    virtual ~cl_points();                                                       //Destruktor
                           
    void build_center();                                                        //Errechnet das Zentrum der Rangpunkte
    void build_reflexion(mpf_class par_factor);                                 //Errechnet den Reflexionspunkt des Simplex
    void build_expansion(mpf_class par_factor);                                 //Errechnet den Expansionspunkt des Simplex
    void build_contraction(mpf_class par_factor, bool par_neg);                 //Errechnet den Kontraktionspunkt des Simplex
    
    bool check_p_range(int par_pts);                                            //Kontrolliert, ob Rangpunkt[par_pts] innerhalb der gesetzten Grenzen liegt
    bool check_reflexion();                                                     //Kontrolliert, ob Reflexionspunkt innerhalb der gesetzten Grenzen liegt
    bool check_expansion();                                                     //Kontrolliert, ob Expansionspunkt innerhalb der gesetzten Grenzen liegt
    bool check_contraction();                                                   //Kontrolliert, ob Kontraktionspunkt innerhalb der gesetzten Grenzen liegt
    
    int compare_pts(int par_pts_1, int par_pts_2);                              //Vergleicht die Fitness zweier Rangpunkte miteinander. pts_1 > pts_2 = 1; pts_1 == pts_2 = 0; pts_1 < pts_2 = -1;
    int compare_last(int par_pts);                                              //Vergleicht die Fitness eines Rangpunkts mit dem schlechtesten Rangpunkt
    int compare_ref(int par_pts);                                               //Vergleicht die Fitness eines Rangpunkts mit dem Reflexionspunkt
    int compare_ex(int par_pts);                                                //Vergleicht die Fitness eines Rangpunkts mit dem Expansionspunkt
    int compare_con(int par_pts);                                               //Vergleicht die Fitness eines Rangpunkts mit dem Kontraktionspunkt
    int compare_result(int par_pts);                                            //Vergleicht einen Rangpunkt mit dem bisherigen Ergebniss
    bool compare_break(mpf_class par_brk);                                      //Kontrolliert, ob der angegebene Wert besser oder schlechter als der des aktuellen Rangpunkt[0][0] ist. TRUE, wenn <; FALSE, wenn >= 
    
    void cycle(int par_cycle);                                                  //Setzt einen Ergebnissparameter eines Rangpunkts mit Zufallswerten 
    
    void general_contraction();                                                 //Addiert auf jeden Parameter jedes Rangpunktes [1-x] den entsprechenden Parameter des besten Rangpunkts [0] und teilt das Ergebniss durch 2
    void loop(int par_loop);                                                    //Eintragen, welcher Loop gerade läuft
    
    void out_p_range(int par_pts);                                              //Gibt alle Ergebnissparameter des gewählten Rangpunktes als cout-Stream aus
    void out_p_result();                                                        //Gibt alle Ergebnissparameter und die Modellwerte des besten Rangpunktes als cout-Stream aus
    void out_stat();                                                            //Gibt Fitnesswert des aktuellen Ergebnisses und des aktuell besten Rangpunktes aus 
    
    void p_range_to_result(int par_pts);                                        //Schreibt einen Rangpunkt in das Ergebnissarray

    void replace_last(int par_pts);                                             //Ersetzt einen Rangpunkt durch den schlechtesten Rangpunkt
    void replace_with_ref(int par_pts);                                         //Ersetzt einen Rangpunkt durch den Reflexionspunkt
    void replace_with_ex(int par_pts);                                          //Ersetzt einen Rangpunkt durch den Expansionspunkt
    void replace_with_con(int par_pts);                                         //Ersetzt einen Rangpunkt durch den Kontraktionspunkt
    void replace_p_range(int par_pts_1,int par_pts_2);                          //Ersetzt einen Rangpunkt durch einen anderen Rangpunkt
    
    int seed_p_range(int par_pts);                                              //Setzt einen Ergebnissparameter eines Rangpunkts mit Zufallswerten 
    void seed_wrong(int par_pts);                                               //Ersetzt Grenzwertverletzende Ergebnissparameter eines Rangpunkts mit Zufallswerten (Hüllfunktion: Weiterleitung über definierten Funktionszeiger)
    
    bool set_analysis(int par_alg, int par_res);                                //Festlegen der Fitnessfunktion und abgleich, ob genügend Ergebnissparameter vorgegeben werden    
    void set_number_of(int par_pts, int par_res,int par_m_values);              //Ändert die Anzahl der Punkte, ihrer Ergebnisswerte, der Messwerte und der berechneten Modellwerte
    int set_m_value(int par_num, mpf_class par_value) ;                         //Schreiben von Messwerten in Array (Eingabetyp: mpf_class)
    int set_m_value(int par_num, string par_value) ;                            //Schreiben von Messwerten in Array (Eingabetyp: string)
    int set_c_value(int par_num, mpf_class par_value);                          //Schreiben von berechneten Modellwerten in Array (Eingabetyp: mpf_class)
    int set_c_value(int par_num, string par_value) ;                            //Schreiben von berechneten Modellwerten in Array (Eingabetyp: string)
    int set_fix_value(int par_num, string par_value) ;                          //Schreiben von Fixparametern in Array (Eingabetyp: string)
    int set_point_value(int par_pts, int par_res, mpf_class par_value);         //Setzen eines Ergebniswertes mittels mpf_class...
    int set_point_value(int par_pts, int par_res, string par_value);            //...oder als String
    int set_border(int par_res, mpf_class par_border_down, mpf_class par_border_up);//Setzt die Grenzwerte für einen Ergebnissparameter mittels mpf_class...
    int set_border(int par_res, string par_border_down, string par_border_up);  //...oder als String
    
    int sort_points();                                                          //Sortiert die Ergebnisswerte der Rangpunkte nach dem ersten Ergebnisswert ( p_range[x][0] )
   
 
private:
    mpf_class mpf_betterval;                                                    //Speichert den neuen Parameter, der aus einer einem Intervall in einer Seed-Funktion gewählt wird (Aus Funktion ausgelagert, um Performance durch weniger Initialisierungen zu erhöhen)
    int a_loop;                                                                 //Aktueller Versuch (=Loop)
    int a_cycle;                                                                //Aktuelle Iteration (=Cycle)
    int l_imp_loop;                                                             //Zählvariable, die bei nicht erfolgeichem Cyclus (= kein besseres Ergebnis) hochgezählt wird
    
    mpf_class dna, e_usable, m_value_mw, ssre_step, ssre;                       //Variablen der MAKERGAUL-Funktion (Aus Funktion ausgelagert, um Performance durch weniger Initialisierungen zu erhöhen)
    mpf_class f_inhib;                                                          //Variablen der MAKERGAUL-C-Funktion (Aus Funktion ausgelagert, um Performance durch weniger Initialisierungen zu erhöhen)
    mpf_class sse_step, sse;                                                    //Variablen der MAK2-Funktion (Aus Funktion ausgelagert, um Performance durch weniger Initialisierungen zu erhöhen)
    mpfr_t mpfr_dna, mpfr_const, mpfr_basis, mpfr_logres, mpfr_addtemp;         //MPFR-Variablen der MAK2-Funktion (Aus Funktion ausgelagert, um Performance durch weniger Initialisierungen zu erhöhen)
    mpf_t mpf_temp;                                                             //MPFT-Zwischenspeichervariable der MAK2-Funktion, um die Rückumwandlung von mpfr_t zu mpf_class möglich zu machen (dient als Initialisierungsparameter einer mpf_class)
    
    mpf_class factori,competimer;                                                           //Variablen der MAKERGAUL-C-Funktion
    
    vector< mpf_class > c_values;                                               //Enthält die berechneten Modellwerte (Anhand der Ergebnisse des besten Punkts berechnet)
    vector< mpf_class > m_values;                                               //Enthält die eingeschlossenen Messwerte (Einschluss/Ausschluss wird im PHP-Anteil des Moduls abgearbeitet)
    
    vector< vector< mpf_class > > h_range;                                      //Zweidimensionaler Hilfsvektor zum Zwischenspeichern von p_range beim Sortiervorgang
    
    vector< mpf_class > f_itv_borders;                                          //Eindimensionaler Array, speichert das Intervall zwischen Oberem und Unterem Grenzwert
    vector< mpz_class > i_itv_borders;                                          //Eindimensionaler Array, speichert das den dezimalen Abstand zwischen Oberem und Unterem Grenzwert
    
    vector< vector< mpf_class > > p_range;                                      //Zweidimensionaler Vektor für die Werte der Rangpunkte wie z.B. DNA, Basisfluoreszens,...
    vector< vector< mpf_class > > p_borders;                                    //Zweidimensionaler Vektor für die Grenzwerte der Ergebnissparameter(einen Oberen, einen Unteren)
    
    vector< mpf_class > p_result;                                               //Eindimensionaler Array für die Ergebnissparameter des besten Rangpunktes
    
    vector< mpf_class > p_center;                                               //Eindimensionaler Array für den Zentrumspunkt
    vector< mpf_class > p_reflexion;                                            //Eindimensionaler Array für den Reflexionspunkt
    vector< mpf_class > p_expansion;                                            //Eindimensionaler Array für den Expansionspunkt
    vector< mpf_class > p_contraction;                                          //Eindimensionaler Array für den Kontraktionspunkt
    
    bool colliding_borders(vector< mpf_class >& par_vec_pt);                    //Berechnet Grenzwertverletzungen des angegebenen Punktes
    void correct_borders(vector< mpf_class >& par_vec_pt);                      //Korrigiert verletzte Grenzwerte eines Punktes
    int compare_all(int par_pts, vector< mpf_class > &par_vec_pt);              //Vergleicht den angegebenen Rangpunkt mit einem anderen Punkt
    
    void replace_all(int par_pts, vector< mpf_class > &par_vec_pt);             //Ersetzt den angegebenen Rangpunkt durch einem anderen Punkt
    
    int seed_p_range_default(int par_pts);                                      //Setzt einen Ergebnissparameter eines Rangpunkts mit Zufallswerten 
    
    void seed_wrong_weak(int par_pts);                                          //Ersetzt Grenzwertverletzende Ergebnissparameter eines Rangpunkts mit Zufallswerten (Weiche Grenzsetzung = immer noch Grenzverletzung möglich)
    void seed_wrong_hard(int par_pts);                                          //Ersetzt Grenzwertverletzende Ergebnissparameter eines Rangpunkts mit Zufallswerten (Harte Grenzsetzung = keine neue Grenzverletzung möglich)
    
    int set_fix_none(int par_num, string par_value);                            //Schreiben von Fixparametern in Array (default: nix passiert = keine zusätzlichen Parameter akzeptieren)
    int set_fix_makergaul_c(int par_num, string par_value);                     //Schreiben von Fixparameter für MAKERGAUL_C (f_inhib)
    
    void makergaul(vector< mpf_class >& par_vec_pt);                            //MAKERGAUL als Fitnessfunktion 
    void makergaul_c(vector< mpf_class >& par_vec_pt);                          //MAKERGAUL_c als Fitnessfunktion 
    void mak2(vector< mpf_class >& par_vec_pt);                                 //MAK2 als Fitnessfunktion
    
};
#endif

