/* 
 * -- makergaulxx --
 * Version 1.0
 * 
 * Stellt die Funktionen der MAKERGAUL sowie der MAK2-Analyse als C++-Implementierung zur Verfügung
 * 
 * File:   main.cpp
 * Copyright 2013 Christoph Bultmann
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//Main-Header
#include "main.h"

/*
 * Hinweis zur Kompilierung: 
 * Das GCC-Compiling-Flag -O3 ist im Falle dieses Programms sehr effektiv und sollte im Rahmen der 
 * bestmöglichen Ausführungsgeschwindigkeit des Programms unbedingt gesetzt werden!
 */
       
int main(int arg, char* ary_arg[])                                                                                      //Beim Start von Main angegebene Parameter arg: Anzahl; ary_arg[]: array, das die Parameterstrings enthält
    { 
        if(arg < 2)                                                                                                     //Keine Parameter angegeben? Dann nichts machen!
            {
                cout <<"No mode selected. Please run the program with -h for help.\n";
                return(1);
            }  
        g_mode = ary_arg[1];                                                                                            //Modus des Programmstarts bestimmen
        if(g_mode == "-v")                                                                                              //-v: Programminformationen anzeigen
            {
                cl_info version;
                version.show_version();
                return(0);
            } 
        else if(g_mode == "-h")                                                                                         //-h: Hilfe öffnen                          
            {
                cl_info help;
                help.show_help();
                return(0);
            } 
        else if(g_mode == "-l")                                                                                         //-l: GPL Anzeigen 
            {
                cl_info license;
                license.show_license();
                return(0);
            } 

        else if(g_mode == "-run")                                                                                       //Keine Parameter angegeben? dann nur die GNU-License anzeigen
            {
                mpf_set_default_prec(2096);                                                                             //Präzision für die MP-Floats der MPIR-Bibliothek setzen
                mpfr_set_default_prec(2096);                                                                            //Präzision für die MP-Floats der MPFR-Bibliothek setzen

                mp_exp_t u;                                                                                             //Variable, um den Exponenten von MPF_Floats zu speichern

                if(arg < 3)                                                                                             //Fehlen bereits Teile der Header-Parameter? Direkt abbrechen!
                    {
                        cout << "ERR#No header values!;\n";
                        return(1);
                    }                                                                                                           
                g_head_pars = strtol(ary_arg[2],NULL,10);                                                               //Header_Pars: Anzahl der Ergebnissparameter 
                g_head_values = strtol(ary_arg[3],NULL,10);                                                             //Header_Values: Anzahl der Messwerte   
                g_head_fix = strtol(ary_arg[4],NULL,10);                                                                //Header_Fix: Anzahl der Fixen Ergebnissparameter (optional, -1 wenn nicht gewünscht) 

                if(g_head_pars == 0L || g_head_fix == 0L || g_head_values == 0L)                                        
                    {
                        cout << "ERR#Unsigned Int supposed in ary_arg[2], [3] or [4]" <<"!;\n";                         //Bei Fehlschlag (z.B. Float statt Int übergeben) mit Fehler abbrechen
                        return(1);                                                                                      //Auch diese Lösung ist nicht perfekt, da auch ein Fehler ausgegeben wird, wenn der Konvertierungswer 0 wäre. Da das aber bei keinem der ganzzahligen Parameter erlaubt ist, stellt diese Einschränkung kein Problem dar.
                    }                                                   
                
                if(g_head_fix < 0) {g_head_fix = 0;}                                                                    //Wenn keine Fixparameter übergeben werden sollen, Wert auf 0 setzen (Keine 0-Übergabe in Eingabe der Komandozeile, da hier Probleme auftreten können)
                
                if(arg != 15 + (g_head_pars-1) * 2 + g_head_values + g_head_fix + 1)                                    //Zu wenige oder zu viele Parameter übergeben? (14 Fixe Parameter + Grenzwerte (Ergebnissparameter - 1 wg.SSRE *2 wg. Oberem und Unterem) + Messwerte +  1 wg. erstem ary_arg[1] (nicht benutzt)
                    {
                        cout << "ERR#Wrong number of given header values!;\n";
                        return(1);
                    }


                g_head_loops = strtol(ary_arg[5],NULL,10);                                                              //Header_Loops: Anzahl der Versuche
                g_head_rounds = strtol(ary_arg[6],NULL,10);                                                             //Header_Rounds: Anzahl der Iterationsschritte
                g_head_wdh = strtol(ary_arg[14],NULL,10);                                                               //Header_Wdh: Nach wie vielen Iterationen auf den gleichen, oberhalb der geforderten Präzision gelegenen Wert wird abgebrochen?
                g_head_analysis = strtol(ary_arg[15],NULL,10);                                                          //Header_Analysis: Bestimmt, welches Analysemodell als Fitnessfunktion der Downhill-Simplex-Optimierung genutzt wird. 1 = MAKERGAUL; 2 = MAK2

                if(g_head_loops == 0L || g_head_rounds == 0L || g_head_wdh == 0L || g_head_analysis == 0L)
                    {
                        cout << "ERR#Unsigned Int supposed in ary_arg[3],[4],[12] or [13]!;\n";                         //Bei Fehlschlag (z.B. Float statt Int übergeben) mit Fehler abbrechen
                        return(1);
                    } 

                g_head_ref = ary_arg[7];                                                                                //Header_Reflexion: Paramter Downhill-Simplex-Reflexionsfaktor
                g_head_exp = ary_arg[8];                                                                                //Header_Expansion: Paramter Downhill-Simplex-Expansionsfaktor
                g_head_con = ary_arg[9];                                                                                //Header_Contraction: Paramter Downhill-Simplex-Kontraktionsfaktor
                g_head_step1 = ary_arg[10];                                                                              //Header_Step1: Erster Fitness-Kontrollwert nach 10% der Iterationen, der unterschritten werden muss, damit die Analyse weitergeht
                g_head_step2 = ary_arg[11];                                                                             //Header_Step2: Zweiter Fitness-Kontrollwert nach 50% der Iterationen, der unterschritten werden muss, damit die Analyse weitergeht
                g_head_step3 = ary_arg[12];                                                                             //Header_Step3: Dritter Fitness-Kontrollwert nach 75% der Iterationen, der unterschritten werden muss, damit die Analyse weitergeht
                g_head_prec = ary_arg[13];                                                                              //Header_Precision: Letzter Fitness-Kontrollwert; wird er unterschritten, ist die Analyse erfolgreich Abgeschlossen und das Programm wird beendet

                cout << "BEGIN: par_res#" << g_head_pars << ";";                                                        //Statusrückmeldung, mit wie vielen Ergebnissparametern, Punkten und Messwerten die Analyse begonnen wurde
                cout << "BEGIN: p_range#" << g_head_pars+1 << ";";
                cout << "BEGIN: m_values#" << g_head_values << ";";          
                cout << "BEGIN NUM: par_ref#" << g_head_ref.get_str(u) << ";";                                          //Statusrückmeldung, mit wie vielen Versuchen, Iterationsschritten und Wiederholungen die Analyse begonnen wurde
                cout << "BEGIN NUM: par_ex#" << g_head_exp.get_str(u) << ";";
                cout << "BEGIN NUM: par_con#" << g_head_con.get_str(u) << ";";       

                downhill_simplex.init(g_head_pars,g_head_pars,g_head_values,g_head_analysis);                           //Downhill-Simplex-Objekt initialisieren (Anzahl Ergebnissparameter, Anzahl Punkte = einer mehr als Ergebnissparameter, Anzahl Messwerte)

                if(downhill_simplex.set_analysis(g_head_analysis,g_head_pars) == false)                                 //Analysefunktion wählen und Adresse in Fitnesszeiger speichern. Wenn das nicht gelingt...                                                                                                   
                    {
                        cout << "ERR#Parameters do not fit analysis function!;\n";                                      //...mit Fehler abbrechen
                        return(1);
                    }

                unsigned int border_a = 16;                                                                             //Das 15. ary_arg-Feld ist per Definition der untere Grenzwert für den ersten Ergebnissparameter (Grenzwerte in ary_arg immer 2 hintereinander für jeden Ergebnissparameter)
                for(int i = 1; i < g_head_pars; i++)                                                                    //Grenzwerte für alle Ergebnissparameter(außer [0]=Fitness-Wert) setzen
                    {   
                        downhill_simplex.set_border(i,ary_arg[border_a],ary_arg[border_a+1]);
                        border_a += 2;
                    }
                for(int i = 0; i < g_head_values; i++)                                                                  //Messwerte einlesen (Start = border_a; Ende: Anzahl Messparameter
                    {   
                        downhill_simplex.set_m_value(i,ary_arg[border_a]);
                        border_a ++;
                    }
                if(g_head_fix > 0)                                                                                      //Fixwerte nur einlesen, wenn welche vorhanden sein sollen
                    {
                        for(int i = 0; i < g_head_fix; i++)                                                             //Fixwerte einlesen (Start = border_a; Ende: Anzahl Fixwerte (Nach Messwerten, da nur optionale Parameter)
                            {   
                                downhill_simplex.set_fix_value(i,ary_arg[border_a]);
                                border_a ++;
                            }
                    }
                downhill_simplex_run();                                                                                 //Nachdem alle Daten eingelesen und kontrolliert sind: Downhill-Simplex starten
            }
        else                                                                                                            //Falsche Parameter angegeben? Fehlermeldung anzeigen und beenden
            {
                cout << "Wrong mode parameter. Please run the program with -h for help.\n";
                return(1);
            } 
        return 0;
    }

int downhill_simplex_run()
    {         
        bool colliding_ref = false;                                                                                     //Speichert später auftretende Kollisionen des Reflexionspunktes mit den Parametergrenzwerten
        int test_time_1 = 250, test_time_2 = 500, test_time_3 = 750;                                                    //Anzahl Iterationsschritte bestimmen, nach denen die Prüfug stattfindet (Werden später über % der maximalen Iterationen gesetzt)
        
        test_time_1 = g_head_rounds / 100 * 10;                                                                         //10% der Maximaliterationen
        test_time_2 = g_head_rounds / 100 * 50;                                                                         //50% der Maximaliterationen
        test_time_3 = g_head_rounds / 100 * 75;                                                                         //75% der Maximaliterationen;
        
        for(int l = 0; l < g_head_loops; l++)                                                                           //Ebene 2: Iterationsschritte
            {
                downhill_simplex.loop(l);                                                                               //Loop eintragen
                downhill_simplex.p_range_to_result(0);                                                                  //Wenn Aktueller Rankpunkt 0 besser als Ergebniss: ersetzen
                downhill_simplex.out_p_result(); 
                
                if(downhill_simplex.compare_break(g_head_prec) == false)                                                //Wurde das gewünschte Ziel erreicht? (Rangpunkt <=  geforderte Precision)        
                    {
                        break;
                    }                                                 

                for(int i = 0; i < g_head_pars; i++)                                                                    //Downhill-Punkte mit Zufallswerten initialisieren
                    {   
                        downhill_simplex.seed_p_range(i);                                                               //Rangpunkt initialisieren
                    }

                for(int r = 0; r < g_head_rounds; r++)                                                                  //Ebene 1: Wiederholungen
                    {
                        cout << "STAT_LOOP#" << l << ";";
                        cout << "STAT_ROUND#" << r << ";";
                        downhill_simplex.sort_points();                                                                 //Rangpunkte sortieren
                        downhill_simplex.out_stat();                                                                    //Bisheriges Ergebniss und aktuell besten Punkt ausgeben
                        
                        if(r == test_time_1 && downhill_simplex.compare_break(g_head_step1) == true)                    //Erste Kontrolle: Nach 10% der Zyklen
                            {
                                break;
                            }
                        else if(r == test_time_2 && downhill_simplex.compare_break(g_head_step2) == true)               //Zweite Kontrolle: Nach 50% der Zyklen
                            {
                                break;
                            }
                        else if(r == test_time_3 && downhill_simplex.compare_break(g_head_step3) == true)               //Dritte Kontrolle: Nach 75% der Zyklen
                            {
                                break;
                            }  
                        
                        downhill_simplex.build_center();                                                                //Zentrum der Punkt (alle außer schlechtestem Rangpunkt) bestimmen
                        downhill_simplex.build_reflexion(g_head_ref);                                                   //Reflexionspunkt bestimmen (incl. Gültigkeitsprüfung und ggf. Korrektur)
                        colliding_ref = downhill_simplex.check_reflexion();                                             //Prüfen, ob Reflexionspunkt innerhalb der Grenzwerte liegt
                        
                        if(colliding_ref == false)                                                                      //Innerhalb der Grenzen?-->Reflexionsprüfung
                            {
                                if(downhill_simplex.compare_ref(0) == 1)                                                //Reflexionspunkt besser als bester Punkt? Expansion durchführen!
                                    {
                                        //cout << "TEST BESSER REFLEXION#end;";
                                        downhill_simplex.build_expansion(g_head_exp);                                   //Expansionspunkt bestimmen (incl. Gültigkeitsprüfung und ggf. Korrektur)
                                        if(downhill_simplex.compare_ex(0) == 1 && downhill_simplex.check_expansion() == false) //Expansionspunkt besser als bester Punkt? Dann schlechtesten Punkt ersetzen und neuen Durchlauf starten!
                                            {
                                                downhill_simplex.replace_with_ex(g_head_pars-1);                        //Schlechtester Punkt = letzter Rangpunkt (index: Ergebnissparameter-1)
                                                //cout << "TEST BESSER EXPANSION#end;";
                                                continue;                                                               //Iteration beendet, nächster Schritt
                                            }
                                        downhill_simplex.replace_with_ref(g_head_pars-1);                               //Ansonsten schlechtesten Punkt nur mit Reflexionspunkt ersetzen
                                        continue;
                                    }
                                if(downhill_simplex.compare_ref(g_head_pars-2) == 1)                                    //Reflexionspunkt besser als zweitschlechtester Punkt? 
                                    {
                                        //cout << "TEST BESSER REFLEXION#end;";
                                        downhill_simplex.replace_with_ref(g_head_pars-1);                               //Schlechtesten Punkt durch Reflexionspunkt ersetzen!
                                        continue;  
                                    }
                                if(downhill_simplex.compare_ref(g_head_pars-1) == 1) 
                                    {
                                        downhill_simplex.build_contraction(g_head_con,false);                           //Reflexionspunkt wenigstens besser als schlechtester Punkt? Kontraktion mit negativem Faktor durchführen!
                                    }  
                                else 
                                    {
                                        downhill_simplex.build_contraction(g_head_con,true);                            //Reflexionspunkt schlechter als schlechtester Punkt? Kontraktion mit positivem Faktor durchführen!
                                    }
                            }
                        else                                                                                            //Außhalb der Grenzen?-->Kontraktioprüfung
                            {
                                downhill_simplex.build_contraction(g_head_con,true);                                    //Reflexionspunkt schlechter als schlechtester Punkt? Kontraktion mit positivem Faktor durchführen!
                            }
                        
                        if(downhill_simplex.compare_con(g_head_pars-1) == 1 && downhill_simplex.check_contraction() == false) //Kontraktionspunkt besser als schlechtester Punkt? 
                            {
                                //cout << "TEST BETTER CONTRACTION#end;";
                                downhill_simplex.replace_with_con(g_head_pars-1);                                       //Schlechtesten Punkt durch Kontraktionspunkt ersetzen!
                                continue;
                            }
                        
                        if(downhill_simplex.check_contraction() == false){downhill_simplex.general_contraction();}      //Wenn nichts hilft: Generalkontraktion!
                        else
                            {
                                //cout << "TEST GENERAL CONTRACTION#end;";
                                downhill_simplex.replace_with_con(g_head_pars-1);                                       //Schlechtester Parameter wird im letzten Fall durch Kontraktionswert ersetzt, anschließend...
                                downhill_simplex.seed_wrong(g_head_pars-1);                                             //... werden alle Grenzwertverletzenden Parameter mit Zufallswerten möglichst innerhalb der Grenzwerte ersetzt
                            }
                    }
            }
        
        downhill_simplex.p_range_to_result(0);                                                                          //Ist der beste Rangpunkt aus dem letzten Zyklus besser als das aktuelle Ergebniss? Ersetzen!
        downhill_simplex.out_p_result();                                                                                //Ganz am Ende Ergebnisse ausgeben                                                                                                                                                              
        cout << "FINISHED#Program ends;";
        return(0);
    }




