<?php
/*
Copyright 2013 Christoph Bultmann

This file is part of the MAKERGAUL-package.

The MAKERGAUL-package is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

The MAKERGAUL-package is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with the MAKERGAUL-package.  If not, see <http://www.gnu.org/licenses/>.
*/

/*Klasse:MAK2 Auswertung*/
class cl_analysis_mak2
{
    //Die Mentexte werden als Konstanten definiert, damit eine Sprachanpassung schnell mglich ist
    const TEXT_DNA = "DNA";
    const TEXT_SSE = "SSE"; 
    const TEXT_K = "Constant";
    const TEXT_BASIS = "Baseline fluorescence";  
    const TEXT_NONE = "???";
    const TEXT_LOW = "< Detektion limit";
    const TEXT_NA = "n.a.";
    const TEXT_NOT_VALID = "not valid";
    //Texts that are used for the log window
    const LOG_WORKING = "                ";
    const LOG_SUCCESS = "reached";
    const LOG_FAILURE = "not reached";
    const LOG_START = "The beginning of the iteration,";   
    const LOG_DOWNHILL_PARAMETER = "Downhill-Simplex-Parameter: ";
    const LOG_TO = " to ";
    const LOG_INCLUDED_VALUES = "Included readings: ";
    const LOG_REFLEXION = "reflection: ";
    const LOG_EXPANSION = "expansion: ";
    const LOG_CONTRACTION = "contraction: ";
    const LOG_SHIFT = "shift: ";
    const LOG_BORDERS = "Limit parameters:";
    const LOG_BREAK_1 = "rst check after: ";
    const LOG_BREAK_2 = "Second check after: ";
    const LOG_BREAK_3 = "Third check after: ";
    const LOG_POINT = "point ";
    const LOG_WEIGHT = "Rank point ";
    const LOG_SORTING = "sorting...";
    const LOG_PROOF_BORDERS = "validation..";
    const LOG_AND = " and ";
    const LOG_GENERAL_CONTRACTION = "Contract all items!";
    const LOG_FIRST_RESULT = "Interim result-";
    //Die Kurztexte fr die einfache Logzeile
    const LOG_WELL = "Well";
    const LOG_LP = " Lp: ";
    const LOG_CYC = " Cyc: ";
    const LOG_BK = " Bk: ";
    const LOG_ACTUAL = "&nbsp&nbsp actual ";
    const LOG_BEST = "|&nbsp&nbsp&nbsp&nbsp best ";
    
    //Die Voreinstellungen fr das gewhlte Settings-Men (Hier Standart-Downhill-Simplex-Men)
    const DS_CLASS = "cl_settings_ds";
    const DS_NOCLASS = "No analysis module!";
    const DS_LP = "100";        //Wiederholungen
    const DS_IT = "100";        //Iterationen
    const DS_PC = "0.0009";     //Zielprzision
    const DS_T1 = "10";         //Testwert 1
    const DS_T2 = "10";         //Testwert 2
    const DS_T3 = "10";         //Testwert 3
    const DS_RF = "2";          //Reflexion
    const DS_EX = "2";          //Expansion
    const DS_CN = "0.5";        //Kontraktion
    const DS_RB = "2";          //Repeatbarriere (Schutz gegen Iterieren auf immer dasselbe Ergebniss)

    /*Globale Arrays: Parameter des Modells, werden von verschiedenen Funktionen verndert*/
    private $PCR_b = array(2);        /*Basisfluoreszens. [0] = Basisfluoreszens im aktuellen Durchgang, [1] = Basisfluoreszens im besten Durchgang*/
    private $PCR_k = array(2);        /*Reaktionskonstante. [0] = Reaktionskonstante im aktuellen Durchgang, [1] = Reaktionskonstante im besten Durchgang*/
    private $PCR_dna = array(2);      /*Ausgangs-DNA. [0] = Ausgangs-DNA im aktuellen Durchgang, [1] = Ausgangs-DNA im besten Durchgang. -->Wichtigster Parameter, soll am Ende ermittelt werden!*/
    /*Arrays fr Grenzwerte der Parameter start_dna, konstante und basis erstellen*/
    private $Spread_dna = array(2); 
    private $Spread_k = array(2);
    private $Spread_basis= array(2);
    /*Globale Arrays: Listen der nach Modell errechneten Fluoreszens und dazu korrespondierenden DNA-Werte */
    private $SYBR_List = array();    /*Fluoreszensliste, nach Modell ermittelt. Parameter des Modells werden so lange optimiert, bis errechnete Fluoreszenswerte mit gemessenen mglichst gut bereinstimmen*/
    private $DNA_List = array();     /*Mit Fluoreszensliste nach Modell korrespondierende DNA-Werte*/ 
    /*Globales Array: Speichert die in determine_ct durchgefhrten Bewertungen der Ausgangsmesswerte. Hier noch nicht als Array definiert, da die Lnge Variieren kann*/
    private $IncludedValues;     /*[0] = Array der eingeschlossenen Fluoreszenswerte, [1] = Maximalfluoreszens ALLER Messwerte, [2] = Basisfuoreszens (basierend auf Well(0)-Fluoreszens)*/ 
    /*Globale Arrays: Punkte des Simplex der Downhill-Simplex-Optimierung*/  
    /*Punktarray: [0] = start_dna(x); [1] = konstante(y); [2] = basis(z); [3] = Fitness-Wert(SSE); [4] = Bewertung (Anhand des Fitnesswerts, 1 = Bester Punkt bis 4 = schlechtester Punkt)*/
    private $Punkt = array(array(5),array(5),array(5),array(5)); 
    /*Punktarray sortiert nach Fitnesswert: [0] = start_dna(x); [1] = konstante(y); [2] = basis(z); [3] = Fitness-Wert(SSE); [4] = Punktnummer*/
    private $Rangpunkt = array(array(5),array(5),array(5),array(5)); 
    /*Punktarrays der Deformationsfunktion*/
    private $Zentrum = array(4);     /*Zentrumspunkt der besten Punkte: [0] = start_dna(x); [1] = konstante(y); [2] = basis(z); [3] = Fitness-Wert(SSE)*/
    private $Reflexion = array(4);   /*Reflexionspunkt des schlechtesten Punktes durch das Zentrum: [0] = start_dna(x); [1] = konstante(y); [2] = basis(z); [3] = Fitness-Wert(SSE)*/
    private $Expansion = array(4);   /*Expansion des schlechtesten Punktes durch das Zentrum: [0] = start_dna(x); [1] = konstante(y); [2] = basis(z); [3] = Fitness-Wert(SSE)*/
    private $Kontraktion = array(4); /*Reflexionspunkt des schlechtesten Punktes durch das Zentrum: [0] = start_dna(x); [1] = konstante(y); [2] = basis(z); [3] = Fitness-Wert(SSE)*/
    /*Array zum Speichern der in den Sektoren errechneten Werten*/
    private $Sektor = array(8); /*[x] = Sektor; [x][y] = Durchlauf; [x][y][0-3] = start_dna, konstante, basis, Fitness-Wert(SSE)*/
    /*Speicherplatz des Wells, dem das Ergebnis zugeordnet wird*/
    private $Session_Well = -1;
    /* Zwischenwerte aus den einzelnen Loops*/
    private $Loopwerte = array(100,100,100,10000,0);
    
    //Variable, die den Namen der gewnschten cl_com-Klasse enthlt (Wird bentigt, um Analysesoftware zu intigrieren, die in C++ geschrieben wurde)
    private $COM_EXEC = "cl_com_ds";
    private $Cxx_Used = FALSE;        //Variable, die speichert, welcher Analysemodus gewhlt wurde
    private $Cxx_Analysis;            //Speicherort fr das cl_com-Objekt
        
    
   //WICHTIG:Wird von jeder Analyseklasse bentigt, um sich beim Hauptprogramm zu registrieren.
    public static function firststart()
        {
            //Prfen, ob gewnschte Settings-Klasse vorhanden ist, und bei Anwesenheit laden
            if(requirement("Settings",self::DS_CLASS) === TRUE)
                {
                    //Static-Aufruf der Settings-Klasse, um Voreinstellungen festzulegen (ACHTUNG: Hierzu ist es ntig zu wissen welche und wie viele Parameter die cl_settings-Funktion verarbeitet sowie deren Reihenfolge! Vorher Klasse anschauen!)
                    $_SESSION["Module"]["Settings"]["Type"]::presets(self::DS_LP,self::DS_IT,self::DS_PC,self::DS_T1,self::DS_T2,self::DS_T3,self::DS_RF,self::DS_EX,self::DS_CN,self::DS_RB);
                    return; //Beenden
                }
        }
    //WICHTIG: In jeder Klasse ntig (Wird von cl_modulechange beim wechsel auf ein anderes Modul aufgerufen)
    public static function unload()
        {
        
            /*Lschen der vom Objekt eingefhrten $SESSION-Variablen*/
            /*Keine eigenen eingefhrt = keine zu lschen!*/ 
            return;
        }    
    //WICHTIG: Wird von jeder Analyseklasse bentigt. Aufruf erfolgt von cl_plate um den zu erstellenden Ergebnissarray der Platte an die Ausgabe des Analysemoduls anzupassen*/
    public static function result_pars()
        {
            $ergebniss = array(4);  //Wie viele Ergebnissparameter entstehen in jedem Lauf?
            //Achtung: immer min. 2 Parameter notwendig ( [0] und [1] )
            //Achtung: jeden Parameter in die Unterarrays ["ID"] und ["Value"] spalten (wird von cl_plate vorrausgesetzt)
            //Namen und Defaultwerte der Parameter
            $ergebniss[0] = array("ID"=>self::TEXT_DNA,"Value"=>self::TEXT_NONE);
            $ergebniss[1] = array("ID"=>self::TEXT_SSE,"Value"=>self::TEXT_NONE);
            $ergebniss[2] = array("ID"=>self::TEXT_K,"Value"=>self::TEXT_NONE);
            $ergebniss[3] = array("ID"=>self::TEXT_BASIS,"Value"=>self::TEXT_NONE);
            return($ergebniss);
        }
    //EMPFOHLEN: Die Rckgabe von Argebnissen an das Plattenobjekt sollte per __destruct erfolgen, damit beim Abbrechen der Analyse die bereits erreichten Werte gespeichert werden*/
    public function __destruct()
        {
            //Existierte gar kein Analyseobjekt (= Statischer Aufruf irgendeiner Funktion) --> nichts zurckgeben!
            if(!isset($this)) {return;} 
            //Sind Loopwerte gltig? Wegen weicher Grenzsetzung wichtig! (nur testen, wenn Werte im nachweisbaren Bereich liegen!)
            $test=0;
            if(!empty($this->IncludedValues) && $this->Cxx_Used === FALSE) //Hat berhaupt eine Analyse stattgefunden? Und wenn ja, lief sie ber die PHP-Implementierung?
                {
                    $test=$this->mak2_validvalue($this->Loopwerte);
                    if($test == 5 || $test == 6){$this->Loopwerte[2]=self::TEXT_NOT_VALID;$this->Loopwerte[0]=self::TEXT_NA;}      //Basisfluoreszens ungltig
                    elseif($test == 3 || $test == 4){$this->Loopwerte[1]=self::TEXT_NOT_VALID;$this->Loopwerte[0]=self::TEXT_NA;}  //Konstante ungltig (Hufiger Fehler bei zu niedrigen Messwerten = keine sinnvoll interpretierbare Messkurve)
                    elseif($test == 1 || $test == 2){$this->Loopwerte[0]=self::TEXT_NOT_VALID;}                             //DNA-Wert ungltig
                    
                    //Wenn Alles O.K. ist, Modellwerte ermitteln (Spetziell noch einmal Konstante berprfen, damit es nicht zu einem Div0-Fehler kommt 
                    if($this->Loopwerte[1] != 0){$_SESSION["Plate_Data"][$this->Session_Well]["Computed_Data"] = $this->mak2_fitness($this->Loopwerte[0],$this->Loopwerte[1],$this->Loopwerte[2],$this->IncludedValues[0],1);}
                
                    //Ergebnisse in Array schreiben  
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][0]["Value"] = $this->Loopwerte[0];
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][1]["Value"] = $this->Loopwerte[3];
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][2]["Value"] = $this->Loopwerte[1];
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][3]["Value"] = $this->Loopwerte[2];
                }
            //Wenn die Analyse ber c++-Implementierung abgewickelt wurde, cl_com-Objekt lschen
            if(!empty($this->Cxx_Analysis)){unset($this->Cxx_Analysis);}
        }
    //WICHTIG: Wird von jeder Analyseklasse bentigt. Aufruf erfolgt von cl_settings und enthlt die Parameter zur Analysesteuerung.*/    
    public function run_analysis($par_well, $par_messwerte, $par_reflect,$par_expand,$par_contract, $par_run, $par_loop, $par_precision="0.00009", $par_firststep="0.09", $par_secondstep="0.009", $par_thirdstep="0.0009",$par_change, $par_silent=0)
        {
            $this->Session_Well = $par_well; //Wellnummer speichern
            /*IncludedValues als Array benutzen, da verschiedene Werte zurckgegeben*/
            $this->IncludedValues = $this-> determine_ct($par_messwerte);
            
            /*Grenzwerte fr Parameter festlegen*/
            $border = array();
            $border[0] = array(0=>"0.000000000001",1=>"0.1");                                       //Start-DNA
            $border[1] = array(0=>"0.001",1=>$this->IncludedValues[1]);                             //Reaktionskonstante                                                     
            $border[2] = array(0=>"-".$this->IncludedValues[2],1=>$this->IncludedValues[2]);        //Basisfluoreszens

            //Prfen, ob c++ - Implementierung der Analyse vorhanden ist und die Vorprfung der einzubeziehenden Messwerte erfolgreich war
            if(class_exists($this->COM_EXEC) === TRUE && $this->IncludedValues[0][0] !== FALSE)
                {
                    $this->Cxx_Analysis = new $this->COM_EXEC;     //Analyseobjekt anlegen
                    //Analyse starten (Parameter 2 = 2 --> Mak2-Analyse wird ausgewhlt). Wenn c++- Analyse fehlschlgt...
                    $this->Cxx_Used = TRUE;
                    if($this->Cxx_Analysis->run_analysis($par_well,2,count($border)+1,$border,-1,$this->IncludedValues[0],$par_loop,$par_run,$par_reflect,$par_expand,$par_contract,$par_firststep,$par_secondstep,$par_thirdstep,$par_precision,$par_change,$par_silent) === FALSE)
                        {
                            //...Auf PHP-Analyse wechseln
                            $this->mak2_downhill_simplex($par_reflect, $par_expand, $par_contract, $border[0], $border[1], $border[2], $par_run, $par_loop, $par_precision, $par_firststep, $par_secondstep, $par_thirdstep, $par_change, $par_silent);
                            $this->Cxx_Used = FALSE;
                        }
                    unset($this->Cxx_Analysis);
                }
            //Ansonsten Analyse mit PHP-Implementierung
            else
                {
                   //Downhill-Simplex-Funktion mit angegebenen Parametern starten, das Ergebnis in Array speichern
                    $this->Cxx_Used = FALSE;
                    $this->mak2_downhill_simplex($par_reflect, $par_expand, $par_contract, $border[0], $border[1], $border[2], $par_run, $par_loop, $par_precision, $par_firststep, $par_secondstep, $par_thirdstep, $par_change, $par_silent);
                }
            
            /*Ergebniss mit angegebener Genauigkeit erreicht? Erfolg abspeichern*/
            if(bc_comp($this->Loopwerte[3],$par_precision) <= 0)
                {
                    if($par_silent > 0) 
                        {
                            echo(self::TEXT_SSE." ".self::LOG_SUCCESS.": ".$this->Loopwerte[3]);
                            echo("</br>");
                        }
                    $this->__destruct();
                }
            /*Ansonsten Misserfolg abspeichern*/    
            else
                {
                    if($par_silent > 0) 
                        {
                            echo(self::TEXT_SSE." ".self::LOG_FAILURE.": ".$this->Loopwerte[3]);
                            echo("</br>");
                        }
                    $this->__destruct();
                }
        } 
    
    /*Funktion zur Optimierung der 3 Funktionsparameter*/
    /* Es wird ein 3-dimensionales Simplex mit 4 Punkten aus zuflligen Startwerten gebildet (Dimensionen pro Simplexpunkt --> P[x,y,z]: die 3 Parameter der MAK2-Funktion --> P[start_dna,konstante,basis]*/
    /* Startwert wird zufllig festgelegt*/
    /* Die "Qualitt" eines jeden Punktes wird durch die Ermittlung der SSE fr eine MAK2-Funktion mit den Parametern des jeweiligen Simplexpunktes ermittelt*/
    /* Abhngig von der Qualitt der Punkte werden verschiedene geometrische Verformungen des Simplex durchgefhrt, bis er um das Optimum der MAK2-Funktion konvergiert*/
    /* Abhngig von der Qualitt der Punkte werden verschiedene geometrische Verformungen des Simplex durchgefhrt, bis er um das Optimum der MAK2-Funktion konvergiert*/
    /* Optimum = Werte fr start_dna, konstante und basis, bei denen mit MAK2-berechnete Fluoreszenswerte am wenigsten von den gemessenen Fluoreszenswerten abweichen (= kleinstmgliche SSE) */
    function mak2_downhill_simplex($par_reflect, $par_expand, $par_contract, $par_spread_dna, $par_spread_k, $par_spread_basis, $par_iter, $par_loop ,$par_precision, $par_firststep, $par_secondstep, $par_thirdstep,$par_change=2, $loud=0)
        {
            /*Keine Messwerte oberhalb der Basisfluoreszens? abbrechen!*/
            if($this->IncludedValues[0][0] === FALSE)
                {
                    $this->Loopwerte[0] = self::TEXT_LOW;
                    $this->Loopwerte[1] = self::TEXT_NA;
                    $this->Loopwerte[2] = self::TEXT_NA;
                    $this->Loopwerte[3] = self::TEXT_NA;
                    $this->Loopwerte[4] = -1;     //Flag auf "Nicht nachweisbar" setzen
                    @$_SESSION["Plate_Data"][$this->Session_Well][0] = $this->Loopwerte;
                    return($this->Loopwerte);
                }
            /*SSE des ersten Vergleichspunkts hochsetzen(technisch ntig fr die erste Iteration)*/
            $this->Rangpunkt[0][3] = 100000;
            /*Loop-Counter initialisieren*/
            $looptest = 1; 
            $loop_break = 0;  
            $change_break = 0;                
            /*Grenzwerte fr Parameter festlegen*/
            $this->Spread_dna[0] = $par_spread_dna[0];
            $this->Spread_dna[1] = $par_spread_dna[1];
            $this->Spread_k[0] = $par_spread_k[0];
            $this->Spread_k[1] = $par_spread_k[1];
            $this->Spread_basis[0] = $par_spread_basis[0];
            $this->Spread_basis[1] = $par_spread_basis[1];
            /*Testzeitpunkt der Stufen fr die Loop-Funktion der zu erreichende Fitness festlegen*/
            $iter_step1 = bc_div($par_iter,4);      /*Erste Stufe: Nach 25 Prozent der Durchgnge erster Loop-Test*/
            $iter_step2 = bc_div($par_iter,2);      /*Zweite Stufe: Nach 50 Prozent der Durchgngezweiter Loop-Test*/
            $iter_step3 = bc_mul($par_iter,"0.75");   /*Dritte Stufe: Nach 75 Prozent der Durchgnge dritter Loop-Test*/
            
            /*Grenzwerte fr die Loop-Funktion der zu erreichende Fitness festlegen*/
            $sse_step1 = $par_firststep;   /*Erste Stufe*/
            $sse_step2 = $par_secondstep;    /*Zweite Stufe*/
            $sse_step3 = $par_thirdstep;     /*Dritte Stufe*/
        
            if($loud > 1)
                {
                    echo ("</br>");
                    echo (self::LOG_START);
                    echo ("</br>");
                    echo (self::LOG_INCLUDED_VALUES.count($this->IncludedValues[0]));
                    echo ("</br>");
                    echo (self::LOG_DOWNHILL_PARAMETER);
                    echo ("</br>");
                    echo (self::LOG_REFLEXION.$par_reflect);
                    echo ("</br>");
                    echo (self::LOG_EXPANSION.$par_expand);
                    echo ("</br>");
                    echo (self::LOG_CONTRACTION.$par_contract);
                    echo ("</br>");
                    echo (self::LOG_BORDERS);
                    echo ("</br>");
                    echo (self::TEXT_DNA.": ".$this->Spread_dna[0].self::LOG_TO.$this->Spread_dna[1]);
                    echo ("</br>");
                    echo (self::TEXT_K.": ".$this->Spread_k[0].self::LOG_TO.$this->Spread_k[1]);
                    echo ("</br>");
                    echo (self::TEXT_BASIS.": ".$this->Spread_basis[0].self::LOG_TO.$this->Spread_basis[1]);
                    echo ("</br>");
                    echo (self::LOG_BREAK_1.$sse_step1);
                    echo ("</br>");
                    echo (self::LOG_BREAK_2.$sse_step2);
                    echo ("</br>");
                    echo (self::LOG_BREAK_3.$sse_step3);
                    echo ("</br>");
                    echo ("</br>");
                }     
            //Beginn der iterativen Optimierung
            for($iter=0; $iter < $par_iter; $iter++)
            {
                if($loud==0)
                    {
                        flush();   
                        echo(self::LOG_WORKING);
                        flush(); 
                    } 
                // Wenn zu viele Loops durchgefhrt oder zu oft der Aktuell beste Wert als Bestwert eines Zyklus erreicht wurde (= Wiederholungszhler zu hoch): abbrechen
                if($loop_break > $par_loop) {break;}
                //Erste Runde: Initialisierung durchfhren
                if($iter == 0) 
                    {
                        //Wenn der beste Wert des letzten Zyklus gut genug war, schon mal als Zwischenergebnis speichern und den Wiederholungszhler zurcksetzen
                        if(bc_comp($this->Rangpunkt[0][3],$this->Loopwerte[3])< 1)
                            { //Werte nahezu gleich? Change_break hochzhlen (Bricht Skript ab, wenn immmer wieder auf gleichen Wert optimiert wird)
                                if(bc_comp(bc_abs(bc_sub($this->Rangpunkt[0][3],$this->Loopwerte[3])),"0.000001")==-1)
                                    {
                                        $change_break++;
                                        if($change_break > $par_change) {break;}
                                    }
                                //Ansonsten Change_break wieder auf Null (neuer, besserer Wert wurde erreicht)
                                else
                                    {
                                        $change_break=0;
                                        $this->Loopwerte = $this->Rangpunkt[0];
                                    }
                            }
                        /*Startwerte fr start_dna, konstante und basis mit eingestelltem Zufallsintervall bestimmen*/
                        for($i=0; $i<4; $i++)
                        {
                            $this-> Punkt[$i][0] = bc_rand($this->Spread_dna[0],$this->Spread_dna[1]);              //Random-Startwert fr Start-DNA
                            $this-> Punkt[$i][1] = bc_rand($this->Spread_k[0],$this->Spread_k[1]);                  //Random-Startwert fr Konstante
                            $this-> Punkt[$i][2] = bc_rand($this->Spread_basis[0],$this->Spread_basis[1]);          //Random-Startwert fr Basisfluoreszens
                            //Fitness des Punktes auswerten
                            $this-> Punkt[$i][3] = $this->mak2_fitness($this-> Punkt[$i][0],$this-> Punkt[$i][1],$this-> Punkt[$i][2],$this->IncludedValues[0]);
        
                            
                            if($loud > 1)
                                {
                                    echo ("</br>");
                                    echo (self::LOG_POINT.$i." ".self::TEXT_DNA.": ");
                                    echo ($this->Punkt[$i][0]);
                                    echo ("</br>");
                                    echo (self::LOG_POINT.$i." ".self::TEXT_K.": ");
                                    echo ($this->Punkt[$i][1]);
                                    echo ("</br>");
                                    echo (self::LOG_POINT.$i." ".self::TEXT_BASIS.": ");
                                    echo ($this->Punkt[$i][2]);
                                    echo ("</br>");
                                    echo (self::LOG_POINT.$i." ".self::TEXT_SSE.": ");
                                    echo ($this->Punkt[$i][3]);
                                    echo ("</br>");
                                    echo ("</br>");
                                }  
                        }
                        /*Punkte sortieren (Anhand des erzielten Fitness-Wertes)*/
                        $this-> globalchanger_mak2_sorting($this->Punkt);
                    }  
                
                // Nach gegebener Zeit immer wieder Prfen, ob auf dem richtigen Weg, ansonsten neu beginnen
                if($iter == $iter_step1) 
                    {
                        if(bc_comp($this->Rangpunkt[0][3],$sse_step1) == 1)
                            {
                                $iter = -1;  
                                $loop_break++;
                            }
                    }
                if($iter == $iter_step2) 
                    {
                        if(bc_comp($this->Rangpunkt[0][3],$sse_step2) == 1)
                            {
                                $iter = -1;
                                $loop_break++;    
                            }
                    }
                if($iter == $iter_step3) 
                    {
                        if(bc_comp($this->Rangpunkt[0][3],$sse_step3) == 1)
                            {
                                $iter = -1;
                                $loop_break++;  
                            }
                    }
                if($iter == $par_iter-1) //letzte Kontrolle
                    {
                        if(bc_comp($this->Rangpunkt[0][3],$par_precision) == 1)
                            {
                                $iter = -1;
                                $loop_break++;  
                            }
                    }
                    
                if($loud > 0)
                    { 
                        echo(self::LOG_WELL."[".$this->Session_Well."]".self::LOG_LP.$loop_break.self::LOG_CYC.$iter.self::LOG_BK.$change_break.self::LOG_BEST.self::TEXT_SSE.": ".number_format($this->Loopwerte[3],7,'.','').self::LOG_ACTUAL.self::TEXT_SSE.": ".number_format($this->Rangpunkt[0][3],7,'.',''));                   
                        echo ("</br>");
                    }    
                if($loud > 1)
                    { 
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_SSE." ".$this->Rangpunkt[0][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_SSE." ".$this->Rangpunkt[1][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_SSE." ".$this->Rangpunkt[2][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_SSE." ".$this->Rangpunkt[3][3]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_SORTING);
                        echo ("</br>");
                    }
                /*Punkte sortieren (anhand des erzielten Fitnes-Wertes)*/
                $this-> globalchanger_mak2_sorting($this->Rangpunkt);
                /*Zentrum der besten Punkte (=alle bis auf schlechtesten) bestimmen*/
                $this-> globalchanger_mak2_zentrum();
                /*Spiegelpunkt des besten Punkts durch das Zentrum bestimmen*/
                $this-> globalchanger_mak2_reflexion($par_reflect);
                /*Reflexionspunkt im gltigen Bereich? In Variable Speichern (Performance-Erhhung)*/
                
                if($loud > 1)
                    {
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_DNA.": ".$this->Rangpunkt[0][0]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_DNA.": ".$this->Rangpunkt[1][0]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_DNA.": ".$this->Rangpunkt[2][0]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_DNA.": ".$this->Rangpunkt[3][0]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_K.": ".$this->Rangpunkt[0][1]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_K.": ".$this->Rangpunkt[1][1]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_K.": ".$this->Rangpunkt[2][1]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_K.": ".$this->Rangpunkt[3][1]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_BASIS.": ".$this->Rangpunkt[0][2]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_BASIS.": ".$this->Rangpunkt[1][2]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_BASIS.": ".$this->Rangpunkt[2][2]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_BASIS.": ".$this->Rangpunkt[3][2]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_SSE.": ".$this->Rangpunkt[0][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_SSE.": ".$this->Rangpunkt[1][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_SSE.": ".$this->Rangpunkt[2][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_SSE.": ".$this->Rangpunkt[3][3]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_REFLEXION.self::LOG_AND.self::LOG_EXPANSION." ".self::LOG_PROOF_BORDERS);
                        echo ("</br>");   
                    }
                $firsttest = $this->mak2_validvalue($this->Reflexion);
                /*Reflexionspunkt besser als bester Punkt & im gltigen Bereich?*/
                if((bc_comp($this->Reflexion[3],$this->Rangpunkt[0][3])==-1) && ($firsttest==9))
                    {
                        /*Dann Expansion durchfhren*/
                        $this-> globalchanger_mak2_expansion($par_expand);
                        /*Expansionspunkt besser als bester Punkt & im gltigen Bereich?*/
                        if((bc_comp($this->Expansion[3],$this->Rangpunkt[0][3])==-1) && ($this->mak2_validvalue($this->Expansion)==9))
                            {
                                //Dann den schlechtesten Punkt durch Expansionspunkt ersetzen
                                $this->Rangpunkt[3] = $this->Expansion;
                                
                                if($loud > 1)
                                    {
                                        echo ("</br>");
                                        echo (self::LOG_EXPANSION.self::TEXT_DNA.": ".$this->Rangpunkt[3][0]);
                                        echo ("</br>");
                                        echo (self::LOG_EXPANSION.self::TEXT_K.": ".$this->Rangpunkt[3][1]);
                                        echo ("</br>");
                                        echo (self::LOG_EXPANSION.self::TEXT_BASIS.": ".$this->Rangpunkt[3][2]);
                                        echo ("</br>");
                                        echo (self::LOG_EXPANSION.self::TEXT_SSE.": ".$this->Rangpunkt[3][3]);
                                        echo ("</br>");
                                    }
                                
                                continue;
                            }
                        /*Ansonsten den schlechtesten Punkt durch Reflexionspunkt ersetzen*/ 
                        else 
                            {
                                $this->Rangpunkt[3] = $this->Reflexion;
                                
                                if($loud > 1)
                                    {
                                        echo ("</br>");
                                        echo (self::LOG_REFLEXION.self::TEXT_DNA.": ".$this->Rangpunkt[3][0]);
                                        echo ("</br>");
                                        echo (self::LOG_REFLEXION.self::TEXT_K.": ".$this->Rangpunkt[3][1]);
                                        echo ("</br>");
                                        echo (self::LOG_REFLEXION.self::TEXT_BASIS.": ".$this->Rangpunkt[3][2]);
                                        echo ("</br>");
                                        echo (self::LOG_REFLEXION.self::TEXT_SSE.": ".$this->Rangpunkt[3][3]);
                                        echo ("</br>");
                                    }
                                    
                                continue;
                            }
                    }
                /*Reflexionspunkt besser als zweitschlechtster Punkt & im gltigen Bereich?*/
                else if((bc_comp($this->Reflexion[3],$this->Rangpunkt[2][3])==-1) && ($firsttest==9))
                    {
                        /*Dann schlechtesten Punkt durch Reflexionspunkt ersetzen*/
                        $this->Rangpunkt[3] = $this->Reflexion;
                        
                        if($loud > 1)
                            {        
                                echo ("</br>");
                                echo (self::LOG_REFLEXION.self::TEXT_DNA.": ".$this->Rangpunkt[3][0]);
                                echo ("</br>");
                                echo (self::LOG_REFLEXION.self::TEXT_K.": ".$this->Rangpunkt[3][1]);
                                echo ("</br>");
                                echo (self::LOG_REFLEXION.self::TEXT_BASIS.": ".$this->Rangpunkt[3][2]);
                                echo ("</br>");
                                echo (self::LOG_REFLEXION.self::TEXT_SSE.": ".$this->Rangpunkt[3][3]);
                                echo ("</br>");
                            }
                                
                        continue;
                    }
                /*Ansonsten Kontraktion durchfhren*/
                else 
                    {
                        /*schlechtester Punkt fitter oder gleich Reflexionspunkt?*/
                        if((bc_comp($this->Reflexion[3],$this->Rangpunkt[3][3]) >= 0)) {$this-> globalchanger_mak2_worse_kontraktion($par_contract);}
                        /*Reflexionspunkt fitter oder gleich schlechtester Punkt ? --> positives Vorzeichen fr Faktor behalten*/
                        else {$this-> globalchanger_mak2_best_kontraktion($par_contract);} 
                        
                        if($loud > 1)
                            {
                                echo ("</br>");
                                echo (self::LOG_CONTRACTION." ".self::LOG_PROOF_BORDERS);
                                echo ("</br>");
                            }
                            
                        /*Reflexionspunkt im gltigen Bereich? In Variable Speichern (Performance-Erhhung)*/
                        $contracttest = $this->mak2_validvalue($this->Kontraktion);
                        
                        /*Kontraktionspunkt im gltigen Bereich?*/  
                        if($contracttest == 9) 
                            {
                                /*Kontraktionspunkt besser als schlechtester Punkt?*/ 
                                if(bc_comp($this->Kontraktion[3],$this->Rangpunkt[3][3]) == -1)  
                                    {
                                        //Dann den schlechtesten Punkt durch Kontraktionspunkt ersetzen
                                        $this->Rangpunkt[3] = $this->Kontraktion;
                                        
                                        if($loud > 1)
                                            {
                                                echo ("</br>");
                                                echo (self::LOG_CONTRACTION.self::TEXT_DNA.": ".$this->Rangpunkt[3][0]);
                                                echo ("</br>");
                                                echo (self::LOG_CONTRACTION.self::TEXT_K.": ".$this->Rangpunkt[3][1]);
                                                echo ("</br>");
                                                echo (self::LOG_CONTRACTION.self::TEXT_BASIS.": ".$this->Rangpunkt[3][2]);
                                                echo ("</br>");
                                                echo (self::LOG_CONTRACTION.self::TEXT_SSE.": ".$this->Rangpunkt[3][3]);
                                                echo ("</br>");
                                            }
                                            
                                        continue;
                                    }
                                /*Ansonsten auf alle Punkte den Wert des besten Punktes addieren & durch 2 dividieren*/  
                                else 
                                    {
                                        for($i=1;$i<4;$i++)
                                            {
                                                for($y=1;$y<3;$y++)
                                                    {
                                                        //Generalkontraktion nur durchfhren, Wenn Punkte im gltigen Bereich bleiben
                                                        $zwischen = bc_div((bc_add($this->Rangpunkt[$i][$y],$this->Rangpunkt[0][$y])),"2");
                                                        /*if($this->mak2_validvalue($zwischen)==9)*/$this->Rangpunkt[$i][$y] = $zwischen;
                                                    }
                                                    /*Fitness bestimmen*/
                                                $this->Rangpunkt[$i][3] = $this->mak2_fitness($this->Rangpunkt[$i][0],$this->Rangpunkt[$i][1],$this->Rangpunkt[$i][2],$this->IncludedValues[0]);
                                            }
                                            
                                        if($loud > 1)
                                            {
                                                echo ("</br>");
                                                echo (self::LOG_GENERAL_CONTRACTION);
                                                echo ("</br>");
                                            }
                                                                      
                                        continue;
                                    }
                        
                            }
                        /*Ansonsten Punkt in den gltigen Bereich verschieben*/ 
                        else
                            {
                                //Start-DNA kleiner als unterer Grenzwert -->Rangpunkt[3] =  Rangpunkt[3] + ( |(Zentrum - Rangpunkt[3])| * 0,5) = Halbe Strecke auf Zentrum zubewegen
                                if($contracttest  == 1) 
                                    {
                                        $this->Rangpunkt[3][0] = bc_rand($this->Spread_dna[0],$this->Zentrum[0]);    
                                    } 
                                //Start-DNA grer als oberer Grenzwert -->Rangpunkt[3] =  Rangpunkt[3] + ( |(Zentrum - Rangpunkt[3])| * -0,5) = Halbe Strecke auf Zentrum zubewegen       
                                else if($contracttest  == 2) 
                                    {
                                        $this->Rangpunkt[3][0] = bc_rand($this->Zentrum[0],$this->Spread_dna[1]);   
                                    }   
                                //Konstante kleiner als unterer Grenzwert -->Rangpunkt[3] =  Rangpunkt[3] + ( |(Zentrum - Rangpunkt[3])| * 0,5) = Halbe Strecke auf Zentrum zubewegen
                                else if($contracttest  == 3) 
                                    {
                                        $this->Rangpunkt[3][1] = bc_rand($this->Spread_k[0],$this->Zentrum[1]);     
                                    } 
                                //Konstante grer als oberer Grenzwert -->Rangpunkt[3] =  Rangpunkt[3] + ( |(Zentrum - Rangpunkt[3])| * -0,5) = Halbe Strecke auf Zentrum zubewegen       
                                else if($contracttest  == 4) 
                                    {
                                        $this->Rangpunkt[3][1] = bc_rand($this->Zentrum[1],$this->Spread_k[1]);    
                                    }                      
                                //Basisfluoreszens kleiner als unterer Grenzwert -->Rangpunkt[3] =  Rangpunkt[3] + ( |(Zentrum - Rangpunkt[3])| * 0,5) = Halbe Strecke auf Zentrum zubewegen
                                else if($contracttest  == 5) 
                                    {
                                        $this->Rangpunkt[3][2] = bc_rand($this->Spread_basis[0],$this->Zentrum[2]);     
                                    } 
                                //Basisfluoreszens grer als oberer Grenzwert -->Rangpunkt[3] =  Rangpunkt[3] + ( |(Zentrum - Rangpunkt[3])| * -0,5) = Halbe Strecke auf Zentrum zubewegen       
                                else if($contracttest  == 6) 
                                    {
                                        $this->Rangpunkt[3][2] = bc_rand($this->Zentrum[2],$this->Spread_basis[1]);    
                                    } 
                                    
                                //Fitness des in einem Parameter vernderten Punktes bestimmen
                                $this->Rangpunkt[3][3] = $this->mak2_fitness($this->Rangpunkt[3][0],$this->Rangpunkt[3][1],$this->Rangpunkt[3][2],$this->IncludedValues[0]);
                                
                                if($loud > 1)
                                    {
                                        echo ("</br>");
                                        echo (self::LOG_SHIFT.self::TEXT_DNA.": ".$this->Rangpunkt[3][0]);
                                        echo ("</br>");
                                        echo (self::LOG_SHIFT.self::TEXT_K.": ".$this->Rangpunkt[3][1]);
                                        echo ("</br>");
                                        echo (self::LOG_SHIFT.self::TEXT_BASIS.": ".$this->Rangpunkt[3][2]);
                                        echo ("</br>");
                                        echo (self::LOG_SHIFT.self::TEXT_SSE.": ".$this->Rangpunkt[3][3]);
                                        echo ("</br>");
                                    }
                                    
                                continue;
                            }
                    }
            }
            
            /*Optimierungs-Ergebniss in Array schreiben*/
            //Wenn der letzte Wert besser als das Zwischenergebnis ist: Zwischenergebnis ersetzen
            if(bc_comp($this->Rangpunkt[0][3],$this->Loopwerte[3]) == -1)
                {
                    $this->Loopwerte = $this->Rangpunkt[0];
                }
            //sind Loopwerte gltig? Wegen weicher Grenzsetzung wichtig!
            $test=$this->mak2_validvalue($this->Loopwerte);
            if($test == 5 || $test == 6){$this->Loopwerte[2]=self::TEXT_NOT_VALID;$this->Loopwerte[0]=self::TEXT_NA;}      //Basisfluoreszens ungltig
            elseif($test == 3 || $test == 4){$this->Loopwerte[1]=self::TEXT_NOT_VALID;$this->Loopwerte[0]=self::TEXT_NA;}  //Konstante ungltig (Hufiger Fehler bei zu niedrigen Messwerten = keine sinnvoll interpretierbare Messkurve)
            elseif($test == 1 || $test == 2){$this->Loopwerte[0]=self::TEXT_NOT_VALID;}                             //DNA-Wert ungltig
            if($loud > 1)
                {
                    echo ("</br>");
                    echo (self::LOG_WEIGHT."0 ".self::TEXT_DNA.": ".$this->Rangpunkt[0][0]);
                    echo ("</br>");
                    echo (self::LOG_WEIGHT."0 ".self::TEXT_K.": ".$this->Rangpunkt[0][1]);
                    echo ("</br>");
                    echo (self::LOG_WEIGHT."0 ".self::TEXT_BASIS.": ".$this->Rangpunkt[0][2]);
                    echo ("</br>");
                    echo (self::LOG_WEIGHT."0 ".self::TEXT_SSE.": ".str_replace(".",",",$this->Rangpunkt[0][3]));
                    echo ("</br>");
                    echo (self::LOG_FIRST_RESULT.self::TEXT_SSE.": ".$this->Loopwerte[3]);
                    echo ("</br>");
                }
            //Objekt zerstren, damit die Ausgabe der Werte erfolgt
            $this->__destruct();
        } 
      
    /*Funktion zum Bestimmen des untersuchten Wertebereichs, da MAK 2-Modellierung nur gilt, bevor eine Sttigung der PCR-Polymerase eintritt
    --> Bestimmung der maximalen Steigung & nur diesen Wertebereich einschlieen*/
    function determine_ct($par)/*Funktion zum CT-Wert bestimmen nach MAK 2-Methode*/
        {
          /* Durch das Vorkommen groer Kommazahlen mssen Funktionen die Fluoreszenswerte mit bc()-Funktionen, beliebiger Genauigkeit, rechnen (macht den Code hsslich, lsst sich aber nicht vermeiden)*/
          /*Bis auf 8 Nachkommastellen*/
          bcscale(8);
          /*Umschreiben des Eingabearrays in sicheren Array*/
          $Well_SYBR_Werte = array(count($par));
          $Well_Zyklen = count($par);
          for($i=0; $i < count($par); $i++)
                 {
                    $Well_SYBR_Werte[$i] = htmlentities($par[$i], ENT_QUOTES);
                 } 
          /*1. Differenzen zwischen einzelnen Messwerten berechnen und in Array einlesen (1.Ableitung = Steigung)
          ( = Delta_SYBR_Werte, eins kleiner als SYBR-Werte, da es eine Differenz weniger als SYBR-Werte gibt :) )*/
          $Delta_SYBR_Werte = array($Well_Zyklen-1);
          for($i=1; $i < $Well_Zyklen; $i++)
                 {
                    $Delta_SYBR_Werte[$i-1] = bc_sub($Well_SYBR_Werte[$i],$Well_SYBR_Werte[$i-1]);
                 }
          /*2. Differenzen zwischen einzelnen Steigungswerten berechnen und in Array einlesen (2.Ableitung = Hhe der Steigung)
          (Wichtig, da nach der hchsten Steigung die lineare Phase der PCR beginnt --> diese Werte mssen zum Funktionieren des MAK2-Modells ausgeschlossen werden)*/
          $Delta_Delta_SYBR_Werte = array($Well_Zyklen-2);
          for($i=1; $i < $Well_Zyklen-1; $i++)
                 {
                    $Delta_Delta_SYBR_Werte[$i-1] = bc_sub($Delta_SYBR_Werte[$i],$Delta_SYBR_Werte[$i-1]);
                 }
          /*3. Summe der benachbarten Steigungshhen berechnen und in Array einlesen (Additon -> Gre des Unterschieds zwischen den Steigungshhen)
          (Unntiger Schritt, da eine Suche, die beim ersten gefundenen i = max(Delta_Delta_SYBR_Werte) abbricht, den selben Zweck der im Paper vorgestellten Routine erfllt )
          $Zwischen_Delta_Delta_SYBR_Werte = array($Well_Zyklen-3);
          for($i=1; $i < $Well_Zyklen-2; $i++)
                 {
                    $Zwischen_Delta_Delta_SYBR_Werte[$i-1] = ($Delta_Delta_SYBR_Werte[$i]+$Delta_Delta_SYBR_Werte[$i-1]);
                 }
          */       
          /*4. erstes Maximum der Steigungshhe berechnen = letzter in Analyse eingeschlossener Wert*/
          $Deltamax = bc_max($Delta_Delta_SYBR_Werte);
          $Cutoff = 0;
          for($i=0; $i < $Well_Zyklen-1; $i++)
                 {
                    if($Delta_Delta_SYBR_Werte[$i] == $Deltamax)
                        {
                             /*+1 da: erstes Array[i] = 0, aber erster Zyklus = 1*/ 
                             /*+2 da: mit jeder Differenzbildung und neuen Arraybefllung (2 * Differenzbildung = Delta_Delta_SYBR_Werte)tritt eine weitere "0 statt 1" = Linksverschiebung auf*/ 
                             $Cutoff=($i+3); 
                             break;
                        }
                 }
           /*5. Liste der in die Analyse eingeschlossenen Werte erstellen*/
           $Included_Values = array();
           for($i=0; $i < $Cutoff; $i++)
                 {
                    $Included_Values[$i] = $Well_SYBR_Werte[$i];
                 }
           /*6. Zustzlich hchsten Fluoreszenswert und Basisfluoreszens bestimmen und zurckgeben (wird spter zur einschrnkung der Optimierungsfunktion bentigt)*/
           $back=array(3);
           $back[0] = $Included_Values;
           $back[1] = bc_max($Well_SYBR_Werte); /*Maximalfluoreszens ALLER Messwerte-> als Einschrnkung fr konstante ntig*/
           $back[2] = bc_abs(bc_mul("1.9",bc_add(bc_abs($Well_SYBR_Werte[0]),0.0001))); /*Basisfluoreszens*/
           /*7. EIGENE GRENZSETZUNG: Wenn der hchste gemessene Fluoreszenswert inner-oder unterhalb des Basisfluoreszensbereiches liegt, werden keine Messwerte eingeschlossen (Keine trennung der PCR-Fluoreszens vom Rauschen mglich!)*/
           if(bc_comp(bc_max($Well_SYBR_Werte),$back[2]) <=0) {$back[0]=array(0=>FALSE);}
           return($back);
        }
    
    function mak2_fitness($par_start_dna, $par_konstante, $par_basis, $par_included_values, $par_all=0)/*Funktion fhrt Fluoreszens+DNA-Vermehrung nach den aktuellen Parametern des Modells durch und vergleicht diese mit den echten Werten. Resultat ist die Bildung der "sum of squared errors" (SSE).*/
        {
          bcscale(64);
          //Globale Arrays des aktuellen Durchlaufs mit angegebenen Parametern fllen. ACHTUNG: DIE BCMATH-FUNKTIONEN FUNKTIONIEREN NUR MIT STRINGS, DAHER KONVERTIERUNG NTIG!!!!
          $this->PCR_dna[0]= $par_start_dna;
          $this->PCR_k[0]= $par_konstante;
          $this->PCR_basis[0]= $par_basis;
          
          //DNA-Liste:erster Wert ist Ausgangs-DNA
          $this->DNA_List[0] = $this->PCR_dna[0];
               
          //MAK2-Modell zum Ausrechnen der aktuellen DNA
          /*Lnge der Listen der berechneten Fluoreszensen & DNA-Konzentrationen = Lnge der Liste der einbezogenen , als Referenz benutzten echten Fluoreszenswerten*/
          /*ACHTUNG: Fr das Array DNA-Liste ist i = 0 auch Zyklus 0! (Vorher: i = 0 war Zyklus 1)*/
          for($i=1; $i < (count($par_included_values)+1); $i++)
              {
                  $dna_last = $this->DNA_List[$i-1]; /*Vorheriger berechneter DNA-Wert aus Liste laden*/
                  /*Das eigentliche MAK2-Modell*/
                  /*Berechnet mittels Funktionen in beliebiger Genauigkeit, daher sehr verschachtelt & alle Daten als Strings!*/
                  $this->DNA_List[$i] = bc_add($dna_last,bc_mul($this->PCR_k[0],bc_ln1p(bc_div($dna_last,$this->PCR_k[0]))));
                  /*Entspricht --> $dna_now = $dna_last + $this->PCR_k[0] * Log(e,(1 + $dna_last / $this->PCR_k[0]));*/
                  $this->SYBR_List[$i-1] = bc_add($this->DNA_List[$i],$this->PCR_basis[0]); /*Fluoreszens = DNA + Basisfluoreszens) FRHESTENS NACH DEM 1.ZYKLUS*/
              }
           $sse = 0;
           
           /*echo(count($par_included_values)." ".count($this->SYBR_List)." ".$par_included_values[0]." ".$par_included_values[26]." ".$this->SYBR_List[25]);*/
           
           /*Bilden der Summe der Fehlerquadrate (SSE) als Ma fr die Abweichung zwischen Modell-Fluoreszenswerten und echten Fluoreszenswerten*/
           for($i=0; $i < (count($par_included_values)); $i++)
              {
                  $se_step = bc_pow(bc_sub($par_included_values[$i],$this->SYBR_List[$i]),2); 
                  /*Entspricht --> $sse_step = ($par_included_values[$i] - $this->SYBR_List[$i]) ^ 2;*/
                  $sse = bc_add($sse,$se_step);
                  /*SE zu aktueller SSE aufsummieren*/
              }
            /*Wenn gewnscht, die ermittelten Modellwerte zurckgeben*/
            if($par_all==1){return($this->SYBR_List);}
            return($sse);
        }
    
       
    function mak2_validvalue($par_testarray)/*Testet, ob der angegebene Wert innerhalb der gesetzten Grenzen liegt und gibt 9 (=TRUE) oder -1 bis 5 (FALSE) zurck*/
        {
            //Werte vergleichen ($array[0] = unterer Vergleich, $array[1] = oberer Vergleich)
            $x = array(2);
            $y = array(2);
            $z = array(2);
            $x[0] = bc_comp($par_testarray[0],$this->Spread_dna[0]);
            $y[0] = bc_comp($par_testarray[1],$this->Spread_k[0]);
            $z[0] = bc_comp($par_testarray[2],$this->Spread_basis[0]);
            $x[1] = bc_comp($par_testarray[0],$this->Spread_dna[1]);
            $y[1] = bc_comp($par_testarray[1],$this->Spread_k[1]);
            $z[1] = bc_comp($par_testarray[2],$this->Spread_basis[1]);
            //kleiner als unterer Grenzwert oder grer als oberer Grenzwert?-->Wert ungltig, mit Fehlernummer abbrechen
            if($x[0] == -1) {return(1);}             /*DNA-Wertebereich, Fehler = 1 oder 2*/
            else if($x[1] == 1) {return(2);}                        
            else if($y[0] == -1){return(3);}    /*Konstante-Wertebereich, Fehler = 3 oder 4*/
            else if($y[1] == 1) {return(4);}       
            else if($z[0] == -1){return(5);}        /*Basis-Wertebereich, Fehler = 5 oder 6*/
            else if($z[1] == 1) {return(6);}           
            return(9);
        }  
        
        function globalchanger_mak2_sorting($par_array)/*Funktion sortiert Punkte des downhill-Simplex in der absteigenden Reihenfolge ihrer Fitness*/
        {  
          /*Zwischenarray um die Fitnessbewertung in Verbindung zur Punktnummer zu speichern*/
          $punktnachfitness = array(array(2),array(2),array(2),array(2));
          //Punktarray die entsprechende Rangbewertung einspeichern
          for($i=0; $i<4; $i++)
            {
                //Nach Fitness sortierbares Array mit Punktnummern bilden. Achtung: bei sortierbaren Punktarrays muss der Unterarray [3] den Fitnesswert enthalten!
                $punktnachfitness[$i][0] = $par_array[$i][3];
                $punktnachfitness[$i][1] = $i;
            }
          //Neuen array sortieren (kleinste-->grte)
          sort($punktnachfitness);
          for($i=0; $i<4; $i++)
            {
               /* Ausgangsarray[$i][4] = Rangwert*/
               $par_array[$punktnachfitness[$i][1]][4] = $i;
               /*Nach Rangbewertung gebildetes Punktarray erstellen*/
               $this->Rangpunkt[$i][0] = $par_array[$punktnachfitness[$i][1]][0];
               $this->Rangpunkt[$i][1] = $par_array[$punktnachfitness[$i][1]][1];
               $this->Rangpunkt[$i][2] = $par_array[$punktnachfitness[$i][1]][2];
               $this->Rangpunkt[$i][3] = $par_array[$punktnachfitness[$i][1]][3];
               /*Rangpunkt[$i][4] = Nummer des Punktes (falls Rangpunkt = Ausgangsarray wird vorher in [$i][4] gesetzter Rangwert mit Punktnummer berschrieben!)*/
               $this->Rangpunkt[$i][4] = $punktnachfitness[$i][1];
            }
        }
     

     
    function globalchanger_mak2_zentrum()/*Funktion bildet Zentrumspunkt der besten Punkte*/
        {
            //Das Zentrum der besten Punkte bestimmen (= Mitelwert aller Punkte auer schlechtestem = Rangpunkt[0][0 bis 2] bis [2][0 bis 2])
            $this->Zentrum[0] = bc_div((bc_add($this->Rangpunkt[0][0],bc_add($this->Rangpunkt[1][0],$this->Rangpunkt[2][0]))),3);
            $this->Zentrum[1] = bc_div((bc_add($this->Rangpunkt[0][1],bc_add($this->Rangpunkt[1][1],$this->Rangpunkt[2][1]))),3);
            $this->Zentrum[2] = bc_div((bc_add($this->Rangpunkt[0][2],bc_add($this->Rangpunkt[1][2],$this->Rangpunkt[2][2]))),3);   
        }  
    //Die Verformungsfunktionen reflexion und expansion machen alle beide dasselbe, sind aber aus Performancegrnden  unter anderem Namen statt einmal mit if-else Kontrollfluss vorhanden
    function globalchanger_mak2_reflexion($par_faktor)/*Funktion bildet Reflexionspunkt des schlechtesten Punktes durch das Zentrum, mit $par_faktor zum genauen Spiegelpunkt*/
        {
            //Den Reflexionspunkt bestimmen (=schlechtester Punkt mit Faktor $par_faktor an Zentrumspunkt gespiegelt)
            $this->Reflexion[0] = bc_add($this->Zentrum[0],bc_mul($par_faktor,(bc_sub($this->Zentrum[0],$this->Rangpunkt[3][0]))));
            $this->Reflexion[1] = bc_add($this->Zentrum[1],bc_mul($par_faktor,(bc_sub($this->Zentrum[1],$this->Rangpunkt[3][1]))));
            $this->Reflexion[2] = bc_add($this->Zentrum[2],bc_mul($par_faktor,(bc_sub($this->Zentrum[2],$this->Rangpunkt[3][2]))));
            /*Fitness*/
            $this->Reflexion[3] = $this->mak2_fitness($this->Reflexion[0],$this->Reflexion[1],$this->Reflexion[2],$this->IncludedValues[0]);
        }   
    function globalchanger_mak2_expansion($par_faktor)/*Funktion bildet Expansionspunkt des schlechtesten Punktes durch das Zentrum, mit $par_faktor zum genauen Spiegelpunkt*/
        {
            //Den Expansionspunkt bestimmen (=schlechtester Punkt mit Faktor $par_faktor an Zentrumspunkt gespiegelt)
            $this->Expansion[0] = bc_add($this->Zentrum[0],bc_mul($par_faktor,(bc_sub($this->Zentrum[0],$this->Rangpunkt[3][0]))));
            $this->Expansion[1] = bc_add($this->Zentrum[1],bc_mul($par_faktor,(bc_sub($this->Zentrum[1],$this->Rangpunkt[3][1]))));
            $this->Expansion[2] = bc_add($this->Zentrum[2],bc_mul($par_faktor,(bc_sub($this->Zentrum[2],$this->Rangpunkt[3][2]))));
            /*Fitness*/
            $this->Expansion[3] = $this->mak2_fitness($this->Expansion[0],$this->Expansion[1],$this->Expansion[2],$this->IncludedValues[0]);
        }       
    function globalchanger_mak2_worse_kontraktion($par_faktor)/*Funktion bildet Kontraktionspunkt des schlechtesten Punktes durch das Zentrum, mit $par_faktor zum genauen Spiegelpunkt*/
        {
            //Den Kontraktionspunkt bestimmen (= Zentrumspunkt + $par_faktor * (schlechtester Punkt - Zentrumspunkt); Kontraktionspunkt liegt vom Zentrum in Richtung schlechtester Punkt)
            $this->Kontraktion[0] = bc_add($this->Zentrum[0],bc_mul($par_faktor,(bc_sub($this->Rangpunkt[3][0],$this->Zentrum[0]))));
            $this->Kontraktion[1] = bc_add($this->Zentrum[1],bc_mul($par_faktor,(bc_sub($this->Rangpunkt[3][1],$this->Zentrum[1]))));
            $this->Kontraktion[2] = bc_add($this->Zentrum[2],bc_mul($par_faktor,(bc_sub($this->Rangpunkt[3][2],$this->Zentrum[2]))));
            /*Fitness*/
            $this->Kontraktion[3] = $this->mak2_fitness($this->Kontraktion[0],$this->Kontraktion[1],$this->Kontraktion[2],$this->IncludedValues[0]);
        }   
    function globalchanger_mak2_best_kontraktion($par_faktor)/*Funktion bildet Kontraktionspunkt des schlechtesten Punktes durch das Zentrum, mit $par_faktor zum genauen Spiegelpunkt*/
        {
            //Den Kontraktionspunkt bestimmen (= Zentrumspunkt - $par_faktor * (schlechtester Punkt - Zentrumspunkt); Kontraktionspunkt liegt vom Zentrum in Richtung Reflexionspunkt)
            $this->Kontraktion[0] = bc_add($this->Zentrum[0],bc_mul("-".$par_faktor,(bc_sub($this->Rangpunkt[3][0],$this->Zentrum[0]))));
            $this->Kontraktion[1] = bc_add($this->Zentrum[1],bc_mul("-".$par_faktor,(bc_sub($this->Rangpunkt[3][1],$this->Zentrum[1]))));
            $this->Kontraktion[2] = bc_add($this->Zentrum[2],bc_mul("-".$par_faktor,(bc_sub($this->Rangpunkt[3][2],$this->Zentrum[2]))));
            /*Fitness*/
            $this->Kontraktion[3] = $this->mak2_fitness($this->Kontraktion[0],$this->Kontraktion[1],$this->Kontraktion[2],$this->IncludedValues[0]);
        }     
     
} 

?>

