<?php
/*
Copyright 2013 Christoph Bultmann

This file is part of the MAKERGAUL-package.

The MAKERGAUL-package is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

The MAKERGAUL-package is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with the MAKERGAUL-package.  If not, see <http://www.gnu.org/licenses/>.
*/

/*Klasse:MAKERGAUL Auswertung*/
class cl_analysis_MAKERGAUL
{
    //The menu texts are defined as constants, so that an adjustment in other language is fast possible
    const TEXT_DNA = "DNA";
    const TEXT_SSE = "SSRE"; 
    const TEXT_ENZYME = "free Polymerase";
    const TEXT_INHIB = "Inhibition factor";  
    const TEXT_BASIS = "Baseline fluorescence";  
    const TEXT_NONE = "???";
    const TEXT_LOW = "< Detection limit";
    const TEXT_NA = "n.a.";
    const TEXT_NOT_VALID = "not valid";
    //Texts that are used for the log window
    const LOG_WORKING = "                ";
    const LOG_SUCCESS = "reached";
    const LOG_FAILURE = "not reached";
    const LOG_START = "The beginning of the iteration,";   
    const LOG_DOWNHILL_PARAMETER = "Downhill-Simplex-Parameter: ";
    const LOG_TO = " to ";
    const LOG_INCLUDED_VALUES = "Included readings: ";
    const LOG_REFLEXION = "reflection: ";
    const LOG_EXPANSION = "expansion: ";
    const LOG_CONTRACTION = "contraction: ";
    const LOG_SHIFT = "shift: ";
    const LOG_BORDERS = "Limit parameters:";
    const LOG_BREAK_1 = "First check after: ";
    const LOG_BREAK_2 = "Second check after: ";
    const LOG_BREAK_3 = "Third check after: ";
    const LOG_POINT = "Point ";
    const LOG_WEIGHT = "Rank point ";
    const LOG_SORTING = "sorting...";
    const LOG_PROOF_BORDERS = "validation..";
    const LOG_AND = " and ";
    const LOG_GENERAL_CONTRACTION = "Contract all items!";
    const LOG_FIRST_RESULT = "Interim result-";
    //the short texts for the simple log line
    const LOG_WELL = "Well";
    const LOG_LP = " Lp: ";
    const LOG_CYC = " Cyc: ";
    const LOG_BK = " Bk: ";
    const LOG_ACTUAL = "&nbsp&nbsp actual ";
    const LOG_BEST = "|&nbsp&nbsp&nbsp&nbsp best ";
    
    //Die Voreinstellungen fr das gewhlte Settings-Men (Hier Standart-Downhill-Simplex-Men)
    const DS_CLASS = "cl_settings_ds";
    const DS_NOCLASS = "No analysis module!";
    const DS_LP = "300";        //Wiederholungen
    const DS_IT = "1500";       //Iterationen
    const DS_PC = "0.01";       //Zielprzision
    const DS_T1 = "10";         //Testwert 1
    const DS_T2 = "3";          //Testwert 2
    const DS_T3 = "0.2";        //Testwert 3
    const DS_RF = "2";          //Reflexion
    const DS_EX = "2";          //Expansion
    const DS_CN = "0.95";       //Kontraktion
    const DS_RB = "3";          //Repeatbarriere (Schutz gegen Iterieren auf immer dasselbe Ergebniss)
    
    /*Globale Arrays: Parameter des Modells, werden von verschiedenen Funktionen verndert*/
    private $PCR_b = array(2);        /*Basisfluoreszens. [0] = Basisfluoreszens im aktuellen Durchgang, [1] = Basisfluoreszens im besten Durchgang*/
    private $PCR_k = array(2);        /*Reaktionskonstante. [0] = Reaktionskonstante im aktuellen Durchgang, [1] = Reaktionskonstante im besten Durchgang*/
    private $PCR_dna = array(2);      /*Ausgangs-DNA. [0] = Ausgangs-DNA im aktuellen Durchgang, [1] = Ausgangs-DNA im besten Durchgang. -->Wichtigster Parameter, soll am Ende ermittelt werden!*/
    /*Arrays fr Grenzwerte der Parameter start_dna, taqmax, inhib und basis erstellen*/
    private $Spread_dna = array(2); 
    private $Spread_taqmax = array(2);
    private $Spread_inhib = array(2);
    private $Spread_basis= array(2);
    /*Globale Arrays: Listen der nach Modell errechneten Fluoreszens und dazu korrespondierenden DNA-Werte */
    private $SYBR_List = array();    /*Fluoreszensliste, nach Modell ermittelt. Parameter des Modells werden so lange optimiert, bis errechnete Fluoreszenswerte mit gemessenen mglichst gut bereinstimmen*/
    private $DNA_List = array();     /*Mit Fluoreszensliste nach Modell korrespondierende DNA-Werte*/ 
    private $Real_DNA_List = array();
    /*Globales Array: Speichert die in determine_ct durchgefhrten Bewertungen der Ausgangsmesswerte. Hier noch nicht als Array definiert, da die Lnge Variieren kann*/
    private $IncludedValues;     /*[0] = Array der eingeschlossenen Fluoreszenswerte, [1] = Maximalfluoreszens ALLER Messwerte, [2] = Basisfuoreszens (basierend auf Well(0)-Fluoreszens)*/ 
    /*Globale Arrays: Punkte des Simplex der Downhill-Simplex-Optimierung*/  
    /*Punktarray: [0] = start_dna(x); [1] = taqmax(y); [2] = hemmfaktor(z); [3] = basis(i); [4] = Fitness-Wert(SSE) [5] = Punktnummer*/
    private $Punkt = array(array(6),array(6),array(6),array(6),array(6)); 
    /*Punktarray sortiert nach Fitnesswert: [0] = start_dna(x); [1] = taqmax(y); [2] = hemmfaktor(z); [3] = basis(i); [4] = Fitness-Wert(SSE) [5] = Punktnummer*/
    private $Rangpunkt = array(array(6),array(6),array(6),array(6),array(6)); 
    /*Punktarrays der Deformationsfunktion*/
    private $Zentrum = array(5);     /*Zentrumspunkt der besten Punkte: [0] = start_dna(x); [1] = taqmax(y); [2] = hemmfaktor(z); [3] = basis(i); [4] = Fitness-Wert(SSE)*/
    private $Reflexion = array(5);   /*Reflexionspunkt des schlechtesten Punktes durch das Zentrum: [0] = start_dna(x); [1] = taqmax(y); [2] = hemmfaktor(z); [3] = basis(i); [4] = Fitness-Wert(SSE)*/
    private $Expansion = array(5);   /*Expansion des schlechtesten Punktes durch das Zentrum: [0] = start_dna(x); [1] = taqmax(y); [2] = hemmfaktor(z); [3] = basis(i); [4] = Fitness-Wert(SSE)*/
    private $Kontraktion = array(5); /*Reflexionspunkt des schlechtesten Punktes durch das Zentrum: [0] = start_dna(x); [1] = taqmax(y); [2] = hemmfaktor(z); [3] = basis(i); [4] = Fitness-Wert(SSE)*/
    
    /*Speicherplatz des Wells, dem das Ergebnis zugeordnet wird*/
    private $Session_Well = 0;
    /* Zwischenwerte aus den einzelnen Loops*/
    private $Loopwerte = array(100,100,100,100,100000);
    
    //Variable, die den Namen der gewnschten cl_com-Klasse enthlt (Wird bentigt, um Analysesoftware zu intigrieren, die in C++ geschrieben wurde)
    private $COM_EXEC = "cl_com_ds";
    private $Cxx_Used = FALSE;        //Variable, die speichert, welcher Analysemodus gewhlt wurde
    private $Cxx_Analysis;            //Speicherort fr das cl_com-Objekt
    
    
    //WICHTIG:Wird von jeder Analyseklasse bentigt, um sich beim Hauptprogramm zu registrieren.
    public static function firststart()
        {
            //Prfen, ob gewnschte Settings-Klasse vorhanden ist, und bei Anwesenheit laden
            if(requirement("Settings",self::DS_CLASS) === TRUE)
                {
                    //Static-Aufruf der Settings-Klasse, um Voreinstellungen festzulegen (ACHTUNG: Hierzu ist es ntig zu wissen welche und wie viele Parameter die cl_settings-Funktion verarbeitet sowie deren Reihenfolge! Vorher Klasse anschauen!)
                    $_SESSION["Module"]["Settings"]["Type"]::presets(self::DS_LP,self::DS_IT,self::DS_PC,self::DS_T1,self::DS_T2,self::DS_T3,self::DS_RF,self::DS_EX,self::DS_CN,self::DS_RB);
                    return; //Beenden
                }
        }
    //WICHTIG: In jeder Klasse ntig (Wird von cl_modulechange beim wechsel auf ein anderes Modul aufgerufen)
    public static function unload()
        {
            /*Lschen der vom Objekt eingefhrten $SESSION-Variablen*/
            /*Keine Eigenen eingefhrt = Keine zu lschen!*/ 
            return;
        }  
        
    //WICHTIG: Wird von jeder Analyseklasse bentigt. Aufruf erfolgt von cl_plate um den zu erstellenden Ergebnissarray der Platte an die Ausgabe des Analysemoduls anzupassen*/
    public static function result_pars()
        {
            $ergebniss = array(5);  //Wie viele Ergebnissparameter entstehen in jedem Lauf?
            //Achtung: immer min. 2 Parameter notwendig ( [0] und [1] )
            //Achtung: jeden Parameter in die Unterarrays ["ID"] und ["Value"] spalten (wird von cl_plate vorrausgesetzt)
            //Namen und Defaultwerte der Parameter
            $ergebniss[0] = array("ID"=>self::TEXT_DNA,"Value"=>self::TEXT_NONE);
            $ergebniss[1] = array("ID"=>self::TEXT_SSE,"Value"=>self::TEXT_NONE);
            $ergebniss[2] = array("ID"=>self::TEXT_ENZYME,"Value"=>self::TEXT_NONE);
            $ergebniss[3] = array("ID"=>self::TEXT_INHIB,"Value"=>self::TEXT_NONE);
            $ergebniss[4] = array("ID"=>self::TEXT_BASIS,"Value"=>self::TEXT_NONE);
            return($ergebniss);
        }
    //EMPFOHLEN: Die Rckgabe von Argebnissen an das Plattenobjekt sollte per __destruct erfolgen, damit beim Abbrechen der Analyse die bereits erreichten Werte gespeichert werden*/
    public function __destruct()
        {
            //Existierte gar kein Analyseobjekt (= Statischer Aufruf irgendeiner Funktion) --> nichts zurckgeben!
            if(!isset($this)) {return;} 
            //sind Loopwerte gltig? Wegen weicher Grenzsetzung wichtig!
            $test=0;
            if(!empty($this->IncludedValues) && $this->Cxx_Used === FALSE) //Hat berhaupt eine Analyse stattgefunden? Und wenn ja, lief sie ber die PHP-Implementierung?
                {
                    $test=@$this->makergaul_validvalue($this->Loopwerte); //Nur testen, wenn Objekt zur Analyse benutzt wurde und Zahlenwerte enthlt!
                    //Steht noch kein Endergebniss fest? Dann nur die Loopwerte eintragen 
                    if($test == 9 && intval($this->Loopwerte[1]) != 0){$this->makergaul_fitness($this->Loopwerte[0],$this->Loopwerte[1],$this->Loopwerte[2],$this->Loopwerte[3],$this->IncludedValues[0],0,1);}  
                    if($test == 7 || $test == 8){$this->Loopwerte[3]=self::TEXT_NOT_VALID;$this->Loopwerte[0]=self::TEXT_NA.$this->Loopwerte[0];} 
                    elseif($test == 5 || $test == 6){$this->Loopwerte[2]=self::TEXT_NOT_VALID;$this->Loopwerte[0]=self::TEXT_NA.$this->Loopwerte[0];}      //Basisfluoreszens ungltig
                    elseif($test == 3 || $test == 4){$this->Loopwerte[1]=self::TEXT_NOT_VALID;$this->Loopwerte[0]=self::TEXT_NA.$this->Loopwerte[0];}  //Konstante ungltig (Hufiger Fehler bei zu niedrigen Messwerten = keine sinnvoll interpretierbare Messkurve)
                    elseif($test == 1 || $test == 2){$this->Loopwerte[0]=self::TEXT_NOT_VALID;}                             //DNA-Wert ungltig
                    
                    //Wenn Alles O.K. ist, Modellwerte ermitteln (Spetziell noch einmal Konstante berprfen, damit es nicht zu einem Div0-Fehler kommt 
                    $_SESSION["Plate_Data"][$this->Session_Well]["Computed_Data"] = @$this->makergaul_fitness($this->Loopwerte[0],$this->Loopwerte[1],$this->Loopwerte[2],$this->Loopwerte[3],$this->IncludedValues[0],0,1);
                
                    //Ergebnisse in Array schreiben  
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][0]["Value"] = $this->Loopwerte[0];
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][1]["Value"] = $this->Loopwerte[4];
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][2]["Value"] = $this->Loopwerte[1];
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][3]["Value"] = $this->Loopwerte[2];
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][4]["Value"] = $this->Loopwerte[3];
                }
            //Wenn die Analyse ber c++-Implementierung abgewickelt wurde, cl_com-Objekt lschen
            if(!empty($this->Cxx_Analysis)){unset($this->Cxx_Analysis);}
        }
    //WICHTIG: Wird von jeder Analyseklasse bentigt. Aufruf erfolgt von cl_settings und enthlt die Parameter zur Analysesteuerung.*/    
    public function run_analysis($par_well, $par_messwerte, $par_reflect,$par_expand,$par_contract, $par_run, $par_loop, $par_precision="0.00009", $par_firststep="0.09", $par_secondstep="0.009", $par_thirdstep="0.0009",$par_change, $par_silent=0)
        {
            $this->Session_Well = $par_well; //Wellnummer speichern
            /*IncludedValues als Array benutzen, da verschiedene Werte zurckgegeben*/
            $this->IncludedValues = $this-> determine_ct($par_messwerte);
            if($this->IncludedValues[0][0] === FALSE) //Werte nicht bestimmbar
                {
                    //Abbruch in Array schreiben  
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][0]["Value"] = self::TEXT_LOW;
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][1]["Value"] = self::TEXT_NA;
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][2]["Value"] = self::TEXT_NA;
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][3]["Value"] = self::TEXT_NA;
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][4]["Value"] = self::TEXT_NA;
                    return(1);
                }
            /*Grenzwerte fr Parameter festlegen*/
            $border = array();
            $border[0] = array(0=>"0.00000000000001",1=>"0.1");                                               //Start-DNA
            $border[1] = array(0=>"0.0001",1=>bc_mul($this->IncludedValues[0][count($this->IncludedValues[0])-1],"2"));   //Polymerase kann nicht hher als der hchste Fluoreszenswert sein"
            $border[2] = array(0=>"0.0009",1=>"1");                                                          //Inhibitionskonstante
            $border[3] = array(0=>"-".$this->IncludedValues[2],1=>$this->IncludedValues[2]);                  //Basisfluoreszens

            //Prfen, ob c++ - Implementierung der Analyse vorhanden ist und die Vorprfung der einzubeziehenden Messwerte erfolgreich war
            if(class_exists($this->COM_EXEC) === TRUE && $this->IncludedValues[0][0] !== FALSE)
                {
                    $this->Cxx_Analysis = new $this->COM_EXEC;     //Analyseobjekt anlegen
                    //Analyse starten (Parameter 2 = 1 --> Makergaul-Analyse wird ausgewhlt). Wenn c++- Analyse fehlschlgt...
                    $this->Cxx_Used = TRUE;
                    if($this->Cxx_Analysis->run_analysis($par_well,1,count($border)+1,$border,-1,$this->IncludedValues[0],$par_loop,$par_run,$par_reflect,$par_expand,$par_contract,$par_firststep,$par_secondstep,$par_thirdstep,$par_precision,$par_change, $par_silent) === FALSE)
                        {
                            //...Auf PHP-Analyse wechseln
                            $this->makergaul_downhill_simplex($par_reflect, $par_expand, $par_contract, $border[0], $border[1], $border[2], $border[3], $par_run, $par_loop, $par_precision, $par_firststep, $par_secondstep, $par_thirdstep, $par_change, $par_silent);
                            $this->Cxx_Used = FALSE;
                        }
                    unset($this->Cxx_Analysis);
                }
            //Ansonsten Analyse mit PHP-Implementierung
            else
                {
                   //Downhill-Simplex-Funktion mit angegebenen Parametern starten, das Ergebnis in Array speichern
                    $this->Cxx_Used = FALSE;
                    $this->makergaul_downhill_simplex($par_reflect, $par_expand, $par_contract, $border[0], $border[1], $border[2], $border[3], $par_run, $par_loop, $par_precision, $par_firststep, $par_secondstep, $par_thirdstep, $par_change, $par_silent);
                }
                
            /*Ergebniss mit angegebener Genauigkeit erreicht? Erfolg abspeichern*/
            if(bc_comp($this->Loopwerte[3],$par_precision) <= 0)
                {
                    if($par_silent > 0) 
                        {
                            echo(self::TEXT_SSE." ".self::LOG_SUCCESS.": ".$this->Loopwerte[3]);
                            echo("</br>");
                        }
                    $this->__destruct();
                }
            /*Ansonsten Misserfolg abspeichern*/    
            else
                {
                    if($par_silent > 0) 
                        {
                            echo(self::TEXT_SSE." ".self::LOG_FAILURE.": ".$this->Loopwerte[3]);
                            echo("</br>");
                        }
                    $this->__destruct();
                }
        
        } 
     
    /*Funktion zur Optimierung der 4 Funktionsparameter*/
    /* Es wird ein 4-dimensionales Simplex mit 5 Punkten aus zuflligen Startwerten gebildet (Dimensionen pro Simplexpunkt --> P[x,y,z,i]: die 4 Parameter der MAKERGAUL-Funktion --> P[start_dna,polymerase,inhibitionskonstante,basis]*/
    /* Startwert wird zufllig festgelegt*/
    /* Die "Qualitt" eines jeden Punktes wird durch die Ermittlung der SSE fr eine MAKERGAUL-Funktion mit den Parametern des jeweiligen Simplexpunktes ermittelt*/
    /* Abhngig von der Qualitt der Punkte werden verschiedene geometrische Verformungen des Simplex durchgefhrt, bis er um das Optimum der MAKERGAUL-Funktion konvergiert*/
    /* Abhngig von der Qualitt der Punkte werden verschiedene geometrische Verformungen des Simplex durchgefhrt, bis er um das Optimum der MAKERGAUL-Funktion konvergiert*/
    /* Optimum = Werte fr start_dna, polymerase, inhibitionskonstante und basis, bei denen mit MAKERGAUL-berechnete Fluoreszenswerte am wenigsten von den gemessenen Fluoreszenswerten abweichen (= kleinstmgliche SSE) */
    function makergaul_downhill_simplex($par_reflect, $par_expand, $par_contract, $par_spread_dna, $par_spread_taqmax, $par_spread_inhib, $par_spread_basis, $par_iter, $par_loop ,$par_precision, $par_firststep, $par_secondstep, $par_thirdstep,$par_change=2, $loud=0)

        {
            /*Keine Messwerte oberhalb der Basisfluoreszens? abbrechen!*/
            if($this->IncludedValues[0][0]===FALSE)
                {
                    $this->Loopwerte[0] = self::TEXT_LOW;
                    $this->Loopwerte[1] = self::TEXT_NA;
                    $this->Loopwerte[2] = self::TEXT_NA;
                    $this->Loopwerte[3] = self::TEXT_NA;
                    $this->Loopwerte[4] = self::TEXT_NA;
                    @$_SESSION["Plattenwerte"][$this->Session_Well][0] = $this->Loopwerte;
                    return($this->Loopwerte);
                }
            /*SSE des ersten Vergleichspunkts hochsetzen(technisch ntig fr die erste Iteration)*/
            $this->Rangpunkt[0][4] = 100000;
            /*Loop-Counter initialisieren*/
            $looptest = 1; 
            $loop_break = 0;  
            $change_break = 0;                
            /*Grenzwerte fr Parameter festlegen*/
            $this->Spread_dna[0] = $par_spread_dna[0];
            $this->Spread_dna[1] = $par_spread_dna[1];
            $this->Spread_taqmax[0] = $par_spread_taqmax[0];
            $this->Spread_taqmax[1] = $par_spread_taqmax[1];
            $this->Spread_inhib[0] = $par_spread_inhib[0];
            $this->Spread_inhib[1] = $par_spread_inhib[1];
            $this->Spread_basis[0] = $par_spread_basis[0];
            $this->Spread_basis[1] = $par_spread_basis[1];
            /*Testzeitpunkt der Stufen fr die Loop-Funktion der zu erreichende Fitness festlegen*/
            $iter_step1 = bc_div($par_iter,4);      /*Erste Stufe: Nach 25 Prozent der Durchgnge erster Loop-Test*/
            $iter_step2 = bc_div($par_iter,2);      /*Zweite Stufe: Nach 50 Prozent der Durchgngezweiter Loop-Test*/
            $iter_step3 = bc_mul($par_iter,"0.75");   /*Dritte Stufe: Nach 75 Prozent der Durchgnge dritter Loop-Test*/
            
            /*Grenzwerte fr die Loop-Funktion der zu erreichende Fitness festlegen*/
            $sse_step1 = $par_firststep;   /*Erste Stufe*/
            $sse_step2 = $par_secondstep;    /*Zweite Stufe*/
            $sse_step3 = $par_thirdstep;     /*Dritte Stufe*/
            //Statusinformation zum Beginn der Optimierung
            if($loud > 1)
                {
                    echo ("</br>");
                    echo (self::LOG_START);
                    echo ("</br>");
                    echo (self::LOG_INCLUDED_VALUES.count($this->IncludedValues[0]));
                    echo ("</br>");
                    echo (self::LOG_DOWNHILL_PARAMETER);
                    echo ("</br>");
                    echo (self::LOG_REFLEXION.$par_reflect);
                    echo ("</br>");
                    echo (self::LOG_EXPANSION.$par_expand);
                    echo ("</br>");
                    echo (self::LOG_CONTRACTION.$par_contract);
                    echo ("</br>");
                    echo (self::LOG_BORDERS);
                    echo ("</br>");
                    echo (self::TEXT_DNA.": ".$this->Spread_dna[0].self::LOG_TO.$this->Spread_dna[1]);
                    echo ("</br>");
                    echo (self::TEXT_ENZYME.": ".$this->Spread_taqmax[0].self::LOG_TO.$this->Spread_taqmax[1]);
                    echo ("</br>");
                    echo (self::TEXT_INHIB.": ".$this->Spread_inhib[0].self::LOG_TO.$this->Spread_inhib[1]);
                    echo ("</br>");
                    echo (self::TEXT_BASIS.": ".$this->Spread_basis[0].self::LOG_TO.$this->Spread_basis[1]);
                    echo ("</br>");
                    echo (self::LOG_BREAK_1.$sse_step1);
                    echo ("</br>");
                    echo (self::LOG_BREAK_2.$sse_step2);
                    echo ("</br>");
                    echo (self::LOG_BREAK_3.$sse_step3);
                    echo ("</br>");
                    echo ("</br>");
                }        
            
            //Beginn der iterativen Optimierung
            for($iter=0; $iter < $par_iter; $iter++)
            {
                if($loud==0)
                    {
                        flush();   
                        echo(self::LOG_WORKING);
                        flush(); 
                    } 
                // Wenn zu viele Loops durchgefhrt oder zu oft der Aktuell beste Wert als Bestwert eines Zyklus erreicht wurde (= Wiederholungszhler zu hoch): abbrechen
                if($loop_break > $par_loop) {break;}
                //Erste Runde: Initialisierung durchfhren
                if($iter == 0) 
                    {
                        //Wenn der beste Wert des letzten Zyklus gut genug war, schon mal als Zwischenergebnis speichern und den Wiederholungszhler zurcksetzen
                        if(bc_comp($this->Rangpunkt[0][4],$this->Loopwerte[4])< 1)
                            { //Werte nahezu gleich? Change_break hochzhlen (Bricht Skript ab, wenn immmer wieder auf gleichen Wert optimiert wird)
                                if(bc_comp(bc_abs(bc_sub($this->Rangpunkt[0][4],$this->Loopwerte[4])),"0.000001")==-1)
                                    {
                                        $change_break++;
                                        if($change_break > $par_change) {break;}
                                    }
                                //Ansonsten Change_break wieder auf Null (neuer, besserer Wert wurde erreicht)
                                else
                                    {
                                        $change_break=0;
                                        $this->Loopwerte = $this->Rangpunkt[0];
                                    }
                            }
                        /*Startwerte fr start_dna, konstante und basis mit eingestelltem Zufallsintervall bestimmen*/
                        for($i=0; $i<5; $i++)
                        {
                            $this-> Punkt[$i][0] = bc_rand($this->Spread_dna[0],$this->Spread_dna[1]);              //Random-Startwert fr Start-DNA
                            $this-> Punkt[$i][1] = bc_rand($this->Spread_taqmax[0],$this->Spread_taqmax[1]);                  //Random-Startwert fr Taqmax
                            $this-> Punkt[$i][2] = bc_rand($this->Spread_inhib[0],$this->Spread_inhib[1]);                  //Random-Startwert fr Inhib
                            $this-> Punkt[$i][3] = bc_rand($this->Spread_basis[0],$this->Spread_basis[1]);          //Random-Startwert fr Basisfluoreszens
                            //Fitness des Punktes auswerten
                            $this-> Punkt[$i][4] = $this->makergaul_fitness($this-> Punkt[$i][0],$this-> Punkt[$i][1],$this-> Punkt[$i][2],$this-> Punkt[$i][3],$this->IncludedValues[0]);
        
                            if($loud > 1)
                                {
                                    echo ("</br>");
                                    echo (self::LOG_POINT.$i." ".self::TEXT_DNA.": ");
                                    echo ($this->Punkt[$i][0]);
                                    echo ("</br>");
                                    echo (self::LOG_POINT.$i." ".self::TEXT_ENZYME.": ");
                                    echo ($this->Punkt[$i][1]);
                                    echo ("</br>");
                                    echo (self::LOG_POINT.$i." ".self::TEXT_INHIB.": ");
                                    echo ($this->Punkt[$i][2]);
                                    echo ("</br>");
                                    echo (self::LOG_POINT.$i." ".self::TEXT_BASIS.": ");
                                    echo ($this->Punkt[$i][3]);
                                    echo ("</br>");
                                    echo (self::LOG_POINT.$i." ".self::TEXT_SSE.": ");
                                    echo ($this->Punkt[$i][4]);
                                    echo ("</br>");
                                    echo ("</br>");
                                }  
                        }
                        /*Punkte sortieren (Anhand des erzielten Fitness-Wertes)*/
                        $this-> globalchanger_makergaul_sorting($this->Punkt);
                    }  
                
                // Nach gegebener Zeit immer wieder Prfen, ob auf dem richtigen Weg, ansonsten neu beginnen
                if($iter == $iter_step1) 
                    {
                        if(bc_comp($this->Rangpunkt[0][4],$sse_step1) == 1)
                            {
                                $iter = -1;  
                                $loop_break++;
                            }
                    }
                if($iter == $iter_step2) 
                    {
                        if(bc_comp($this->Rangpunkt[0][4],$sse_step2) == 1)
                            {
                                $iter = -1;
                                $loop_break++;    
                            }
                    }
                if($iter == $iter_step3) 
                    {
                        if(bc_comp($this->Rangpunkt[0][4],$sse_step3) == 1)
                            {
                                $iter = -1;
                                $loop_break++;  
                            }
                    }
                if($iter == $par_iter-1) //letzte Kontrolle
                    {
                        if(bc_comp($this->Rangpunkt[0][4],$par_precision) == 1)
                            {
                                $iter = -1;
                                $loop_break++;  
                            }
                    }
                    
                if($loud > 0)
                    { 
                        echo(self::LOG_WELL."[".$this->Session_Well."]".self::LOG_LP.$loop_break.self::LOG_CYC.$iter.self::LOG_BK.$change_break.self::LOG_BEST.self::TEXT_SSE.": ".number_format($this->Loopwerte[4],7,'.','').self::LOG_ACTUAL.self::TEXT_SSE.": ".number_format($this->Rangpunkt[0][4],7,'.',''));                   
                        echo ("</br>");
                    }    
                if($loud > 1)
                    { 
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_SSE." ".$this->Rangpunkt[0][4]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_SSE." ".$this->Rangpunkt[1][4]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_SSE." ".$this->Rangpunkt[2][4]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_SSE." ".$this->Rangpunkt[3][4]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."4 ".self::TEXT_SSE." ".$this->Rangpunkt[4][4]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_SORTING);
                        echo ("</br>");
                    }
                /*Punkte sortieren (anhand des erzielten Fitnes-Wertes)*/
                $this-> globalchanger_makergaul_sorting($this->Rangpunkt);
                /*Zentrum der besten Punkte (=alle bis auf schlechtesten) bestimmen*/
                $this-> globalchanger_makergaul_zentrum();
                /*Spiegelpunkt des besten Punkts durch das Zentrum bestimmen*/
                $this-> globalchanger_makergaul_reflexion($par_reflect);
                /*Reflexionspunkt im gltigen Bereich? In Variable Speichern (Performance-Erhhung)*/
                
                if($loud > 1)
                    {
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_DNA.": ".$this->Rangpunkt[0][0]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_DNA.": ".$this->Rangpunkt[1][0]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_DNA.": ".$this->Rangpunkt[2][0]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_DNA.": ".$this->Rangpunkt[3][0]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."4 ".self::TEXT_DNA.": ".$this->Rangpunkt[4][0]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_ENZYME.": ".$this->Rangpunkt[0][1]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_ENZYME.": ".$this->Rangpunkt[1][1]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_ENZYME.": ".$this->Rangpunkt[2][1]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_ENZYME.": ".$this->Rangpunkt[3][1]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."4 ".self::TEXT_ENZYME.": ".$this->Rangpunkt[4][1]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_INHIB.": ".$this->Rangpunkt[0][2]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_INHIB.": ".$this->Rangpunkt[1][2]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_INHIB.": ".$this->Rangpunkt[2][2]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_INHIB.": ".$this->Rangpunkt[3][2]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."4 ".self::TEXT_INHIB.": ".$this->Rangpunkt[4][2]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_BASIS.": ".$this->Rangpunkt[0][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_BASIS.": ".$this->Rangpunkt[1][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_BASIS.": ".$this->Rangpunkt[2][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_BASIS.": ".$this->Rangpunkt[3][3]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."4 ".self::TEXT_BASIS.": ".$this->Rangpunkt[4][3]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."0 ".self::TEXT_SSE.": ".$this->Rangpunkt[0][4]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."1 ".self::TEXT_SSE.": ".$this->Rangpunkt[1][4]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."2 ".self::TEXT_SSE.": ".$this->Rangpunkt[2][4]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."3 ".self::TEXT_SSE.": ".$this->Rangpunkt[3][4]);
                        echo ("</br>");
                        echo (self::LOG_WEIGHT."4 ".self::TEXT_SSE.": ".$this->Rangpunkt[4][4]);
                        echo ("</br>");
                        echo ("</br>");
                        echo (self::LOG_REFLEXION.self::LOG_AND.self::LOG_EXPANSION." ".self::LOG_PROOF_BORDERS);
                        echo ("</br>");   
                    }
                $firsttest = $this->makergaul_validvalue($this->Reflexion);
                /*Reflexionspunkt besser als bester Punkt & im gltigen Bereich?*/
                if((bc_comp($this->Reflexion[4],$this->Rangpunkt[0][4])==-1) && ($firsttest==9))
                    {
                        /*Dann Expansion durchfhren*/
                        $this-> globalchanger_makergaul_expansion($par_expand);
                        /*Expansionspunkt besser als bester Punkt & im gltigen Bereich?*/
                        if((bc_comp($this->Expansion[4],$this->Rangpunkt[0][4])==-1) && ($this->makergaul_validvalue($this->Expansion)==9))
                            {
                                //Dann den schlechtesten Punkt durch Expansionspunkt ersetzen
                                $this->Rangpunkt[4] = $this->Expansion;
                                
                                if($loud > 1)
                                    {
                                        echo ("</br>");
                                        echo (self::LOG_EXPANSION.self::TEXT_DNA.": ".$this->Rangpunkt[4][0]);
                                        echo ("</br>");
                                        echo (self::LOG_EXPANSION.self::TEXT_ENZYME.": ".$this->Rangpunkt[4][1]);
                                        echo ("</br>");
                                        echo (self::LOG_EXPANSION.self::TEXT_INHIB.": ".$this->Rangpunkt[4][2]);
                                        echo ("</br>");
                                        echo (self::LOG_EXPANSION.self::TEXT_BASIS.": ".$this->Rangpunkt[4][3]);
                                        echo ("</br>");
                                        echo (self::LOG_EXPANSION.self::TEXT_SSE.": ".$this->Rangpunkt[4][4]);
                                        echo ("</br>");
                                    }
                                
                                continue;
                            }
                        /*Ansonsten den schlechtesten Punkt durch Reflexionspunkt ersetzen*/ 
                        else 
                            {
                                $this->Rangpunkt[4] = $this->Reflexion;
                                
                                if($loud > 1)
                                    {
                                        echo ("</br>");
                                        echo (self::LOG_REFLEXION.self::TEXT_DNA.": ".$this->Rangpunkt[4][0]);
                                        echo ("</br>");
                                        echo (self::LOG_REFLEXION.self::TEXT_ENZYME.": ".$this->Rangpunkt[4][1]);
                                        echo ("</br>");
                                        echo (self::LOG_REFLEXION.self::TEXT_INHIB.": ".$this->Rangpunkt[4][2]);
                                        echo ("</br>");
                                        echo (self::LOG_REFLEXION.self::TEXT_BASIS.": ".$this->Rangpunkt[4][3]);
                                        echo ("</br>");
                                        echo (self::LOG_REFLEXION.self::TEXT_SSE.": ".$this->Rangpunkt[4][4]);
                                        echo ("</br>");
                                    }
                                    
                                continue;
                            }
                    }
                /*Reflexionspunkt besser als zweitschlechtster Punkt & im gltigen Bereich?*/
                else if((bc_comp($this->Reflexion[4],$this->Rangpunkt[3][4])==-1) && ($firsttest==9))
                    {
                        /*Dann schlechtesten Punkt durch Reflexionspunkt ersetzen*/
                        $this->Rangpunkt[4] = $this->Reflexion;
                        
                        if($loud > 1)
                            {        
                                echo ("</br>");
                                echo (self::LOG_REFLEXION.self::TEXT_DNA.": ".$this->Rangpunkt[4][0]);
                                echo ("</br>");
                                echo (self::LOG_REFLEXION.self::TEXT_ENZYME.": ".$this->Rangpunkt[4][1]);
                                echo ("</br>");
                                echo (self::LOG_REFLEXION.self::TEXT_INHIB.": ".$this->Rangpunkt[4][2]);
                                echo ("</br>");
                                echo (self::LOG_REFLEXION.self::TEXT_BASIS.": ".$this->Rangpunkt[4][3]);
                                echo ("</br>");
                                echo (self::LOG_REFLEXION.self::TEXT_SSE.": ".$this->Rangpunkt[4][4]);
                                echo ("</br>");
                            }
                                
                        continue;
                    }
                /*Ansonsten Kontraktion durchfhren*/
                else 
                    {
                        /*schlechtester Punkt fitter oder gleich Reflexionspunkt?*/
                        if((bc_comp($this->Reflexion[4],$this->Rangpunkt[4][4]) >= 0)) {$this-> globalchanger_makergaul_worse_kontraktion($par_contract);}
                        /*Reflexionspunkt fitter oder gleich schlechtester Punkt ? --> positives Vorzeichen fr Faktor behalten*/
                        else {$this-> globalchanger_makergaul_best_kontraktion($par_contract);} 
                        
                        if($loud > 1)
                            {
                                echo ("</br>");
                                echo (self::LOG_CONTRACTION." ".self::LOG_PROOF_BORDERS);
                                echo ("</br>");
                            }
                            
                        /*Reflexionspunkt im gltigen Bereich? In Variable Speichern (Performance-Erhhung)*/
                        $contracttest = $this->makergaul_validvalue($this->Kontraktion);
                        
                        /*Kontraktionspunkt im gltigen Bereich?*/  
                        if($contracttest == 9) 
                            {
                                /*Kontraktionspunkt besser als schlechtester Punkt?*/ 
                                if(bc_comp($this->Kontraktion[4],$this->Rangpunkt[4][4]) == -1)  
                                    {
                                        //Dann den schlechtesten Punkt durch Kontraktionspunkt ersetzen
                                        $this->Rangpunkt[4] = $this->Kontraktion;
                                        
                                        if($loud > 1)
                                            {
                                                echo ("</br>");
                                                echo (self::LOG_CONTRACTION.self::TEXT_DNA.": ".$this->Rangpunkt[4][0]);
                                                echo ("</br>");
                                                echo (self::LOG_CONTRACTION.self::TEXT_ENZYME.": ".$this->Rangpunkt[4][1]);
                                                echo ("</br>");
                                                echo (self::LOG_CONTRACTION.self::TEXT_INHIB.": ".$this->Rangpunkt[4][2]);
                                                echo ("</br>");
                                                echo (self::LOG_CONTRACTION.self::TEXT_BASIS.": ".$this->Rangpunkt[4][3]);
                                                echo ("</br>");
                                                echo (self::LOG_CONTRACTION.self::TEXT_SSE.": ".$this->Rangpunkt[4][4]);
                                                echo ("</br>");
                                            }
                                            
                                        continue;
                                    }
                                /*Ansonsten auf alle Punkte den Wert des besten Punktes addieren & durch 2 dividieren*/  
                                else 
                                    {
                                        for($i=1;$i<5;$i++)
                                            {
                                                for($y=1;$y<4;$y++)
                                                    {
                                                        //Generalkontraktion nur durchfhren, Wenn Punkte im gltigen Bereich bleiben
                                                        $zwischen = bc_div((bc_add($this->Rangpunkt[$i][$y],$this->Rangpunkt[0][$y])),"2");
                                                        /*if($this->makergaul_validvalue($zwischen)==9)*/$this->Rangpunkt[$i][$y] = $zwischen;
                                                    }
                                                    /*Fitness bestimmen*/
                                                $this->Rangpunkt[$i][4] = $this->makergaul_fitness($this->Rangpunkt[$i][0],$this->Rangpunkt[$i][1],$this->Rangpunkt[$i][2],$this->Rangpunkt[$i][3],$this->IncludedValues[0]);
                                            }
                                            
                                        if($loud > 1)
                                            {
                                                echo ("</br>");
                                                echo (self::LOG_GENERAL_CONTRACTION);
                                                echo ("</br>"); 
                                                echo (self::LOG_PROOF_BORDERS);
                                                echo ("</br>");
                                            }
                                                                      
                                        continue;
                                    }
                        
                            }
                        /*Ansonsten Punkt in den gltigen Bereich verschieben*/ 
                        else
                            {
                                //Start-DNA kleiner als unterer Grenzwert -->Rangpunkt[4] =  Rangpunkt[4] + ( |(Zentrum - Rangpunkt[4])| * 0,5) = Halbe Strecke auf Zentrum zubewegen
                                if($contracttest  == 1) 
                                    {
                                        $this->Rangpunkt[4][0] = bc_rand($this->Spread_dna[0],$this->Zentrum[0]);    
                                    } 
                                //Start-DNA grer als oberer Grenzwert -->Rangpunkt[4] =  Rangpunkt[4] + ( |(Zentrum - Rangpunkt[4])| * -0,5) = Halbe Strecke auf Zentrum zubewegen       
                                else if($contracttest  == 2) 
                                    {
                                        $this->Rangpunkt[4][0] = bc_rand($this->Zentrum[0],$this->Spread_dna[1]);   
                                    }   
                                //Taqmax kleiner als unterer Grenzwert -->Rangpunkt[4] =  Rangpunkt[4] + ( |(Zentrum - Rangpunkt[4])| * 0,5) = Halbe Strecke auf Zentrum zubewegen
                                else if($contracttest  == 3) 
                                    {
                                        $this->Rangpunkt[4][1] = bc_rand($this->Spread_taqmax[0],$this->Zentrum[1]);     
                                    } 
                                //Taqmax grer als oberer Grenzwert -->Rangpunkt[4] =  Rangpunkt[4] + ( |(Zentrum - Rangpunkt[4])| * -0,5) = Halbe Strecke auf Zentrum zubewegen       
                                else if($contracttest  == 4) 
                                    {
                                        $this->Rangpunkt[4][1] = bc_rand($this->Zentrum[1],$this->Spread_taqmax[1]);    
                                    }                      
                                //Inhibitionsfaktor kleiner als unterer Grenzwert -->Rangpunkt[4] =  Rangpunkt[4] + ( |(Zentrum - Rangpunkt[4])| * 0,5) = Halbe Strecke auf Zentrum zubewegen
                                else if($contracttest  == 5) 
                                    {
                                        $this->Rangpunkt[4][2] = bc_rand($this->Spread_inhib[0],$this->Zentrum[2]);     
                                    } 
                                //Inhibitionsfaktor grer als oberer Grenzwert -->Rangpunkt[4] =  Rangpunkt[4] + ( |(Zentrum - Rangpunkt[4])| * -0,5) = Halbe Strecke auf Zentrum zubewegen       
                                else if($contracttest  == 6) 
                                    {
                                        $this->Rangpunkt[4][2] = bc_rand($this->Zentrum[2],$this->Spread_inhib[1]);    
                                    } 
                                //Basisfluoreszens kleiner als unterer Grenzwert -->Rangpunkt[4] =  Rangpunkt[4] + ( |(Zentrum - Rangpunkt[4])| * 0,5) = Halbe Strecke auf Zentrum zubewegen
                                else if($contracttest  == 7) 
                                    {
                                        $this->Rangpunkt[4][3] = bc_rand($this->Spread_basis[0],$this->Zentrum[3]);     
                                    } 
                                //Basisfluoreszens grer als oberer Grenzwert -->Rangpunkt[4] =  Rangpunkt[4] + ( |(Zentrum - Rangpunkt[4])| * -0,5) = Halbe Strecke auf Zentrum zubewegen       
                                else if($contracttest  == 8) 
                                    {
                                        $this->Rangpunkt[4][3] = bc_rand($this->Zentrum[3],$this->Spread_basis[1]);    
                                    }     
                                    
                                //Fitness des in einem Parameter vernderten Punktes bestimmen
                                $this->Rangpunkt[4][4] = $this->makergaul_fitness($this->Rangpunkt[4][0],$this->Rangpunkt[4][1],$this->Rangpunkt[4][2],$this->Rangpunkt[4][3],$this->IncludedValues[0]);
                                
                                if($loud > 1)
                                    {
                                        echo ("</br>");
                                        echo (self::LOG_SHIFT.self::TEXT_DNA.": ".$this->Rangpunkt[4][0]);
                                        echo ("</br>");
                                        echo (self::LOG_SHIFT.self::TEXT_ENZYME.": ".$this->Rangpunkt[4][1]);
                                        echo ("</br>");
                                        echo (self::LOG_SHIFT.self::TEXT_INHIB.": ".$this->Rangpunkt[4][2]);
                                        echo ("</br>");
                                        echo (self::LOG_SHIFT.self::TEXT_BASIS.": ".$this->Rangpunkt[4][3]);
                                        echo ("</br>");
                                        echo (self::LOG_SHIFT.self::TEXT_SSE.": ".$this->Rangpunkt[4][4]);
                                        echo ("</br>");
                                    }
                                    
                                continue;
                            }
                    }
            }
            
            /*Optimierungs-Ergebniss in Array schreiben*/
            //Wenn der letzte Wert besser als das Zwischenergebnis ist: Zwischenergebnis ersetzen
            if(bc_comp($this->Rangpunkt[0][4],$this->Loopwerte[4]) == -1)
                {
                    $this->Loopwerte = $this->Rangpunkt[0];
                }
            if($loud > 1)
                {
                    echo ("</br>");
                    echo (self::LOG_WEIGHT."0 ".self::TEXT_DNA.": ".$this->Rangpunkt[0][0]);
                    echo ("</br>");
                    echo (self::LOG_WEIGHT."0 ".self::TEXT_ENZYME.": ".$this->Rangpunkt[0][1]);
                    echo ("</br>");
                    echo (self::LOG_WEIGHT."0 ".self::TEXT_INHIB.": ".$this->Rangpunkt[0][2]);
                    echo ("</br>");
                    echo (self::LOG_WEIGHT."0 ".self::TEXT_BASIS.": ".$this->Rangpunkt[0][3]);
                    echo ("</br>");
                    echo (self::LOG_WEIGHT."0 ".self::TEXT_SSE.": ".str_replace(".",",",$this->Rangpunkt[0][4]));
                    echo ("</br>");
                    echo (self::LOG_FIRST_RESULT.self::TEXT_SSE.": ".$this->Loopwerte[4]);
                    echo ("</br>");
                }
            //Objekt zerstren, damit die Ausgabe der Werte erfolgt
            $this->__destruct();
        } 
    
    /*Funktion zum Bestimmen des untersuchten Wertebereichs, da MAK 2-Modellierung nur gilt, bevor eine Sttigung der PCR-Polymerase eintritt
    --> Bestimmung der maximalen Steigung & nur diesen Wertebereich einschlieen*/
    function determine_ct($par)/*Funktion zum CT-Wert bestimmen nach MAK 2-Methode*/
        {
            /* Durch das Vorkommen groer Kommazahlen mssen Funktionen die Fluoreszenswerte mit bc()-Funktionen, beliebiger Genauigkeit, rechnen (macht den Code hsslich, lsst sich aber nicht vermeiden)*/
            /*Bis auf 10 Nachkommastellen*/
            bcscale(10);
            /*Umschreiben des Eingabearrays in sicheren Array*/
            $Well_SYBR_Werte = array(count($par));
            $Well_Zyklen = count($par);
            for($i=0; $i < count($par); $i++)
                {
                    $Well_SYBR_Werte[$i] = htmlentities($par[$i], ENT_QUOTES);
                } 
            /*1. Differenzen zwischen einzelnen Messwerten berechnen und in Array einlesen (1.Ableitung = Steigung)
            ( = Delta_SYBR_Werte, eins kleiner als SYBR-Werte, da es eine Differenz weniger als SYBR-Werte gibt :) )*/
            $Delta_SYBR_Werte = array($Well_Zyklen-1);
            for($i=1; $i < $Well_Zyklen; $i++)
                {
                    $Delta_SYBR_Werte[$i-1] = bc_sub($Well_SYBR_Werte[$i],$Well_SYBR_Werte[$i-1]);
                }
            /*2. Differenzen zwischen einzelnen Steigungswerten berechnen und in Array einlesen (2.Ableitung = Hhe der Steigung)
            (Wichtig, da nach der hchsten Steigung die lineare Phase der PCR beginnt --> diese Werte mssen zum Funktionieren des MAKERGAUL-Modells ausgeschlossen werden)*/
            $Delta_Delta_SYBR_Werte = array($Well_Zyklen-2);
            for($i=1; $i < $Well_Zyklen-1; $i++)
                {
                    $Delta_Delta_SYBR_Werte[$i-1] = bc_sub($Delta_SYBR_Werte[$i],$Delta_SYBR_Werte[$i-1]);
                }     
            /*3. erstes Maximum der Steigung berechnen*/ 
            $Cutoff = $Well_Zyklen;
            for($i=$Well_Zyklen-1; $i > 2; $i--) //Vom Ende her analysieren
                {
                    //Drei Differenzen bilden, um Zufallsanstieg zu vermeiden
                    $diff1 = bc_sub($Well_SYBR_Werte[$i],$Well_SYBR_Werte[$i-1]); //Differenz bilden-->negative Differenz = Kurvenabfall am Ende
                    $diff2 = bc_sub($Well_SYBR_Werte[$i-1],$Well_SYBR_Werte[$i-2]); //Differenz bilden-->negative Differenz = Kurvenabfall am Ende
                    $diff3 = bc_sub($Well_SYBR_Werte[$i-2],$Well_SYBR_Werte[$i-3]); //Differenz bilden-->negative Differenz = Kurvenabfall am Ende
                    if(bc_comp($diff1,"0") >= 0 && bc_comp($diff2,"0") >= 0 && bc_comp($diff3,"0") >= 0) //< Wenn aktuelle und die vorausgehenden beiden Steigungen positiv sind, letze Steigung = Cutoff (die nachfolgenden sind zumindest teilweise negativ-->Fluoreszensabfall)
                        {
                             /*+1 da: erstes Array[i] = 0, aber erster Zyklus = 1*/ 
                             $Cutoff=($i+1); 
                             break;
                        }
                }
            /*4. Liste der in die Analyse eingeschlossenen Werte erstellen*/
            $Included_Values = array();
            for($i=0; $i < $Cutoff; $i++)
                {
                    $Included_Values[$i] = $Well_SYBR_Werte[$i];
                }
            /*5. Zustzlich hchsten Fluoreszenswert und Basisfluoreszens bestimmen und zurckgeben (wird spter zur einschrnkung der Optimierungsfunktion bentigt)*/
            $back=array(3);
            $back[0] = $Included_Values;
            $back[1] = bc_max($Well_SYBR_Werte); /*Maximalfluoreszens ALLER Messwerte-> als Einschrnkung fr konstante in mak ntig*/
            $back[2] = bc_abs(bc_mul("1.9",bc_add(bc_abs($Well_SYBR_Werte[0]),0.0001))); /*Basisfluoreszens*/
            /*6. EIGENE GRENZSETZUNG: Wenn der hchste gemessene Fluoreszenswert < 2* die Basisfluoreszensbereiches liegt, werden keine Messwerte eingeschlossen (Keine Trennung der PCR-Fluoreszens vom Rauschen mglich!)*/
            if(bc_comp($back[1],bc_mul("2",$back[2])) <0) {$back[0]=array(0=>FALSE);}
            return($back);
        }
     
     
    function makergaul_fitness($par_start_dna, $par_e_max, $par_inhib, $par_basis, $par_included_values,$output=0,$par_all=0)/*Funktion fhrt Fluoreszens+DNA-Vermehrung nach den aktuellen Parametern des Modells durch und vergleicht diese mit den echten Werten. Resultat ist die Bildung der "sum of squared errors" (SSE).*/
        {
            bcscale(64);
            //Globale Arrays des aktuellen Durchlaufs mit angegebenen Parametern fllen. ACHTUNG: DIE BCMATH-FUNKTIONEN FUNKTIONIEREN NUR MIT STRINGS, DAHER KONVERTIERUNG NTIG!!!!
            $this->PCR_dna[0]= $par_start_dna;
            $this->PCR_taqmax[0]= $par_e_max;
            $this->PCR_inhib[0]= $par_inhib;
            $this->PCR_basis[0]= $par_basis;
            $dna_real=0;
            $e_unusable=0;
            $e_usable = $par_e_max;
            //DNA-Liste:erster Wert ist Ausgangs-DNA
            $this->DNA_List[0] = $par_start_dna;
            
            //MAKERGAUL-Modell zum Ausrechnen der aktuellen DNA
            /*Lnge der Listen der berechneten Fluoreszensen & DNA-Konzentrationen = Lnge der Liste der einbezogenen , als Referenz benutzten echten Fluoreszenswerten*/
            /*ACHTUNG: Fr das Array DNA-Liste ist i = 0 auch Zyklus 0! (Vorher: i = 0 war Zyklus 1)*/
            for($i=1; $i < (count($par_included_values)+1); $i++)
                {
                    $dna_last = $this->DNA_List[$i-1]; 
                    $e_usable = bc_div($e_usable,bc_add(1,bc_mul($par_inhib,$dna_last)));
                    $add_factor = bc_div($e_usable,bc_add($e_usable,$dna_last)); //Benutzbare Enzymmenge
                    $this->DNA_List[$i] = bc_add($dna_last,bc_mul($dna_last,$add_factor));
                    $this->SYBR_List[$i-1] = bc_add($this->DNA_List[$i],$par_basis); 
                }
            /*Bilden der Summe der Fehlerquadrate (SSE) als Ma fr die Abweichung zwischen Modell-Fluoreszenswerten und echten Fluoreszenswerten*/
            $sse =0;
            $mean_model = bcdiv(bc_sum_of_array($par_included_values),count($par_included_values));
            for($i=0; $i < (count($par_included_values)); $i++)
                {
                    $sub1 = bc_sub($par_included_values[$i],$this->SYBR_List[$i]);
                    $sub2 = bc_sub($par_included_values[$i],$mean_model);
                    $se_step = bc_pow(bc_div($sub1,$sub2),2); 
                    /*Entspricht --> $sse_step = ($par_included_values[$i] - $this->SYBR_List[$i]) ^ 2;*/
                    $sse = bc_add($sse,$se_step);
                    /*SE zu aktueller SSE aufsummieren*/
                }
            if($par_all==1){return($this->SYBR_List);}
            return($sse);
        }
 
function makergaul_fitness_old($par_start_dna, $par_e_max, $par_inhib, $par_basis, $par_included_values,$output=0,$par_all=0)/*Funktion fhrt Fluoreszens+DNA-Vermehrung nach den aktuellen Parametern des Modells durch und vergleicht diese mit den echten Werten. Resultat ist die Bildung der "sum of squared errors" (SSE).*/
     {
        bcscale(64);
        //Globale Arrays des aktuellen Durchlaufs mit angegebenen Parametern fllen. ACHTUNG: DIE BCMATH-FUNKTIONEN FUNKTIONIEREN NUR MIT STRINGS, DAHER KONVERTIERUNG NTIG!!!!
        $this->PCR_dna[0]= $par_start_dna;
        $this->PCR_taqmax[0]= $par_e_max;
        $this->PCR_inhib[0]= $par_inhib;
        $this->PCR_basis[0]= $par_basis;
        $dna_real=0;
        $e_usable = $par_e_max; //Startmenge = Alle Enzyme sind nutzbar
        $e_unusable = 0; //Startmenge = Kein Enzym ist gehemmt
   
        //DNA-Liste:erster Wert ist Ausgangs-DNA
        $this->DNA_List[0] = $this->PCR_dna[0];
 for($i=0; $i < (count($par_included_values)); $i++)
            {
                $se_step = bc_pow(bc_sub($par_included_values[$i],$this->SYBR_List[$i]),2); 
                /*Entspricht --> $sse_step = ($par_included_values[$i] - $this->SYBR_List[$i]) ^ 2;*/
                $sse = bc_add($sse,$se_step);
                /*SE zu aktueller SSE aufsummieren*/
            }
        //SYBR-Liste:erster Wert ist Ausgangs-DNA + Basisfluoreszens ACHTUNG: SO NICHT IM PAPER ZU FINDEN, AUSGANGSFLUORESZENS WIRD LEERGELASSEN
        $this->SYBR_List[0] = bc_add($this->PCR_dna[0],$this->PCR_basis[0]);
  
        //MAKERGAUL-Modell zum Ausrechnen der aktuellen DNA
        /*Lnge der Listen der berechneten Fluoreszensen & DNA-Konzentrationen = Lnge der Liste der einbezogenen , als Referenz benutzten echten Fluoreszenswerten*/
        /*ACHTUNG: Fr das Array DNA-Liste ist i = 0 auch Zyklus 0! (Vorher: i = 0 war Zyklus 1)*/
        for($i=1; $i < (count($par_included_values)+1); $i++)
            {
                $dna_last = $this->DNA_List[$i-1]; 
                $e_unusable = bc_add($e_unusable,bc_mul($e_usable,bc_mul($par_inhib,$dna_last)));
                $e_usable = bc_sub($par_e_max,$e_unusable); //Benutzbare Enzymmenge
                if(bc_comp($e_usable,0) == -1) {$e_usable=0;}
                if(bc_comp($e_usable,$dna_last) == 1) {$dna_real=$dna_last;}
                else{$dna_real=$e_usable;}
                
                $this->DNA_List[$i] = bc_add($dna_last,$dna_real);
                $this->SYBR_List[$i-1] = bc_add($this->DNA_List[$i],$par_basis); 
            }
         /*Bilden der Summe der Fehlerquadrate (SSE) als Ma fr die Abweichung zwischen Modell-Fluoreszenswerten und echten Fluoreszenswerten*/
         $sse =0;
         for($i=0; $i < (count($par_included_values)); $i++)
            {
                $se_step = bc_pow(bc_sub($par_included_values[$i],$this->SYBR_List[$i]),2); 
                /*Entspricht --> $sse_step = ($par_included_values[$i] - $this->SYBR_List[$i]) ^ 2;*/
                $sse = bc_add($sse,$se_step);
                /*SE zu aktueller SSE aufsummieren*/
            }
          if($output == 1)
              {
              echo("SYBR:\n");      
              for($i=0; $i < (count($this->SYBR_List)); $i++)
                      {
                          echo(str_replace(".",",",$this->SYBR_List[$i])."\n");
                      }
              echo("DNA:\n");  
              for($i=0; $i < (count($this->DNA_List)); $i++)
                      {
                          echo(str_replace(".",",",$this->DNA_List[$i])."\n");
                      }
              echo("Werte:\n");      
              for($i=0; $i < (count($par_included_values)); $i++)
                      {
                          echo(str_replace(".",",",$par_included_values[$i])."\n");
                      }
              }
          if($par_all==1){return($this->SYBR_List);}
          return($sse);
     } 
         
    function makergaul_validvalue($par_testarray)/*Testet, ob der angegebene Wert innerhalb der gesetzten Grenzen liegt und gibt 9 (=TRUE) oder -1 bis 5 (FALSE) zurck*/
        {
            //Werte vergleichen ($array[0] = unterer Vergleich, $array[1] = oberer Vergleich)
            $x = array(2);
            $y = array(2);
            $z = array(2);
            $i = array(2);
            $ergebnis = 9;
            $x[0] = bc_comp($par_testarray[0],$this->Spread_dna[0]);
            $y[0] = bc_comp($par_testarray[1],$this->Spread_taqmax[0]);
            $z[0] = bc_comp($par_testarray[2],$this->Spread_inhib[0]);
            $i[0] = bc_comp($par_testarray[3],$this->Spread_basis[0]);
            $x[1] = bc_comp($par_testarray[0],$this->Spread_dna[1]);
            $y[1] = bc_comp($par_testarray[1],$this->Spread_taqmax[1]);
            $z[1] = bc_comp($par_testarray[2],$this->Spread_inhib[1]);
            $i[1] = bc_comp($par_testarray[3],$this->Spread_basis[1]);
            //kleiner als unterer Grenzwert oder grer als oberer Grenzwert?-->Wert ungltig, mit Fehlernummer abbrechen
            if($x[0] == -1) {$ergebnis = 1;}             /*DNA-Wertebereich, Fehler = 1 oder 2*/
            else if($x[1] == 1) {$ergebnis = 2;}                        
            else if($y[0] == -1){$ergebnis = 3;}        /*Taqmax-Wertebereich, Fehler = 3 oder 4*/
            else if($y[1] == 1) {$ergebnis = 4;}       
            else if($z[0] == -1){$ergebnis = 5;}        /*Inhib-Wertebereich, Fehler = 5 oder 6*/
            else if($z[1] == 1) {$ergebnis = 6;}   
            else if($i[0] == -1){$ergebnis = 7;}        /*Basis-Wertebereich, Fehler = 7 oder 8*/
            else if($i[1] == 1) {$ergebnis = 8;} 
            //echo ("</br>");   
            //echo("Gltigkeit: ".$ergebnis);   
            //echo ("</br>"); 
            return($ergebnis);
        }  
    
    function globalchanger_makergaul_sorting($par_array)/*Funktion sortiert Punkte des downhill-Simplex in der absteigenden Reihenfolge ihrer Fitness*/
        {  
            /*Zwischenarray um die Fitnessbewertung in Verbindung zur Punktnummer zu speichern*/
            $punktnachfitness = array(array(2),array(2),array(2),array(2),array(2));
            //Punktarray die entsprechende Rangbewertung einspeichern
            for($i=0; $i<5; $i++)
                {
                    //Nach Fitness sortierbares Array mit Punktnummern bilden. Achtung: bei sortierbaren Punktarrays muss der Unterarray [4] den Fitnesswert enthalten!
                    $punktnachfitness[$i][0] = $par_array[$i][4];
                    $punktnachfitness[$i][1] = $i;
                }
            //Neuen array sortieren (kleinste-->grte)
            sort($punktnachfitness);
            for($i=0; $i<5; $i++)
                {
                     /* Ausgangsarray[$i][4] = Rangwert*/
                     $par_array[$punktnachfitness[$i][1]][5] = $i;
                     /*Nach Rangbewertung gebildetes Punktarray erstellen*/
                     $this->Rangpunkt[$i][0] = $par_array[$punktnachfitness[$i][1]][0];
                     $this->Rangpunkt[$i][1] = $par_array[$punktnachfitness[$i][1]][1];
                     $this->Rangpunkt[$i][2] = $par_array[$punktnachfitness[$i][1]][2];
                     $this->Rangpunkt[$i][3] = $par_array[$punktnachfitness[$i][1]][3];
                     $this->Rangpunkt[$i][4] = $par_array[$punktnachfitness[$i][1]][4];
                     /*Rangpunkt[$i][5] = Nummer des Punktes (falls Rangpunkt = Ausgangsarray wird vorher in [$i][5] gesetzter Rangwert mit Punktnummer berschrieben!)*/
                     $this->Rangpunkt[$i][5] = $punktnachfitness[$i][1];
                }
        }
    
    function globalchanger_makergaul_zentrum()/*Funktion bildet Zentrumspunkt der besten Punkte*/
        {
            //Das Zentrum der besten Punkte bestimmen (= Mitelwert aller Punkte auer schlechtestem = Rangpunkt[0][0 bis 2] bis [2][0 bis 2])
            $this->Zentrum[0] = bc_div((bc_add($this->Rangpunkt[0][0],bc_add($this->Rangpunkt[1][0],bc_add($this->Rangpunkt[2][0],$this->Rangpunkt[3][0])))),4);
            $this->Zentrum[1] = bc_div((bc_add($this->Rangpunkt[0][1],bc_add($this->Rangpunkt[1][1],bc_add($this->Rangpunkt[2][1],$this->Rangpunkt[3][1])))),4);
            $this->Zentrum[2] = bc_div((bc_add($this->Rangpunkt[0][2],bc_add($this->Rangpunkt[1][2],bc_add($this->Rangpunkt[2][2],$this->Rangpunkt[3][2])))),4);
            $this->Zentrum[3] = bc_div((bc_add($this->Rangpunkt[0][3],bc_add($this->Rangpunkt[1][3],bc_add($this->Rangpunkt[2][3],$this->Rangpunkt[3][3])))),4);   
        }  
    //Die Verformungsfunktionen reflexion und expansion machen alle beide dasselbe, sind aber aus Performancegrnden  unter anderem Namen statt einmal mit if-else Kontrollfluss vorhanden
    function globalchanger_makergaul_reflexion($par_faktor)/*Funktion bildet Reflexionspunkt des schlechtesten Punktes durch das Zentrum, mit $par_faktor zum genauen Spiegelpunkt*/
        {
            //Den Reflexionspunkt bestimmen (=schlechtester Punkt mit Faktor $par_faktor an Zentrumspunkt gespiegelt)
            $this->Reflexion[0] = bc_add($this->Zentrum[0],bc_mul($par_faktor,(bc_sub($this->Zentrum[0],$this->Rangpunkt[4][0]))));
            //$this->Reflexion[0] = "0.0000000015933623075076592185350763233130727227047385265794122951";
            $this->Reflexion[1] = bc_add($this->Zentrum[1],bc_mul($par_faktor,(bc_sub($this->Zentrum[1],$this->Rangpunkt[4][1]))));
            $this->Reflexion[2] = bc_add($this->Zentrum[2],bc_mul($par_faktor,(bc_sub($this->Zentrum[2],$this->Rangpunkt[4][2]))));
            $this->Reflexion[3] = bc_add($this->Zentrum[3],bc_mul($par_faktor,(bc_sub($this->Zentrum[3],$this->Rangpunkt[4][3]))));
            /*Fitness*/
            $this->Reflexion[4] = $this->makergaul_fitness($this->Reflexion[0],$this->Reflexion[1],$this->Reflexion[2],$this->Reflexion[3],$this->IncludedValues[0]);
        }   
    function globalchanger_makergaul_expansion($par_faktor)/*Funktion bildet Expansionspunkt des schlechtesten Punktes durch das Zentrum, mit $par_faktor zum genauen Spiegelpunkt*/
        {
            //Den Expansionspunkt bestimmen (=schlechtester Punkt mit Faktor $par_faktor an Zentrumspunkt gespiegelt)
            $this->Expansion[0] = bc_add($this->Zentrum[0],bc_mul($par_faktor,(bc_sub($this->Zentrum[0],$this->Rangpunkt[4][0]))));
            //$this->Expansion[0] = "0.0000000015933623075076592185350763233130727227047385265794122951";
            $this->Expansion[1] = bc_add($this->Zentrum[1],bc_mul($par_faktor,(bc_sub($this->Zentrum[1],$this->Rangpunkt[4][1]))));
            $this->Expansion[2] = bc_add($this->Zentrum[2],bc_mul($par_faktor,(bc_sub($this->Zentrum[2],$this->Rangpunkt[4][2]))));
            $this->Expansion[3] = bc_add($this->Zentrum[3],bc_mul($par_faktor,(bc_sub($this->Zentrum[3],$this->Rangpunkt[4][3]))));
            /*Fitness*/
            $this->Expansion[4] = $this->makergaul_fitness($this->Expansion[0],$this->Expansion[1],$this->Expansion[2],$this->Expansion[3],$this->IncludedValues[0]);
        }       
    function globalchanger_makergaul_worse_kontraktion($par_faktor)/*Funktion bildet Kontraktionspunkt des schlechtesten Punktes durch das Zentrum, mit $par_faktor zum genauen Spiegelpunkt*/
        {
            //Den Kontraktionspunkt bestimmen (= Zentrumspunkt + $par_faktor * (schlechtester Punkt - Zentrumspunkt); Kontraktionspunkt liegt vom Zentrum in Richtung schlechtester Punkt)
            $this->Kontraktion[0] = bc_add($this->Zentrum[0],bc_mul($par_faktor,(bc_sub($this->Rangpunkt[4][0],$this->Zentrum[0]))));
            //$this->Kontraktion[0] = "0.0000000015933623075076592185350763233130727227047385265794122951";
            $this->Kontraktion[1] = bc_add($this->Zentrum[1],bc_mul($par_faktor,(bc_sub($this->Rangpunkt[4][1],$this->Zentrum[1]))));
            $this->Kontraktion[2] = bc_add($this->Zentrum[2],bc_mul($par_faktor,(bc_sub($this->Rangpunkt[4][2],$this->Zentrum[2]))));
            $this->Kontraktion[3] = bc_add($this->Zentrum[3],bc_mul($par_faktor,(bc_sub($this->Rangpunkt[4][3],$this->Zentrum[3]))));
            /*Fitness*/
            $this->Kontraktion[4] = $this->makergaul_fitness($this->Kontraktion[0],$this->Kontraktion[1],$this->Kontraktion[2],$this->Kontraktion[3],$this->IncludedValues[0]);
        }   
    function globalchanger_makergaul_best_kontraktion($par_faktor)/*Funktion bildet Kontraktionspunkt des schlechtesten Punktes durch das Zentrum, mit $par_faktor zum genauen Spiegelpunkt*/
        {
            //Den Kontraktionspunkt bestimmen (= Zentrumspunkt - $par_faktor * (schlechtester Punkt - Zentrumspunkt); Kontraktionspunkt liegt vom Zentrum in Richtung Reflexionspunkt)
            $this->Kontraktion[0] = bc_add($this->Zentrum[0],bc_mul("-".$par_faktor,(bc_sub($this->Rangpunkt[4][0],$this->Zentrum[0]))));
            //$this->Kontraktion[0] = "0.0000000015933623075076592185350763233130727227047385265794122951";
            $this->Kontraktion[1] = bc_add($this->Zentrum[1],bc_mul("-".$par_faktor,(bc_sub($this->Rangpunkt[4][1],$this->Zentrum[1]))));
            $this->Kontraktion[2] = bc_add($this->Zentrum[2],bc_mul("-".$par_faktor,(bc_sub($this->Rangpunkt[4][2],$this->Zentrum[2]))));
            $this->Kontraktion[3] = bc_add($this->Zentrum[3],bc_mul("-".$par_faktor,(bc_sub($this->Rangpunkt[4][3],$this->Zentrum[3]))));
            /*Fitness*/
            $this->Kontraktion[4] = $this->makergaul_fitness($this->Kontraktion[0],$this->Kontraktion[1],$this->Kontraktion[2],$this->Kontraktion[3],$this->IncludedValues[0]);
        }      
}   
?>

