<?php
/*
Copyright 2013 Christoph Bultmann

This file is part of the MAKERGAUL-package.

The MAKERGAUL-package is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

The MAKERGAUL-package is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with the MAKERGAUL-package.  If not, see <http://www.gnu.org/licenses/>.
*/

/*Klasse:MAKERGAUL Auswertung*/
class cl_analysis_MAKERGAUL_C
{
    //The menu texts are defined as constants, so that an adjustment in other language is fast possible
    const TEXT_DNA = "DNA";
    const TEXT_SSE = "SSRE"; 
    const TEXT_ENZYME = "Free polymerase";
    const TEXT_INHIB = "Inhibition factor (predetermined)";  
    const TEXT_BASIS = "Baseline fluorescence";  
    const TEXT_NONE = "???";
    const TEXT_LOW = "< Detection limit";
    const TEXT_NA = "n.a.";
    const TEXT_NOT_VALID = "not valid";
    const TEXT_NO_XX = "Modul will not start without a working version of makergaulxx in your binary-folder. Please contact your system administrator.";
    
    //Die Voreinstellungen fr das gewhlte Settings-Men (Hier Standart-Downhill-Simplex-Men)
    const DS_CLASS = "cl_settings_mkg_c";
    const DS_NOCLASS = "No analysis module!";
    const DS_LP = "300";        //Wiederholungen
    const DS_IT = "1500";       //Iterationen
    const DS_PC = "0.01";       //Zielprzision
    const DS_T1 = "10";         //Testwert 1
    const DS_T2 = "40";          //Testwert 2
    const DS_T3 = "0.2";        //Testwert 3
    const DS_RF = "2";          //Reflexion
    const DS_EX = "2";          //Expansion
    const DS_CN = "0.95";       //Kontraktion
    const DS_RB = "3";          //Repeatbarriere (Schutz gegen Iterieren auf immer dasselbe Ergebniss)
    const DS_IB = "0.0485";          //Repeatbarriere (Schutz gegen Iterieren auf immer dasselbe Ergebniss)
    
    /*Globale Arrays: Listen der nach Modell errechneten Fluoreszens und dazu korrespondierenden DNA-Werte */
    private $SYBR_List = array();    /*Fluoreszensliste, nach Modell ermittelt. Parameter des Modells werden so lange optimiert, bis errechnete Fluoreszenswerte mit gemessenen mglichst gut bereinstimmen*/
    private $DNA_List = array();     /*Mit Fluoreszensliste nach Modell korrespondierende DNA-Werte*/ 
    private $Real_DNA_List = array();
    /*Globales Array: Speichert die in determine_ct durchgefhrten Bewertungen der Ausgangsmesswerte. Hier noch nicht als Array definiert, da die Lnge Variieren kann*/
    private $IncludedValues;     /*[0] = Array der eingeschlossenen Fluoreszenswerte, [1] = Maximalfluoreszens ALLER Messwerte, [2] = Basisfuoreszens (basierend auf Well(0)-Fluoreszens)*/ 
    /*Globale Arrays: Punkte des Simplex der Downhill-Simplex-Optimierung*/  
    
    //Variable, die den Namen der gewnschten cl_com-Klasse enthlt (Wird bentigt, um Analysesoftware zu intigrieren, die in C++ geschrieben wurde)
    private $COM_EXEC = "cl_com_ds";
    private $Cxx_Used = FALSE;        //Variable, die speichert, welcher Analysemodus gewhlt wurde
    private $Cxx_Analysis;            //Speicherort fr das cl_com-Objekt
    
    
    //WICHTIG:Wird von jeder Analyseklasse bentigt, um sich beim Hauptprogramm zu registrieren.
    public static function firststart()
        {
            //Prfen, ob gewnschte Settings-Klasse vorhanden ist, und bei Anwesenheit laden
            if(requirement("Settings",self::DS_CLASS) === TRUE)
                {
                    //Static-Aufruf der Settings-Klasse, um Voreinstellungen festzulegen (ACHTUNG: Hierzu ist es ntig zu wissen welche und wie viele Parameter die cl_settings-Funktion verarbeitet sowie deren Reihenfolge! Vorher Klasse anschauen!)
                    $_SESSION["Module"]["Settings"]["Type"]::presets(self::DS_LP,self::DS_IT,self::DS_PC,self::DS_T1,self::DS_T2,self::DS_T3,self::DS_RF,self::DS_EX,self::DS_CN,self::DS_RB,self::DS_IB);
                    return; //Beenden
                }
        }
    //WICHTIG: In jeder Klasse ntig (Wird von cl_modulechange beim wechsel auf ein anderes Modul aufgerufen)
    public static function unload()
        {
            /*Lschen der vom Objekt eingefhrten $SESSION-Variablen*/
            /*Keine Eigenen eingefhrt = Keine zu lschen!*/ 
            return;
        }  
        
    //WICHTIG: Wird von jeder Analyseklasse bentigt. Aufruf erfolgt von cl_plate um den zu erstellenden Ergebnissarray der Platte an die Ausgabe des Analysemoduls anzupassen*/
    public static function result_pars()
        {
            $ergebniss = array(5);  //Wie viele Ergebnissparameter entstehen in jedem Lauf?
            //Achtung: immer min. 2 Parameter notwendig ( [0] und [1] )
            //Achtung: jeden Parameter in die Unterarrays ["ID"] und ["Value"] spalten (wird von cl_plate vorrausgesetzt)
            //Namen und Defaultwerte der Parameter
            $ergebniss[0] = array("ID"=>self::TEXT_DNA,"Value"=>self::TEXT_NONE);
            $ergebniss[1] = array("ID"=>self::TEXT_SSE,"Value"=>self::TEXT_NONE);
            $ergebniss[2] = array("ID"=>self::TEXT_ENZYME,"Value"=>self::TEXT_NONE);
            $ergebniss[3] = array("ID"=>self::TEXT_BASIS,"Value"=>self::TEXT_NONE);
            $ergebniss[4] = array("ID"=>self::TEXT_INHIB,"Value"=>self::TEXT_NONE);
            return($ergebniss);
        }
    //EMPFOHLEN: Die Rckgabe von Argebnissen an das Plattenobjekt sollte per __destruct erfolgen, damit beim Abbrechen der Analyse die bereits erreichten Werte gespeichert werden*/
    public function __destruct()
        {
            //cl_com-Objekt lschen
            if(!empty($this->Cxx_Analysis)){unset($this->Cxx_Analysis);}
        }
    //WICHTIG: Wird von jeder Analyseklasse bentigt. Aufruf erfolgt von cl_settings und enthlt die Parameter zur Analysesteuerung.*/    
    public function run_analysis($par_well, $par_messwerte, $par_reflect,$par_expand,$par_contract, $par_run, $par_loop, $par_precision="0.00009", $par_firststep="0.09", $par_secondstep="0.009", $par_thirdstep="0.0009",$par_change, $par_inhibition = "0.0482", $par_silent=0)
        {
            $this->Session_Well = $par_well; //Wellnummer speichern
            //Den Fixwert des Inhibitionsfaktors eintragen
            $_SESSION["Plate_Data"][$this->Session_Well]["Results"][4]["Value"] = $par_inhibition;
            /*IncludedValues als Array benutzen, da verschiedene Werte zurckgegeben*/
            $this->IncludedValues = $this-> determine_ct($par_messwerte);
            if($this->IncludedValues[0][0] === FALSE) //Werte nicht bestimmbar
                {
                    //Abbruch in Array schreiben  
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][0]["Value"] = self::TEXT_LOW;
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][1]["Value"] = self::TEXT_NA;
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][2]["Value"] = self::TEXT_NA;
                    $_SESSION["Plate_Data"][$this->Session_Well]["Results"][3]["Value"] = self::TEXT_NA;
                    return(1);
                }
            /*Grenzwerte fr Parameter festlegen*/
            $border = array();
            $border[0] = array(0=>"0.00000000000001",1=>"0.1");                                               //Start-DNA
            $border[1] = array(0=>"0.00001",1=>"100");                                                        //freie Polymerase
            $border[2] = array(0=>"-".$this->IncludedValues[2],1=>$this->IncludedValues[2]);                  //Basisfluoreszens
            
            //$par_ihibition in array befrdern, damit er von $this->COM_EXEC verarbeitet werden kann (Sieht umstndlich aus, ist aber der Variabilitt fr mehr als einen Fixparameter geschuldet)
            $array_f_inhib = array(0=>$par_inhibition);
            //Prfen, ob c++ - Implementierung der Analyse vorhanden ist und die Vorprfung der einzubeziehenden Messwerte erfolgreich war
            if(class_exists($this->COM_EXEC) === TRUE && $this->IncludedValues[0][0] !== FALSE)
                {
                    $this->Cxx_Analysis = new $this->COM_EXEC;     //Analyseobjekt anlegen
                    //Analyse starten (Parameter 2 = 1 --> Makergaul-Analyse wird ausgewhlt). Wenn c++- Analyse fehlschlgt...
                    $this->Cxx_Used = TRUE;
                    if($this->Cxx_Analysis->run_analysis($par_well,3,count($border)+1,$border,$array_f_inhib,$this->IncludedValues[0],$par_loop,$par_run,$par_reflect,$par_expand,$par_contract,$par_firststep,$par_secondstep,$par_thirdstep,$par_precision,$par_change, $par_silent) === FALSE)
                        {
                            //...Abbrechen (nur c++-Implementierung!)
                            unset($this->Cxx_Analysis);
			                      echo(self::TEXT_NO_XX);
                            return(1);
                        }
                    unset($this->Cxx_Analysis);
                }
            //Ansonsten abbrechen (nur c++-Implementierung!)
            else
                {
                    echo(self::TEXT_NO_XX);
                    return(1);
                }
        } 
        
    /*Funktion zum Bestimmen des untersuchten Wertebereichs, da MAK 2-Modellierung nur gilt, bevor eine Sttigung der PCR-Polymerase eintritt
    --> Bestimmung der maximalen Steigung & nur diesen Wertebereich einschlieen*/
    function determine_ct($par)/*Funktion zum CT-Wert bestimmen nach MAK 2-Methode*/
        {
            /* Durch das Vorkommen groer Kommazahlen mssen Funktionen die Fluoreszenswerte mit bc()-Funktionen, beliebiger Genauigkeit, rechnen (macht den Code hsslich, lsst sich aber nicht vermeiden)*/
            /*Bis auf 10 Nachkommastellen*/
            bcscale(10);
            /*Umschreiben des Eingabearrays in sicheren Array*/
            $Well_SYBR_Werte = array(count($par));
            $Well_Zyklen = count($par);
            for($i=0; $i < count($par); $i++)
                {
                    $Well_SYBR_Werte[$i] = htmlentities($par[$i], ENT_QUOTES);
                } 
            /*1. Differenzen zwischen einzelnen Messwerten berechnen und in Array einlesen (1.Ableitung = Steigung)
            ( = Delta_SYBR_Werte, eins kleiner als SYBR-Werte, da es eine Differenz weniger als SYBR-Werte gibt :) )*/
            $Delta_SYBR_Werte = array($Well_Zyklen-1);
            for($i=1; $i < $Well_Zyklen; $i++)
                {
                    $Delta_SYBR_Werte[$i-1] = bc_sub($Well_SYBR_Werte[$i],$Well_SYBR_Werte[$i-1]);
                }
            /*2. Differenzen zwischen einzelnen Steigungswerten berechnen und in Array einlesen (2.Ableitung = Hhe der Steigung)
            (Wichtig, da nach der hchsten Steigung die lineare Phase der PCR beginnt --> diese Werte mssen zum Funktionieren des MAKERGAUL-Modells ausgeschlossen werden)*/
            $Delta_Delta_SYBR_Werte = array($Well_Zyklen-2);
            for($i=1; $i < $Well_Zyklen-1; $i++)
                {
                    $Delta_Delta_SYBR_Werte[$i-1] = bc_sub($Delta_SYBR_Werte[$i],$Delta_SYBR_Werte[$i-1]);
                }     
            /*3. erstes Maximum der Steigung berechnen*/ 
            $Cutoff = $Well_Zyklen;
            for($i=$Well_Zyklen-1; $i > 2; $i--) //Vom Ende her analysieren
                {
                    //Drei Differenzen bilden, um Zufallsanstieg zu vermeiden
                    $diff1 = bc_sub($Well_SYBR_Werte[$i],$Well_SYBR_Werte[$i-1]); //Differenz bilden-->negative Differenz = Kurvenabfall am Ende
                    $diff2 = bc_sub($Well_SYBR_Werte[$i-1],$Well_SYBR_Werte[$i-2]); //Differenz bilden-->negative Differenz = Kurvenabfall am Ende
                    $diff3 = bc_sub($Well_SYBR_Werte[$i-2],$Well_SYBR_Werte[$i-3]); //Differenz bilden-->negative Differenz = Kurvenabfall am Ende
                    if(bc_comp($diff1,"0") >= 0 && bc_comp($diff2,"0") >= 0 && bc_comp($diff3,"0") >= 0) //< Wenn aktuelle und die vorausgehenden beiden Steigungen positiv sind, letze Steigung = Cutoff (die nachfolgenden sind zumindest teilweise negativ-->Fluoreszensabfall)
                        {
                             /*+1 da: erstes Array[i] = 0, aber erster Zyklus = 1*/ 
                             $Cutoff=($i+1); 
                             break;
                        }
                }
            /*4. Liste der in die Analyse eingeschlossenen Werte erstellen*/
            $Included_Values = array();
            for($i=0; $i < $Cutoff; $i++)
                {
                    $Included_Values[$i] = $Well_SYBR_Werte[$i];
                }
            /*5. Zustzlich hchsten Fluoreszenswert und Basisfluoreszens bestimmen und zurckgeben (wird spter zur einschrnkung der Optimierungsfunktion bentigt)*/
            $back=array(3);
            $back[0] = $Included_Values;
            $back[1] = bc_max($Well_SYBR_Werte); /*Maximalfluoreszens ALLER Messwerte-> als Einschrnkung fr konstante in mak ntig*/
            $back[2] = bc_abs(bc_mul("5",bc_add(bc_abs($Well_SYBR_Werte[0]),0.0001))); /*Basisfluoreszens*/
            /*6. EIGENE GRENZSETZUNG: Wenn der hchste gemessene Fluoreszenswert < 2* die Basisfluoreszensbereiches liegt, werden keine Messwerte eingeschlossen (Keine Trennung der PCR-Fluoreszens vom Rauschen mglich!)*/
            if(bc_comp($back[1],bc_mul("2",$back[2])) <0) {$back[0]=array(0=>FALSE);}
            return($back);
        }
}
?>

