<?php
/*
Copyright 2013 Christoph Bultmann

This file is part of the MAKERGAUL-package.

The MAKERGAUL-package is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

The MAKERGAUL-package is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with the MAKERGAUL-package.  If not, see <http://www.gnu.org/licenses/>.
*/

/*Klasse:Platte mit Wells & Daten*/
class cl_Plate
{
    //Constant, which is responsible for the display size of the wells (in pixels)
    const INT_WELL_AREA = 105;      /*Size, spacing of wells (the same for x and y, because square)*/
    //The menu texts are defined as constants, so that an adjustment in other language is fast possible
    const TEXT_LOAD_VALUES = "Load values!";
    const TEXT_VALUES = "Fluorescence values:"; 
    const TEXT_NONE = "???";
    const TEXT_BUTTON = "Add values";
    const TEXT_NAME_NONE = "Not known";
    const TEXT_DETECTOR_NONE = "No detector";
    const TEXT_CYCLE = "Cycle";
    //voreingestellte Farben als Konstanten definiert (Hexadezimale Farbcodes)
    const COLOR_FIRST = "#D8D8D8";
    const COLOR_SECOND = "#D3D3FF";
    const COLOR_THIRD = "#F5D0A9";
    const COLOR_FOURTH = "#A9F5BC";
    const COLOR_FIFTH = "#FBFF9A";
    
    //Funktion, die einen Array mit der gewnschten Anzahl Wells pro Platte aufbau
    public static function prepare_plate()
    {
          //Plattengre festlegen
          $_SESSION["Plate_Data"] = array($_SESSION["Plate_Facts"]["Area"]+1);
          //Array fr Checkboxen der Wells anlegen
          $_SESSION["Plate_Checkbox"] = array(1);
          //Plattenfarben auf Anfang setzen
          unset($_SESSION["Plate_Detectors"]);
          $_SESSION["Plate_Detectors"] = array(array("Detector"=>self::TEXT_DETECTOR_NONE,"Color"=>self::COLOR_FIRST));
          //Plattennamen auf Anfang setzen
          unset($_SESSION["Plate_Wellnames"]);
          $_SESSION["Plate_Wellnames"] = array(0=>self::TEXT_NAME_NONE);
          //Struktur des Plattenarrays anlegen
          //Existiert Analyseklasse?
          $exist_class = class_exists($_SESSION["Module"]["Analysis"]["Type"]);
          for($i=1; $i<($_SESSION["Plate_Facts"]["Area"]+1); $i++) //Ebene 1: Wells[0-X]
              {
                  //Array fr Checkboxen der Wells anlegen
                  $_SESSION["Plate_Checkbox"][$i] = -1;
                  //Ebene 2: Results, Measured_Data, "Computed_Data", "Name", "Position"; Ebene 3: "Name"["Well","Detector"], "Position"["X","Y"]
                  $_SESSION["Plate_Data"][$i] = array("Results"=>array(1),"Measured_Data"=>array(),"Computed_Data"=>array(),"Name"=>array("Well"=>self::TEXT_NAME_NONE,"Detector"=>self::TEXT_DETECTOR_NONE),"Position"=>array("X"=>0,"Y"=>0)); 
                  //Ebene 2: "Results"; Ebene 3: "Results"[0-X]; Ebene 4: "Results"[0-X]["ID","Value"];
                  if($exist_class === TRUE) {$_SESSION["Plate_Data"][$i]["Results"] = $_SESSION["Module"]["Analysis"]["Type"]::result_pars();} //Der Ergebnisarray hngt von der Anlyseklasse ab
                  //Wenn diese nicht existiert/fehlerhaft ist, einen Blindeintrag erstellen
                  else
                      {
                          $_SESSION["Plate_Data"][$i]["Results"] = array(0=>array("ID"=>self::TEXT_NONE,"Value"=>self::TEXT_NONE));
                      }
              }
          //Verteilung der Wells auf der Platte berechnen    
          /*Zeilen der Platte berechnen, bei ungradem Ergebnis +1*/
          $lines = bc_div(($_SESSION["Plate_Facts"]["Area"]),$_SESSION["Plate_Facts"]["Rows"],1);
          /*Wenn keine 0 hinter dem Komma, $lines+1 erhhen*/
          if(bc_comp(intval($lines),$lines,1) < 0) {$lines = intval($lines)+1;}
          else {$lines = intval($lines);}
          $m=1;  
          //Zeilenwerte(x und y-Verschiebung in der Anzeige des Wells)
          for($h=0; $h < $lines+1; $h++)
              {
                  /*Verschiebung um x[$k] und y[$h]*/
                  for($k=0; ($k < $_SESSION["Plate_Facts"]["Rows"]) && ($m < $_SESSION["Plate_Facts"]["Area"]+1); $k++)
                      {
                          $_SESSION["Plate_Data"][$m]["Position"]["X"] = (self::INT_WELL_AREA*$k);
                          $_SESSION["Plate_Data"][$m]["Position"]["Y"] = (self::INT_WELL_AREA*$h);
                          $m++;
                      }
              }   
     }   
       
    //Platte anzeigen
    public function show_plate()
    { 
        /* Alle Wells kontrollieren*/
        for($a=1; $a < $_SESSION["Plate_Facts"]["Area"]+1; $a++)
            {
                $this->show_well($a);
            } 
    } 
    
    /*Die mit einer Platte verbundenen Session-Arrays werde zurckgesetzt*/     
    public function reset_data()
     {
          /*----$_SESSION["Analysis"] zurcksetzen-----*/
          unset($_SESSION["Module"]["Analysis"]["Active"]); //Analyseobjekt lschen
          unset($_SESSION["Plate_Data"]);//Alte Plattendaten lschen (Reihenfolge wichtig, da die meisten cl_analysis-Klassen beim Aufruf von __destruct() noch einmal Daten an $_SESSION["Plate_Data"] senden
          //Platte: Farbmen
          unset($_SESSION["Plate_Detectors"]);
          $_SESSION["Plate_Detectors"] = array(array("Detector"=>self::TEXT_DETECTOR_NONE,"Color"=>self::COLOR_FIRST));
          //Plattennamen auf Anfang setzen
          unset($_SESSION["Plate_Wellnames"]);
          $_SESSION["Plate_Wellnames"] = array(0=>self::TEXT_NAME_NONE);
          //Checked Wells zurcksetzen
          $_SESSION["Plate_Checkbox"] = -1;
     } 
     
    /*Die Struktur des Ergebnissparameter-Arrays hat sich verndert (Analysemodulwechsel) --> Diesen Teil neu laden*/     
    public function reload_structure()
     {
          for($i=1; $i<($_SESSION["Plate_Facts"]["Area"]+1); $i++) 
              {
                  unset($_SESSION["Plate_Data"][$i]["Results"]);    //Alte Daten lschen, damit es nicht zu Konflikten kommt 
                  $_SESSION["Plate_Data"][$i]["Results"] = $_SESSION["Module"]["Analysis"]["Type"]::result_pars();

              }
     } 
   
    //Well anzeigen--Erstellt den HTML-Teil
    private function show_well($par_well)
    {
        $par_well = intval($par_well);
        /*Farbe fr Well anhand des Wellnamens bestimmen*/
        $wellcolor = $this->color_well($par_well);
        /*Angepasste Farbe fr die Selektions & Option-Felder (etwas aufhellen)*/
        $changedcolor = color_up_down($wellcolor,1,30);
        //Rahmen des Well (X- und Y-Ausrichtung werden vom Hauptprogramm vorgegeben, die Frbung ber die Funktion color_well im Objekt bestimmt
        echo("<div style=\"background-color:".$wellcolor."; left:".$_SESSION["Plate_Data"][$par_well]["Position"]["X"]."px; top:".$_SESSION["Plate_Data"][$par_well]["Position"]["Y"]."px\"; id=\"well_".$par_well."\" class=\"css_plate_well\" >\n");
        //-->Head-Bereich
        echo("    <div style=\"background-color:".$changedcolor.";\" id=\"well_head_".$par_well."\" class=\"css_plate_well_head\">\n");
        //Well gechecked? --> 1
        if($_SESSION["Plate_Checkbox"][$par_well] == 1)
            {
                echo("        <input type=\"checkbox\" name=\"Plate_Checkbox[".$par_well."]\" class=\"css_plate_well_checkbox\" checked=\"checked\" value=\"1\">\n");
            }
        //Well unchecked? --> 0 (-1 wird kein Check-Feld erstellt, da dann ein leeres Well vorliegt)
        elseif($_SESSION["Plate_Checkbox"][$par_well] == 0)
            {
                echo("        <input type=\"checkbox\" name=\"Plate_Checkbox[".$par_well."]\" class=\"css_plate_well_checkbox\" value=\"1\">\n");
            }
        //Nummer und Name
        echo("        <p id=\"well_number".$par_well."\" class=\"css_plate_well_number\">".$par_well."</p>\n");
        //Liste mit allen Fluoreszenswerten Erstellen (nur lesen, Verndern auf eigener Seite)
        $listlength = count($_SESSION["Plate_Data"][$par_well]["Measured_Data"]);
        echo("            <select style=\"background-color:".$changedcolor.";\" name=\"Well_".$par_well."_Select\" id=\"_".$par_well."\" class=\"css_plate_well_select_1\" size=\"1\">\n");
        //Erster Listeneintrag = Wellname
        echo("                <option value=\"".$_SESSION["Plate_Data"][$par_well]["Name"]["Well"]."\">".$_SESSION["Plate_Data"][$par_well]["Name"]["Well"]."</option>\n");
        //Zweiter Listeneintrag = Detektor
        echo("                <option value=\"".$_SESSION["Plate_Data"][$par_well]["Name"]["Detector"]."\">".$_SESSION["Plate_Data"][$par_well]["Name"]["Detector"]."</option>\n");
        //berhaupt Werte geladen?
        if($listlength < 1)
            {
                echo("                <option disabled=\"disabled\" value=\"".self::TEXT_LOAD_VALUES."\">".self::TEXT_LOAD_VALUES."</option>\n");
            }
        else
            {
                echo("                <option disabled=\"disabled\" value=\"".self::TEXT_VALUES."\">".self::TEXT_VALUES."</option>\n");
                for($i=1; $i<($listlength+1); $i++)
                    {
                        echo("                <option disabled=\"disabled\" value=\"".$_SESSION["Plate_Data"][$par_well]["Measured_Data"][$i-1]."\">".self::TEXT_CYCLE." ".$i.": ".$_SESSION["Plate_Data"][$par_well]["Measured_Data"][$i-1]."</option>\n");
                    }
            }
        //Liste schlieen
        echo("            </select>\n");
        echo("    </div>\n");
        
        //-->Body-Bereich
        //Alle anderen Ergebnisse in Array schreiben(Dient der besseren Lesbarkeit der nachfolgenden Zeilen)
        unset($all_results);
        $all_results = array(1);

        for($i=0;$i<count($_SESSION["Plate_Data"][$par_well]["Results"]);$i++)
            {
                $all_results[$i]=array("ID"=>$_SESSION["Plate_Data"][$par_well]["Results"][$i]["ID"],"Value"=>$_SESSION["Plate_Data"][$par_well]["Results"][$i]["Value"]); //Name und Wert des Ergebnissparameters auslesen
            }
         
        //Position des Scientific-E in den ersten beiden Ergebnissen finden, falls vorhanden
        $dna_E = array("ID"=>self::TEXT_NONE,"Value"=>self::TEXT_NONE);
        $sse_E = array("ID"=>self::TEXT_NONE,"Value"=>self::TEXT_NONE);
        $dna_E["ID"] = $all_results[0]["ID"]; //Fr die Entwicklung: erster Rckgabewert einer cl_analysis-Klasse sollte die DNA sein... 
        $sse_E["ID"] = $all_results[1]["ID"]; //...zweiter Rckgabewert die erreichte Przision (meist SSE)
        $dna_E["Value"] = strpos(floatval($all_results[0]["Value"]),"E")-3;       
        $sse_E["Value"] = strpos(floatval($all_results[1]["Value"]),"E")-3;
        if($dna_E["Value"] < 0) {$dna_E["Value"] = 0;}
        if($sse_E["Value"] < 0) {$sse_E["Value"] = 0;}
        //Ergebnisse in Float-Format formatieren(1.234E10) und Zwischenstck bis E10 ausschneiden, damit sie in die Anzeige passen (mathematisch nicht korrekt, dient nur der Lesbarkeit und wird nicht als Wert gespeichert)
        $dna_out = substr(floatval($all_results[0]["Value"]),2, $dna_E["Value"]);
        $sse_out = substr(floatval($all_results[1]["Value"]),2, $sse_E["Value"]);
        if($all_results[0]["Value"] != self::TEXT_NONE) {$dna_float = str_replace($dna_out,"",floatval($all_results[0]["Value"]));}
        else{$dna_float = self::TEXT_NONE;}
        if($all_results[1]["Value"] != self::TEXT_NONE) {$sse_float = str_replace($sse_out,"",floatval($all_results[1]["Value"]));}
        else{$sse_float = self::TEXT_NONE;}
        //Besondere Krzung fr Float-Zahlen ohne E oder E-
        if($dna_E["Value"] == 0 && is_numeric($all_results[0]["Value"])){$dna_float = substr(floatval($all_results[0]["Value"]),0,7);}
        if($sse_E["Value"] == 0 && is_numeric($all_results[1]["Value"])){$sse_float = substr(floatval($all_results[1]["Value"]),0,8);}

        

        
        echo("    <div class=\"css_plate_well_body\">\n");
        echo("            <select style=\"background-color:".$wellcolor.";\" name=\"Well_".$par_well."_Select_DNA\" class=\"css_plate_well_select_2\" size=\"1\">\n"); 
        echo("                <option value=\"".$par_well."\">\n");//Erster Listeneintrag =  DNA anzeigeoptimiert
        echo("                    ".$all_results[0]["ID"].": ".$dna_float."\n");
        echo("                </option>\n");
        echo("                <option value=\"".$par_well."\">\n");//Erster Listeneintrag =  DNA
        echo("                    ".$all_results[0]["ID"].": ".$all_results[0]["Value"]."\n");
        echo("                </option>\n");
        echo("            </select>\n");//Liste schlieen
        echo("            <select style=\"background-color:".$wellcolor.";\" name=\"Well_".$par_well."_Select_Results\" class=\"css_plate_well_select_3\" size=\"1\">\n"); 
        echo("                <option value=\"".$par_well."\">\n");//Erster Listeneintrag = SSE anzeigenoptimiert
        echo("                    ".$all_results[1]["ID"].": ".$sse_float."\n");
        echo("                </option>\n");  
        echo("                <option value=\"".$par_well."\">\n");//Erster Listeneintrag = SSE
        echo("                    ".$all_results[1]["ID"].": ".$all_results[1]["Value"]."\n");
        echo("                </option>\n"); 
        //Alle brigen Result-Eintrge mittels For-Schleife ergnzen (Diese Parameter knnen je nach cl_analysis variieren)
        for($i=2;$i<count($_SESSION["Plate_Data"][$par_well]["Results"]);$i++)
            {
                echo("                <option value=\"".$par_well."\">\n");
                echo("                    ".$all_results[$i]["ID"].": ".$all_results[$i]["Value"]."\n");
                echo("                </option>\n");
            }
        echo("            </select>\n");//Liste schlieen
        echo("    </div>\n");
        
        //-->Bottom-Bereich
        echo("    <div class=\"css_plate_well_foot\">\n");
        //Formular, fhrt zum manuellen Einlesen von Daten
 

        //Button zum Manuellen Einladen der Werte
        echo("          <input name=\"Plate_Well_Button[".$par_well."]\" class=\"css_plate_well_button\" type=\"submit\" value=\"".self::TEXT_BUTTON."\">\n"); 
        //Formular schlieen
     
        echo("    </div>\n");
        //Rahmen schlieen
        echo("</div>\n");
    }  
    
    //Wells nach Namen unterschiedlich frben
    private function color_well($par_well)
    { 
        //Voreingestellte Farben
        $colors = array(self::COLOR_FIRST,self::COLOR_SECOND,self::COLOR_THIRD,self::COLOR_FOURTH,self::COLOR_FIFTH);
        /* Fr jedes Well die Farb-Well-Liste durchgehen*/
        for($i=0; $i < (count($_SESSION["Plate_Detectors"])+1); $i++)
            {
                //Detektor schon mit Farbe verbunden?-->Einfrben & Durchsuchen abbrechen
                if($i < count($_SESSION["Plate_Detectors"]) && $_SESSION["Plate_Data"][$par_well]["Name"]["Detector"] === $_SESSION["Plate_Detectors"][$i]["Detector"] && $_SESSION["Plate_Detectors"][$i]["Color"] !== NULL) {break;}
                //Detektor vorhanden, aber noch keine Farbe?
                elseif(isset($_SESSION["Plate_Detectors"][$i]) && $_SESSION["Plate_Detectors"][$i]["Color"] === NULL) 
                    {
                        if($i<count($colors)) {$_SESSION["Plate_Detectors"][$i]["Color"] = $colors[$i];}               //Noch Farben aus Liste frei? Eine davon benutzen.
                        else {$_SESSION["Plate_Detectors"][$i]["Color"] = ("#".dechex(mt_rand(100,255)).dechex(mt_rand(100,255)).dechex(mt_rand(100,255)));}   //Ansonsten zufllige Farbe auslosen und fr Detektor speichern.
                        break;
                    }
                //Neuer Detektor?-->Einfrben & in Farbarray speichern
                elseif($i == count($_SESSION["Plate_Detectors"])) 
                    {
                        $_SESSION["Plate_Detectors"][$i]["Detector"] = $_SESSION["Plate_Data"][$par_well]["Name"]["Detector"];   //Detektornamen speichern
                        if($i<count($colors)) {$_SESSION["Plate_Detectors"][$i]["Color"] = $colors[$i];}               //Noch Farben aus Liste frei? Eine davon benutzen.
                        else {$_SESSION["Plate_Detectors"][$i]["Color"] = ("#".dechex(mt_rand(100,255)).dechex(mt_rand(100,255)).dechex(mt_rand(100,255)));}   //Ansonsten zufllige Farbe auslosen und fr Detektor speichern.
                        break;
                    }
            }
        //Gewhlte Farbe zurckgeben
        return($_SESSION["Plate_Detectors"][$i]["Color"]); 
    }      
    
           
}             
?>

