<?php
/*
Copyright 2013 Christoph Bultmann

This file is part of the MAKERGAUL-package.

The MAKERGAUL-package is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

The MAKERGAUL-package is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with the MAKERGAUL-package.  If not, see <http://www.gnu.org/licenses/>.
*/

//Eine Eigene Erweiterung der BC-Funktionen, um:
//1. Formatierungsprobleme von Float-Zahlen der Form "0123.0E-10" abzufangen, da diese nicht von den normalen BC-Funktionen verarbeitet werden
//2. Ntzliche Funktionen zu ergnzen, z.B. Random-Werte mit Kommazahlen (\-?[0-9]{1,}\.?[0-9]{0,})(E\-?[0-9]{1,3})?

/*Parameter zur Nutzung in BC-Funktionen formatieren (keine Angaben wie "E-X" bei kleinen oder "EX" bei groen Zahlen, da sonst die BC-Funktionen abbrechen und 0 ausgeben)*/
function bc_parformat($par,$c = 64)
{
    /*Parameter formatieren*/
    if(is_float($par)||is_numeric($par)) 
         {
              return(number_format(strval($par),$c,'.',''));
         } 
    else{return($par);}
}

/*bc-rand*/
/*Zufallszahlen mit unbegrenzter Nachkommastelle erzeugen*/
/*ACHTUNG: Verfahrensbedingt (mathematische Entkopplung 9-stelliger Zufallsblcke) knnen Zahlen auerhalb des angegebenen Intervalls erzeugt werden. Dieser Fehler ist so selten (<1:100 000 000) und fr die Anwendung nicht relevant, 
sodass er im Hinblick auf die Ausfhrungsgeschwindigkeit der Funktion nicht korrigiert wird. Fr den Downhill-Simplex-Algorythmus ist es vor allem wichtig, dass alle Zufallszahlen im Intervall erreicht werden knnen 
(keine Rundungen/Abschneiden, da "Blinde Bereiche" entstehen, die durch die Simplexverformung nicht erreicht werden knnen). Zahlen ausserhalb des Intervalls werden durch die Grenzwerte automatisch aussortiert.*/
function bc_rand($par_a=0,$par_b=0,$c = 64)
{
    //Liegt Intervall im selben Zahlenbereich? (pos. oder neg.)
    $test = bc_comp(bc_mul($par_a,$par_b,$c+9),0);
    if($test >= 0)
        {
        //echo("Test: ".$test."</br>"); 
            if(bc_comp($par_a,$par_b)>=0)
                {
                    $a = bc_sub($par_a,$par_b); 
                    $b = $par_b; 
                    //echo("Par_a-Par_b,Par_a,Par_b: ".$a." ".$par_a." ".$b."</br>");  
                }  
            else
                {
                    $a = bc_sub($par_b,$par_a); 
                    $b = $par_a;
                    //echo("Par_b-Par_a,Par_b,Par_a: ".$a." ".$par_b." ".$b."</br>");    
                }           
        }
    else{$a= bc_add($par_a,0,9+$c); $b= bc_add($par_b,0,9+$c);}
    //Wichtige Variablen initialisieren 
    $a_point = strpos($a,".");
    $a_second = substr($a,$a_point);
    $a_first = substr($a,0,$a_point);
    $r="";
    $x="";
    $rand_a="";
    $count_a=0;
    $a_after="";
    $a_befor="";

    //Intervall umfasst beide Zahlenbereiche? Auswrfeln, ob B berechnet wird
    if(mt_rand(0,strlen($a)) < mt_rand(0,strlen($b)) && $test ==-1) 
        {
            //Wichtige Variablen initialisieren 
            $a_point = strpos($b,".");
            $a_second = substr($b,$a_point);
            $a_first = substr($b,0,$a_point); 
        }
    //Zerlegung Nachkomma a
    for($i=10;$i<$c+9;$i+=9)
        {
            //Zerlegungsfraktion von Nachkommastellen abhngig machen (normale Fraktion =9-Stellig)
            $count_a=$c-$i;
            if($c-$i >= 0){$count_a=9;}
            //Zerlegungsfraktion vonvorausgegangener Fraktion abhngig machen
            if($rand_a > 0){$rand_a=substr("999999999",0,$count_a);}  //Das ergebnis aus der letzten Zerlegung testen: wenn das schon >0 war, darf das nchste auch > 0 sein!
            else{$rand_a=substr($a_second,$i-9,$count_a);}            //Ansonsten aus Abschnitt-0 Zufallszahl bilden
            $r = mt_rand(0,$rand_a);                  //Zufallszahl des entsprechenden Abschnitts bilden (max. 9 Stellen lang)
            $r = str_repeat("0",9-strlen($r)).$r;     //Mit Nullen fllen
            $a_after = $a_after.$r;                   //Zusammenfhren 
        } 
    if($a_point > 0)
        {
            //negativ?
            if(substr($a_first,0,1) == "-") {$x = "-";}
            //Zerlegung Vorkomma a
            for($i=9;$i<strlen($a_first)+9;$i+=9)
                {
                    $a_befor = $a_befor.mt_rand(0,bc_abs(substr($a_first,$i-9,9),9));  
                } 
            $a_befor = $x.$a_befor.".";              
        }
    else{$a_befor = "";} //Kein Komma? Nichts zum anhngen im String
    $a_complete = $a_befor.$a_after;
    //Intervall? Unter Grenze addieren und Ergebnis ausgeben 
    if($test >= 0)
        {
            $ergebnis = bc_add($a_complete,$b);
            return($ergebnis);
        }
    //Ansonsten das andere Ergebniss ausgeben 
    $ergebnis = $a_befor.$a_after;
    return($ergebnis);
}

/*Funktion zur Arraysummierung mittels BC*/   
function bc_sum_of_array($par,$offset=0,$c = 64)
{
    $result=0;
    /*Array einlesen*/
    for($i=$offset; $i<count($par); $i++)
    {
         /* wenn par[$i] = float oder integer --> erst in String umwandeln*/
         if(is_float($par[$i])) 
         {
              $par[$i] = number_format($par[$i],$c,'.','');
         }
         /* Danach: mit $result vergleichen und ersetzen, wenn grer*/
         $result = $result + $par[$i];
    }
    return($result);  
}  

/*Funktion zur Maximumfindung mittels BC*/   
function bc_max($par,$offset=0,$c = 64,$return_number=0)
{
    /*Default-Result = erster Wert des Arrays*/
    $result = $par[$offset];
    $value = 0;
    /*Array einlesen*/
    for($i=$offset; $i<count($par); $i++)
    {
         /* wenn par[$i] = float oder integer --> erst in String umwandeln*/
         if(is_float($par[$i])) 
         {
              $par[$i] = number_format($par[$i],$c,'.','');
         }
         /* Danach: mit $result vergleichen und ersetzen, wenn grer*/
         if(bccomp($result,$par[$i]) == -1)
         {
              $result = $par[$i];
              $value = $i;
         }
    }
    if($return_number == 1){return($value);}
    return($result);  
}  
/*Funktion konvertiert eingegebene Strings in Float-Datentyp und zurck, damit mit log1p gerechnet werden kann*/  
function bc_ln1p($a,$c = 64) 
{ 
    floatval($a);
    /*Ergebnis in String speichern, der formatiert wird, um ggf. "E-X" verhindern (die bcmath-Funktionen knnen damit nicht rechnen!)*/
    return(number_format(log1p($a),$c,'.','')); 
}

/* Genderte Standart-BC-Funktionen, die vorher eine geeignete Formatierung der Parameter durchfhren*/
/*Funktion zur Betragbildung mittels BC*/   
function bc_abs($par,$c = 64)
{
    return(bcsqrt(bcpow(bc_parformat($par,$c),2)));  
}  
function bc_add($a,$b,$c = 64)
{
    return(bcadd(bc_parformat($a,$c),bc_parformat($b,$c),$c));  
}  
function bc_sub($a,$b,$c = 64)
{
    return(bcsub(bc_parformat($a,$c),bc_parformat($b,$c),$c));  
}  
function bc_mul($a,$b,$c = 64)
{
    return(bcmul(bc_parformat($a,$c),bc_parformat($b,$c),$c));  
}  
function bc_div($a,$b,$c = 64)
{
    return(@bcdiv(bc_parformat($a,$c),bc_parformat($b,$c),$c));  
}  
function bc_pow($a,$b,$c = 64)
{
    return(bcpow(bc_parformat($a,$c),bc_parformat($b,$c),$c));  
}  
function bc_sqrt($a,$c = 64)
{
    return(bcsqrt(bc_parformat($a,$c),$c));  
} 
/*Wird im MAK-Modul nicht bentigt, da alle angepassten BC-Funktionen Strings ausgeben-->im MAK-Modul wird aus Performance-Grnden bccomp() benutzt*/
function bc_comp($a,$b,$c = 64)
{
    return(bccomp(bc_parformat($a,$c),bc_parformat($b,$c),$c));  
}  

/*Funktion zur Aufhellung oder Abdunkelung von Hex-Farben (insgesamt(nur $b angegeben) oder nach Kanlen getrennt($c und $d angegeben))*/ 
function color_up_down($a,$sum_or_sub=1,$b,$c=0,$d=0)
{
    /*HTML-Farbraute abschneiden, um 6-Stelligen Hex-Wert zu erhalten*/
    $color = substr($a,1);
    /*andere Parameter gesetzt?*/
    if($c==0){$c=$b;}
    if($d==0){$d=$b;}
    /*Color in Kanle zerlegen & in Dezimalzahlen zerlegen, um Einzeladditionen/Subtraktion durchzufhren*/
    $color_r = hexdec(substr($color,0,2));
    $color_g = hexdec(substr($color,2,2));
    $color_b = hexdec(substr($color,4,2));
    /*Color_new = Addition/Subtraktion pro Kanal (abhngig davon, ob $sum_or_sub positiv oder negativ ist*/
    $color_r_new = $color_r + (intval($sum_or_sub) * $b);
    $color_g_new = $color_g + (intval($sum_or_sub) * $c);
    $color_b_new = $color_b + (intval($sum_or_sub) * $d);
    /*Richtiges Ergebnis? Sonst begrenzen(>255 und <0 sind unzulssig)*/
    if($color_r_new > 255){$color_r_new = 255;}
    if($color_g_new > 255){$color_g_new = 255;}
    if($color_b_new > 255){$color_b_new = 255;}
    if($color_r_new < 0){$color_r_new = 0;}
    if($color_g_new < 0){$color_g_new = 0;}
    if($color_b_new < 0){$color_b_new = 0;}
    /*In HEX-Werte umwandeln, um 6-Stelligen Hex-Farbwert zu bilden*/
    $color_r_new =dechex($color_r_new);
    $color_g_new =dechex($color_g_new);
    $color_b_new =dechex($color_b_new);
    /*Wenn nicht 2 Stellen vorliegen, eine Stelle mit 0 auffllen (Zahl muss am Ende 6-Stellig sein)*/
    if(strlen($color_r_new)<2){$color_r_new="0".$color_r_new;}
    if(strlen($color_g_new)<2){$color_g_new="0".$color_g_new;}
    if(strlen($color_b_new)<2){$color_b_new="0".$color_b_new;}
    /*Durch String-Verkettung 6-Stellige Hexadezimalzahl mit Farbraute bilden bilden*/
    $color = "#".$color_r_new.$color_g_new.$color_b_new;
    return($color);  
} 

/*Dividiert zwei Zahlen und rundet bei Nachkommastelle IMMER auf*/
function div_int_up($par_a,$par_b)
{
   $up = ceil($par_a/$par_b);
   return($up);
} 


?>

